import {
  PLazy
} from "../../chunks/chunk-IKINPSS5.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/adapter/next/cms.tsx
import { Headers } from "@alinea/iso";
import { Database } from "alinea/backend/Database";
import { createPreviewParser } from "alinea/backend/resolver/ParsePreview";
import { Client } from "alinea/core/Client";
import { CMS } from "alinea/core/CMS";
import { Config } from "alinea/core/Config";
import { outcome } from "alinea/core/Outcome";
import { assign } from "alinea/core/util/Objects";
import { getPreviewPayloadFromCookies } from "alinea/preview/PreviewCookies";
import { devUrl, requestContext } from "./context.js";
import { jsx } from "react/jsx-runtime";
var NextCMS = class extends CMS {
  constructor(config, baseUrl) {
    let lastSync = 0;
    const init = PLazy.from(async () => {
      if (process.env.NEXT_RUNTIME === "edge")
        throw "assert";
      const { generatedStore } = await import("alinea/backend/store/GeneratedStore");
      const store = await generatedStore;
      const db = new Database(config, store);
      const previews = createPreviewParser(db);
      return { db, previews };
    });
    super(config, async () => {
      const context = await requestContext(config);
      const client = new Client({
        url: context.handlerUrl.href,
        applyAuth(init2) {
          const headers = new Headers(init2?.headers);
          headers.set("Authorization", `Bearer ${context.apiKey}`);
          return { ...init2, headers };
        }
      });
      const clientResolve = client.resolve.bind(client);
      return assign(client, {
        async resolve(params) {
          const isDev = Boolean(devUrl());
          let preview;
          const { cookies, draftMode } = await import("next/headers.js");
          const [isDraft] = outcome(() => draftMode().isEnabled);
          if (isDraft) {
            const cookie = cookies();
            const payload = getPreviewPayloadFromCookies(cookie.getAll());
            if (payload)
              preview = { payload };
          }
          if (process.env.NEXT_RUNTIME === "edge" || isDev)
            return clientResolve({ preview, ...params });
          const { PHASE_PRODUCTION_BUILD } = await import("next/constants.js");
          const isBuild = process.env.NEXT_PHASE === PHASE_PRODUCTION_BUILD;
          const { db, previews } = await init;
          const sync = () => db.syncWith(client);
          if (!isBuild) {
            if (preview) {
              preview = await previews.parse(
                preview,
                sync,
                client.getDraft.bind(client)
              );
            } else {
              const syncInterval = params.syncInterval ?? 60;
              const now = Date.now();
              if (now - lastSync >= syncInterval * 1e3) {
                lastSync = now;
                await sync();
              }
            }
          }
          return db.resolver.resolve({ preview, ...params });
        }
      });
    });
    this.baseUrl = baseUrl;
  }
  async user() {
    const { cookies } = await import("next/headers.js");
    const context = await requestContext(this.config);
    const client = new Client({
      url: context.handlerUrl.href,
      applyAuth: (init) => {
        const headers = new Headers(init?.headers);
        const cookie = cookies();
        const alinea = cookie.getAll().filter(({ name }) => name.startsWith("alinea"));
        for (const { name, value } of alinea)
          headers.append("cookie", `${name}=${value}`);
        return { ...init, headers };
      }
    });
    return client.user();
  }
  previews = async ({ widget, workspace, root }) => {
    const { draftMode } = await import("next/headers.js");
    const { default: dynamic } = await import("next/dynamic.js");
    const [isDraft] = outcome(() => draftMode().isEnabled);
    if (!isDraft)
      return null;
    const context = await requestContext(this.config);
    let file = this.config.dashboardFile ?? "/admin.html";
    if (!file.startsWith("/"))
      file = `/${file}`;
    const dashboardUrl = devUrl() ?? new URL(file, context.handlerUrl).href;
    const NextPreviews = dynamic(() => import("./previews.js"), {
      ssr: false
    });
    return /* @__PURE__ */ jsx(
      NextPreviews,
      {
        dashboardUrl,
        widget,
        workspace,
        root
      }
    );
  };
};
function createCMS(config) {
  const baseUrl = process.env.ALINEA_BASE_URL ?? Config.baseUrl(config);
  return new NextCMS(config, baseUrl);
}
export {
  NextCMS,
  createCMS
};
