import "../../chunks/chunk-U5RRZUYZ.js";

// src/adapter/next/handler.ts
import {
  createBackend
} from "alinea/backend/api/CreateBackend";
import { createHandler as createCoreHandler } from "alinea/backend/Handler";
import { JWTPreviews } from "alinea/backend/util/JWTPreviews";
import { cloudBackend } from "alinea/cloud/CloudBackend";
import { Entry } from "alinea/core/Entry";
import { createSelection } from "alinea/core/pages/CreateSelection";
import { getPreviewPayloadFromCookies } from "alinea/preview/PreviewCookies";
import { devUrl, requestContext } from "./context.js";
var handlers = /* @__PURE__ */ new WeakMap();
function createHandler(cms, backend = cloudBackend(cms.config)) {
  if (handlers.has(cms))
    return handlers.get(cms);
  const api = "database" in backend ? createBackend(backend) : backend;
  const handleBackend = createCoreHandler(cms, api);
  const handle = async (request) => {
    try {
      const context = await requestContext(cms.config);
      const previews = new JWTPreviews(context.apiKey);
      const { searchParams } = new URL(request.url);
      const previewToken = searchParams.get("preview");
      if (previewToken) {
        const { draftMode, cookies } = await import("next/headers.js");
        const { searchParams: searchParams2 } = new URL(request.url);
        const previewToken2 = searchParams2.get("preview");
        if (!previewToken2)
          return new Response("Not found", { status: 404 });
        const info = await previews.verify(previewToken2);
        const cookie = cookies();
        const connection = devUrl() ? await cms.connect() : handleBackend.connect(context);
        const payload = getPreviewPayloadFromCookies(cookie.getAll());
        const url = await connection.resolve({
          selection: createSelection(
            Entry({ entryId: info.entryId }).select(Entry.url).first()
          ),
          preview: payload ? { payload } : void 0
        });
        if (!url)
          return new Response("Not found", { status: 404 });
        const source = new URL(request.url);
        if (source.hostname === "0.0.0.0")
          source.hostname = "localhost";
        const location = new URL(url, source.origin);
        draftMode().enable();
        return new Response(`Redirecting...`, {
          status: 302,
          headers: { location: String(location) }
        });
      }
      return handleBackend(request, context);
    } catch (error) {
      console.error(error);
      return new Response("Internal server error", { status: 500 });
    }
  };
  handlers.set(cms, handle);
  return handle;
}
export {
  createHandler
};
