import { Config } from 'alinea/core/Config';
import { SyncResponse, Syncable } from 'alinea/core/Connection';
import { EntryRecord } from 'alinea/core/EntryRecord';
import { Mutation } from 'alinea/core/Mutation';
import { PageSeed } from 'alinea/core/Page';
import { Resolver } from 'alinea/core/Resolver';
import { Sql } from 'rado';
import { EntryRow } from '../core/EntryRow.js';
import { Target } from './Backend.js';
import { Source } from './Source.js';
import { Store } from './Store.js';
interface Seed {
    type: string;
    workspace: string;
    root: string;
    filePath: string;
    page: PageSeed;
}
export declare class Database implements Syncable {
    config: Config;
    store: Store;
    seed: Map<string, Seed>;
    resolver: Resolver;
    constructor(config: Config, store: Store);
    syncRequired(contentHash: string): Promise<boolean>;
    sync(contentHashes: Array<string>): Promise<SyncResponse>;
    contentHashes(): Promise<string[]>;
    syncWith(remote: Syncable, force?: boolean): Promise<Array<string>>;
    applyMutations(mutations: Array<Mutation>, commitHash?: string): Promise<string[]>;
    private applyPublish;
    private updateChildren;
    logEntries(): Promise<void>;
    private applyMutation;
    updateHash(tx: Store, condition: Sql<boolean>): Promise<string[]>;
    meta(): Promise<{
        commitHash: string;
        contentHash: string;
        modifiedAt: number;
    }>;
    static index(tx: Store): Promise<void[]>;
    private writeMeta;
    inited: boolean;
    init(): Promise<void>;
    computeEntry(record: EntryRecord, meta: {
        workspace: string;
        root: string;
        filePath: string;
    }, seed?: Seed): Omit<EntryRow, 'rowHash' | 'fileHash'>;
    seedData(): Map<string, Seed>;
    fill(source: Source, commitHash: string, target?: Target, fix?: boolean): Promise<void>;
    preview<T>(entry: EntryRow, query: (tx: Store) => Promise<T>): Promise<T | undefined>;
}
export {};
