import {
  array,
  enums,
  object,
  string
} from "../chunks/chunk-AOATU4QL.js";
import {
  pLimit
} from "../chunks/chunk-QQTYTFWR.js";
import {
  mergeUpdatesV2
} from "../chunks/chunk-BWVXXM6J.js";
import "../chunks/chunk-FFKU477P.js";
import "../chunks/chunk-IOTY7UTU.js";
import "../chunks/chunk-BRTZLYIM.js";
import {
  PLazy
} from "../chunks/chunk-IKINPSS5.js";
import "../chunks/chunk-U5RRZUYZ.js";

// src/backend/Handler.ts
import { Database } from "alinea/backend/Database";
import { JWTPreviews } from "alinea/backend/util/JWTPreviews";
import { cloudBackend } from "alinea/cloud/CloudBackend";
import { Graph } from "alinea/core/Graph";
import { MutationType } from "alinea/core/Mutation";
import { Realm } from "alinea/core/pages/Realm";
import { Selection } from "alinea/core/pages/ResolveData";
import { decode } from "alinea/core/util/BufferToBase64";
import { base64 } from "alinea/core/util/Encoding";
import { assign } from "alinea/core/util/Objects";
import { HandleAction } from "./HandleAction.js";
import { ChangeSetCreator } from "./data/ChangeSet.js";
import { createPreviewParser } from "./resolver/ParsePreview.js";
import { generatedStore } from "./store/GeneratedStore.js";
var limit = pLimit(1);
var ResolveBody = object({
  selection: Selection.adt,
  realm: enums(Realm).optional,
  locale: string.optional,
  preview: object({ payload: string }).optional
});
var PrepareBody = object({
  filename: string
});
var PreviewBody = object({
  entryId: string
});
var SyncBody = array(string);
function createHandler(cms, backend = cloudBackend(cms.config), database = generatedStore.then((store) => new Database(cms.config, store))) {
  const init = PLazy.from(async () => {
    const db = await database;
    const previews = createPreviewParser(db);
    const resolver = db.resolver;
    const changes = new ChangeSetCreator(
      cms.config,
      new Graph(cms.config, resolver)
    );
    let lastSync = 0;
    return { db, mutate, resolve, periodicSync, syncPending };
    async function resolve(ctx, params) {
      if (!params.preview) {
        await periodicSync(ctx, params.syncInterval);
        return resolver.resolve(params);
      }
      const preview = await previews.parse(
        params.preview,
        () => syncPending(ctx),
        (entryId) => backend.drafts.get(ctx, entryId)
      );
      return resolver.resolve({
        ...params,
        preview
      });
    }
    function periodicSync(ctx, syncInterval = 60) {
      if (syncInterval === Infinity)
        return;
      const now = Date.now();
      if (now - lastSync < syncInterval * 1e3)
        return Promise.resolve();
      lastSync = now;
      return syncPending(ctx).catch(() => void 0);
    }
    function syncPending(ctx) {
      return limit(async () => {
        const meta = await db.meta();
        if (!backend.pending)
          return meta;
        try {
          const toApply = await backend.pending.since(ctx, meta.commitHash);
          const total = toApply?.mutations.length ?? 0;
          console.info(`> sync ${total} pending mutations`);
          if (!toApply)
            return meta;
          await db.applyMutations(toApply.mutations, toApply.toCommitHash);
        } catch (error) {
          console.error(error);
          console.warn("> could not sync pending mutations");
        }
        return db.meta();
      });
    }
    async function mutate(ctx, mutations, retry = false) {
      const changeSet = await changes.create(mutations);
      let fromCommitHash = await db.meta().then((meta) => meta.commitHash);
      try {
        const result = await backend.target.mutate(ctx, {
          commitHash: fromCommitHash,
          mutations: changeSet
        });
        await db.applyMutations(mutations, result.commitHash);
        const tasks = [];
        for (const mutation of mutations) {
          switch (mutation.type) {
            case MutationType.Edit:
              tasks.push(persistEdit(ctx, mutation));
              continue;
          }
        }
        await Promise.all(tasks);
        return { commitHash: result.commitHash };
      } catch (error) {
        if ("expectedCommitHash" in error) {
          if (retry)
            throw error;
          await syncPending(ctx);
          return mutate(ctx, mutations, true);
        }
        throw error;
      }
    }
    async function persistEdit(ctx, mutation) {
      if (!mutation.update)
        return;
      const update = new Uint8Array(await decode(mutation.update));
      const currentDraft = await backend.drafts.get(ctx, mutation.entryId);
      const updatedDraft = currentDraft ? mergeUpdatesV2([currentDraft.draft, update]) : update;
      const draft = {
        entryId: mutation.entryId,
        fileHash: mutation.entry.fileHash,
        draft: updatedDraft
      };
      await backend.drafts.store(ctx, draft);
      const { contentHash } = await db.meta();
      previews.setDraft(mutation.entryId, { contentHash, draft });
    }
  });
  return assign(handle, { connect });
  function connect(context) {
    return {
      async user() {
        return "user" in context ? context.user : void 0;
      },
      async resolve(params) {
        const { resolve } = await init;
        return resolve(context, params);
      },
      async previewToken(request) {
        const previews = new JWTPreviews(context.apiKey);
        return previews.sign(request);
      },
      async prepareUpload(file) {
        return backend.media.prepareUpload(context, file);
      },
      async mutate(mutations) {
        const { mutate } = await init;
        return mutate(context, mutations);
      },
      async syncRequired(contentHash) {
        const { db } = await init;
        return db.syncRequired(contentHash);
      },
      async sync(contentHashes) {
        const { db } = await init;
        return db.sync(contentHashes);
      },
      async revisions(file) {
        return backend.history.list(context, file);
      },
      async revisionData(file, revisionId) {
        return backend.history.revision(
          context,
          file,
          revisionId
        );
      },
      async getDraft(entryId) {
        return backend.drafts.get(context, entryId);
      },
      async storeDraft(draft) {
        return backend.drafts.store(context, draft);
      }
    };
  }
  async function handle(request, context) {
    try {
      const { db, resolve, mutate, periodicSync, syncPending } = await init;
      const previews = new JWTPreviews(context.apiKey);
      const url = new URL(request.url);
      const params = url.searchParams;
      const auth = params.get("auth");
      if (auth)
        return backend.auth.authenticate(context, request);
      const action = params.get("action");
      const expectJson = () => {
        const acceptsJson = request.headers.get("accept")?.includes("application/json");
        if (!acceptsJson)
          throw new Response("Expected JSON", { status: 400 });
      };
      const body = PLazy.from(() => {
        const isJson = request.headers.get("content-type")?.includes("application/json");
        if (!isJson)
          throw new Response("Expected JSON", { status: 400 });
        return request.json();
      });
      const verified = PLazy.from(() => backend.auth.verify(context, request));
      const internal = PLazy.from(async function verifyInternal() {
        try {
          return await verified;
        } catch {
          const authorization = request.headers.get("authorization");
          const bearer = authorization?.slice("Bearer ".length);
          if (!context.apiKey)
            throw new Error("Missing API key");
          if (bearer !== context.apiKey)
            throw new Error("Expected matching api key");
          return context;
        }
      });
      if (action === HandleAction.PreviewToken && request.method === "POST") {
        await verified;
        expectJson();
        return Response.json(await previews.sign(PreviewBody(await body)));
      }
      if (action === HandleAction.User && request.method === "GET") {
        expectJson();
        try {
          const { user } = await backend.auth.verify(context, request);
          return Response.json(user);
        } catch {
          return Response.json(null);
        }
      }
      if (action === HandleAction.Resolve && request.method === "POST") {
        const ctx = await internal;
        expectJson();
        return Response.json(
          await resolve(ctx, ResolveBody(await body)) ?? null
        );
      }
      if (action === HandleAction.Pending && request.method === "GET") {
        const ctx = await internal;
        expectJson();
        const commitHash = string(url.searchParams.get("commitHash"));
        return Response.json(await backend.pending?.since(ctx, commitHash));
      }
      if (action === HandleAction.History && request.method === "GET") {
        const ctx = await verified;
        expectJson();
        const file = string(url.searchParams.get("file"));
        const revisionId = string.nullable(url.searchParams.get("revisionId"));
        const result = await (revisionId ? backend.history.revision(ctx, file, revisionId) : backend.history.list(ctx, file));
        return Response.json(result ?? null);
      }
      if (action === HandleAction.Sync && request.method === "GET") {
        const ctx = await internal;
        expectJson();
        const contentHash = string(url.searchParams.get("contentHash"));
        if ("user" in ctx)
          await periodicSync(ctx);
        else
          await syncPending(context);
        return Response.json(await db.syncRequired(contentHash));
      }
      if (action === HandleAction.Sync && request.method === "POST") {
        const ctx = await internal;
        expectJson();
        if ("user" in ctx)
          await periodicSync(ctx);
        else
          await syncPending(context);
        return Response.json(await db.sync(SyncBody(await body)));
      }
      if (action === HandleAction.Upload) {
        const { media } = backend;
        const entryId = url.searchParams.get("entryId");
        if (!entryId) {
          const ctx = await verified;
          expectJson();
          return Response.json(
            await media.prepareUpload(ctx, PrepareBody(await body).filename)
          );
        }
        const isPost = request.method === "POST";
        if (isPost) {
          if (!media.handleUpload)
            throw new Response("Bad Request", { status: 400 });
          const ctx = await verified;
          await media.handleUpload(ctx, entryId, await request.blob());
          return new Response("OK", { status: 200 });
        }
        if (!media.previewUpload)
          throw new Response("Bad Request", { status: 400 });
        return media.previewUpload(context, entryId);
      }
      if (action === HandleAction.Draft && request.method === "GET") {
        const ctx = await internal;
        expectJson();
        const entryId = string(url.searchParams.get("entryId"));
        const draft = await backend.drafts.get(ctx, entryId);
        return Response.json(
          draft ? { ...draft, draft: base64.stringify(draft.draft) } : null
        );
      }
      if (action === HandleAction.Draft && request.method === "POST") {
        expectJson();
        const data = await body;
        const draft = { ...data, draft: base64.parse(data.draft) };
        return Response.json(await backend.drafts.store(await verified, draft));
      }
      if (action === HandleAction.Mutate && request.method === "POST") {
        expectJson();
        return Response.json(await mutate(await verified, await body));
      }
    } catch (error) {
      if (error instanceof Response)
        return error;
      console.error(error);
      return new Response("Internal Server Error", { status: 500 });
    }
    return new Response("Bad Request", { status: 400 });
  }
}
export {
  createHandler
};
