import {
  AsyncDatabase,
  Dialect,
  Emitter,
  NamedParam,
  SyncDatabase,
  ValueParam,
  connect,
  execTransaction,
  sqliteDialect,
  sqliteDiff,
  txGenerator
} from "../../chunks/chunk-4FCVG65Y.js";
import "../../chunks/chunk-OMOA6SIP.js";
import {
  and,
  asc,
  eq,
  gt,
  isNotNull,
  isNull,
  like,
  not,
  or,
  when
} from "../../chunks/chunk-PE5G5FUO.js";
import {
  column
} from "../../chunks/chunk-GR7UQZEH.js";
import {
  Functions,
  input,
  table
} from "../../chunks/chunk-AOFVF5VC.js";
import {
  getData,
  getTable,
  sql
} from "../../chunks/chunk-NXCD6SNE.js";
import {
  __export
} from "../../chunks/chunk-U5RRZUYZ.js";

// node_modules/rado/dist/driver.js
var driver_exports = {};
__export(driver_exports, {
  "@electric-sql/pglite": () => connect8,
  "@libsql/client": () => connect5,
  "@neondatabase/serverless": () => connect7,
  "@vercel/postgres": () => connect7,
  "better-sqlite3": () => connect2,
  "bun:sqlite": () => connect3,
  d1: () => connect4,
  mysql2: () => connect6,
  pg: () => connect7,
  "sql.js": () => connect
});

// node_modules/rado/dist/universal/functions.js
var insertId = sql.universal({
  sqlite: Functions.last_insert_rowid(),
  postgres: Functions.lastval(),
  mysql: Functions.last_insert_id()
}).mapWith(Number);
function concat(...slices) {
  return sql.universal({
    mysql: Functions.concat(...slices),
    default: sql.join(slices.map(input), sql` || `)
  });
}

// node_modules/rado/dist/driver/better-sqlite3.js
var PreparedStatement = class {
  constructor(stmt, isSelection) {
    this.stmt = stmt;
    this.isSelection = isSelection;
  }
  all(params) {
    return this.stmt.all(...params);
  }
  run(params) {
    return this.stmt.run(...params);
  }
  get(params) {
    return this.stmt.get(...params);
  }
  values(params) {
    if (this.isSelection)
      return this.stmt.raw(true).all(...params);
    this.stmt.run(...params);
    return [];
  }
  free() {
  }
};
var BetterSqlite3Driver = class _BetterSqlite3Driver {
  constructor(client, depth = 0) {
    this.client = client;
    this.depth = depth;
  }
  parsesJson = false;
  supportsTransactions = true;
  exec(query) {
    this.client.exec(query);
  }
  close() {
    this.client.close();
  }
  prepare(sql2, options) {
    return new PreparedStatement(this.client.prepare(sql2), options.isSelection);
  }
  batch(queries) {
    return this.transaction(
      (tx) => queries.map(
        ({ sql: sql2, params, isSelection }) => tx.prepare(sql2, { isSelection }).values(params)
      ),
      {}
    );
  }
  transaction(run, options) {
    return execTransaction(
      this,
      this.depth,
      (depth) => new _BetterSqlite3Driver(this.client, depth),
      run,
      options
    );
  }
};
function connect2(db) {
  return new SyncDatabase(
    new BetterSqlite3Driver(db),
    sqliteDialect,
    sqliteDiff
  );
}

// node_modules/rado/dist/driver/bun-sqlite.js
var PreparedStatement2 = class {
  constructor(stmt) {
    this.stmt = stmt;
  }
  all(params) {
    return this.stmt.all(...params);
  }
  run(params) {
    return this.stmt.run(...params);
  }
  get(params) {
    return this.stmt.get(...params);
  }
  values(params) {
    return this.stmt.values(...params);
  }
  free() {
    this.stmt.finalize();
  }
};
var BunSqliteDriver = class _BunSqliteDriver {
  constructor(client, depth = 0) {
    this.client = client;
    this.depth = depth;
  }
  parsesJson = false;
  supportsTransactions = true;
  exec(query) {
    this.client.exec(query);
  }
  close() {
    this.client.close();
  }
  prepare(sql2) {
    return new PreparedStatement2(this.client.prepare(sql2));
  }
  batch(queries) {
    return this.transaction((tx) => {
      return queries.map(({ sql: sql2, params }) => tx.prepare(sql2).values(params));
    }, {});
  }
  transaction(run, options) {
    return execTransaction(
      this,
      this.depth,
      (depth) => new _BunSqliteDriver(this.client, depth),
      run,
      options
    );
  }
};
function connect3(db) {
  return new SyncDatabase(new BunSqliteDriver(db), sqliteDialect, sqliteDiff);
}

// node_modules/rado/dist/driver/d1.js
var PreparedStatement3 = class {
  constructor(stmt, isSelection) {
    this.stmt = stmt;
    this.isSelection = isSelection;
  }
  async all(params) {
    return this.stmt.bind(...params).all().then(({ results }) => results);
  }
  async run(params) {
    const results = await this.stmt.bind(...params).run();
  }
  async get(params) {
    return this.stmt.bind(...params).first();
  }
  async values(params) {
    if (this.isSelection)
      return this.stmt.bind(...params).raw();
    await this.stmt.bind(...params).run();
    return [];
  }
  free() {
  }
};
var D1Driver = class {
  constructor(client) {
    this.client = client;
  }
  parsesJson = false;
  supportsTransactions = false;
  async exec(query) {
    await this.client.exec(query);
  }
  prepare(sql2, options) {
    return new PreparedStatement3(this.client.prepare(sql2), options.isSelection);
  }
  async close() {
  }
  async batch(queries) {
    const stmts = queries.map(
      ({ sql: sql2, params }) => this.client.prepare(sql2).bind(...params)
    );
    const rows = await this.client.batch(stmts);
    return rows.map((row) => row.results);
  }
  async transaction() {
    throw new Error("Transactions are not supported in D1");
  }
};
function connect4(client) {
  return new AsyncDatabase(new D1Driver(client), sqliteDialect, sqliteDiff);
}

// node_modules/rado/dist/driver/libsql.js
var PreparedStatement4 = class {
  constructor(client, sql2) {
    this.client = client;
    this.sql = sql2;
  }
  async all(params) {
    const result = await this.client.execute({ sql: this.sql, args: params });
    return result.rows;
  }
  async run(params) {
    const result = await this.client.execute({ sql: this.sql, args: params });
  }
  async get(params) {
    return (await this.all(params))[0] ?? null;
  }
  async values(params) {
    const result = await this.client.execute({ sql: this.sql, args: params });
    return result.rows;
  }
  free() {
  }
};
var LibSQLClient = class _LibSQLClient {
  constructor(client, depth = 0) {
    this.client = client;
    this.depth = depth;
  }
  parsesJson = false;
  supportsTransactions = true;
  async exec(query) {
    await this.client.execute(query);
  }
  prepare(sql2, options) {
    return new PreparedStatement4(this.client, sql2);
  }
  async close() {
    if ("close" in this.client)
      return this.client.close();
  }
  async batch(queries) {
    const stmts = queries.map(({ sql: sql2, params }) => {
      return { sql: sql2, args: params };
    });
    const rows = await this.client.batch(stmts);
    return rows.map((row) => row.rows);
  }
  async transaction(run, options) {
    const client = "transaction" in this.client ? await this.client.transaction() : this.client;
    const driver = new _LibSQLClient(client, this.depth + 1);
    if (this.depth > 0)
      await client.execute(`savepoint d${this.depth}`);
    try {
      const result = await run(driver);
      if (this.depth > 0)
        await client.execute(`release savepoint d${this.depth}`);
      else
        await client.commit();
      return result;
    } catch (error) {
      if (this.depth > 0)
        await client.execute(`rollback to savepoint d${this.depth}`);
      else
        await client.rollback();
      throw error;
    } finally {
      if (this.depth === 0)
        client.close();
    }
  }
};
function connect5(client) {
  return new AsyncDatabase(new LibSQLClient(client), sqliteDialect, sqliteDiff);
}

// node_modules/rado/dist/mysql/dialect.js
var BACKTICK = "`";
var ESCAPE_BACKTICK = "``";
var MATCH_BACKTICK = /`/g;
var SINGLE_QUOTE = "'";
var ESCAPE_SINGLE_QUOTE = "''";
var MATCH_SINGLE_QUOTE = /'/g;
var mysqlDialect = new Dialect(
  "mysql",
  class extends Emitter {
    runtime = "mysql";
    paramIndex = 0;
    emitValue(value) {
      this.sql += "?";
      this.params.push(new ValueParam(value));
    }
    emitJsonPath({ target, asSql, segments }) {
      target.emitTo(this);
      this.sql += asSql ? "->>" : "->";
      this.sql += this.quoteString(
        `$${segments.map((p) => typeof p === "number" ? `[${p}]` : `.${p}`).join("")}`
      );
    }
    emitInline(value) {
      if (value === null || value === void 0)
        return this.sql += "null";
      if (typeof value === "number" || typeof value === "boolean")
        return this.sql += value;
      if (typeof value === "string")
        return this.sql += this.quoteString(value);
      this.sql += this.quoteString(JSON.stringify(value));
    }
    emitPlaceholder(name) {
      this.sql += "?";
      this.params.push(new NamedParam(name));
    }
    quoteString(input2) {
      return SINGLE_QUOTE + input2.replace(MATCH_SINGLE_QUOTE, ESCAPE_SINGLE_QUOTE) + SINGLE_QUOTE;
    }
    emitIdentifier(identifier) {
      this.sql += BACKTICK + identifier.replace(MATCH_BACKTICK, ESCAPE_BACKTICK) + BACKTICK;
    }
  }
);

// node_modules/rado/dist/mysql/diff.js
var mysqlDiff = (hasTable) => {
  throw new Error("Diffing for MySQL is not yet implemented");
};

// node_modules/rado/dist/mysql/transactions.js
function startTransaction(options) {
  return [
    "start transaction",
    options.withConsistentSnapshot && "with consistent snapshot",
    options.accessMode
  ].filter(Boolean).join(" ");
}
function setTransaction(options) {
  if (!options.isolationLevel)
    return;
  return [
    "set transaction",
    options.isolationLevel && `isolation level ${options.isolationLevel}`
  ].filter(Boolean).join(" ");
}

// node_modules/rado/dist/driver/mysql2.js
var PreparedStatement5 = class {
  constructor(client, sql2, name) {
    this.client = client;
    this.sql = sql2;
    this.name = name;
  }
  #transformParam = (param) => {
    if (param instanceof Uint8Array)
      return Buffer.from(param);
    return param;
  };
  all(params) {
    return this.client.query(this.sql, params.map(this.#transformParam)).then((res) => res[0]);
  }
  async run(params) {
    await this.client.query(this.sql, params.map(this.#transformParam));
  }
  get(params) {
    return this.all(params).then((rows) => rows[0] ?? null);
  }
  values(params) {
    return this.client.query({
      sql: this.sql,
      values: params.map(this.#transformParam),
      rowsAsArray: true
    }).then((res) => res[0]);
  }
  free() {
  }
};
var Mysql2Driver = class _Mysql2Driver {
  constructor(client, depth = 0) {
    this.client = client;
    this.depth = depth;
  }
  parsesJson = true;
  supportsTransactions = true;
  async exec(query) {
    await this.client.query(query);
  }
  prepare(sql2, options) {
    return new PreparedStatement5(this.client, sql2, options?.name);
  }
  async close() {
    if ("end" in this.client)
      return this.client.end();
  }
  async batch(queries) {
    const transact = async (tx) => {
      const results = [];
      for (const { sql: sql2, params } of queries)
        results.push(await tx.prepare(sql2).values(params));
      return results;
    };
    if (this.depth > 0)
      return transact(this);
    return this.transaction(transact, {});
  }
  async transaction(run, options) {
    const client = isPool(this.client) ? await this.client.getConnection() : this.client;
    const driver = new _Mysql2Driver(client, this.depth + 1);
    try {
      await client.query(
        this.depth > 0 ? `savepoint d${this.depth}` : startTransaction(options)
      );
      if (this.depth === 0) {
        const setOptions = setTransaction(options);
        if (setOptions)
          await client.query(setOptions);
      }
      const result = await run(driver);
      await client.query(
        this.depth > 0 ? `release savepoint d${this.depth}` : "commit"
      );
      return result;
    } catch (error) {
      await client.query(
        this.depth > 0 ? `rollback to savepoint d${this.depth}` : "rollback"
      );
      throw error;
    } finally {
      if (isPool(this.client))
        this.client.releaseConnection(client);
    }
  }
};
function isPool(client) {
  return "getConnection" in client;
}
function connect6(client) {
  return new AsyncDatabase(
    new Mysql2Driver("promise" in client ? client.promise() : client),
    mysqlDialect,
    mysqlDiff
  );
}

// node_modules/rado/dist/postgres/dialect.js
var DOUBLE_QUOTE = '"';
var ESCAPE_DOUBLE_QUOTE = '""';
var MATCH_DOUBLE_QUOTE = /"/g;
var SINGLE_QUOTE2 = "'";
var ESCAPE_SINGLE_QUOTE2 = "''";
var MATCH_SINGLE_QUOTE2 = /'/g;
var postgresDialect = new Dialect(
  "postgres",
  class extends Emitter {
    runtime = "postgres";
    paramIndex = 0;
    emitValue(value) {
      this.sql += `$${++this.paramIndex}`;
      this.params.push(new ValueParam(value));
    }
    emitJsonPath({ target, asSql, segments }) {
      target.emitTo(this);
      for (let i = 0; i < segments.length; i++) {
        const access = segments[i];
        if (i <= segments.length - 2)
          this.sql += "->";
        else if (i === segments.length - 1)
          this.sql += asSql ? "->>" : "->";
        if (typeof access === "number")
          this.sql += access;
        else
          this.sql += this.quoteString(access);
      }
    }
    emitInline(value) {
      if (value === null || value === void 0)
        return this.sql += "null";
      if (typeof value === "number" || typeof value === "boolean")
        return this.sql += value;
      if (typeof value === "string")
        return this.sql += this.quoteString(value);
      this.sql += this.quoteString(JSON.stringify(value));
    }
    emitPlaceholder(name) {
      this.sql += `$${++this.paramIndex}`;
      this.params.push(new NamedParam(name));
    }
    quoteString(input2) {
      return SINGLE_QUOTE2 + input2.replace(MATCH_SINGLE_QUOTE2, ESCAPE_SINGLE_QUOTE2) + SINGLE_QUOTE2;
    }
    emitIdentifier(identifier) {
      this.sql += DOUBLE_QUOTE + identifier.replace(MATCH_DOUBLE_QUOTE, ESCAPE_DOUBLE_QUOTE) + DOUBLE_QUOTE;
    }
  }
);

// node_modules/rado/dist/postgres/columns.js
function integer(name) {
  return column({ name, type: column.integer() });
}
function oid(name) {
  return column({ name, type: column.oid() });
}
function boolean(name) {
  return column({ name, type: column.boolean() });
}
function text(name) {
  return column({ name, type: column.text() });
}

// node_modules/rado/dist/postgres/diff.js
var Attr = table("pg_attribute", {
  name: text("attname").notNull(),
  typId: oid("atttypid").notNull(),
  typMod: integer("atttypmod").notNull(),
  notNull: boolean("attnotnull").notNull(),
  relId: oid("attrelid").notNull(),
  isDropped: boolean("attisdropped").notNull(),
  num: integer("attnum").notNull(),
  identity: text("attidentity"),
  hasDef: boolean("atthasdef").notNull()
});
var Class = table("pg_class", {
  oid: oid().notNull(),
  relname: text().notNull(),
  relnamespace: oid().notNull()
});
var Namespace = table("pg_namespace", {
  oid: oid().notNull(),
  name: text("nspname").notNull()
});
var Type = table("pg_type", {
  oid: oid().notNull(),
  name: text("typname").notNull()
});
var AttrDef = table("pg_attrdef", {
  relId: oid("adrelid").notNull(),
  num: integer("adnum").notNull(),
  bin: text("adbin").notNull()
});
var inline = (sql2) => postgresDialect.inline(sql2);
var columnType = sql`pg_catalog.format_type(${Attr.typId}, ${Attr.typMod})`;
var columnDefaultValue = when(
  [Attr.hasDef, sql`pg_catalog.pg_get_expr(${AttrDef.bin}, ${AttrDef.relId})`],
  null
);
var identity = when(
  [eq(Attr.identity, "a"), sql`'generated always as identity'`],
  [eq(Attr.identity, "d"), sql`'generated by default as identity'`],
  null
);
var postgresDiff = (hasTable) => {
  return txGenerator(function* (tx) {
    const tableApi = getTable(hasTable);
    const def = tx.select({
      table_name: Class.relname,
      name: Attr.name,
      type: columnType,
      notNull: Attr.notNull,
      defaultValue: columnDefaultValue,
      identity,
      schema_name: Namespace.name
    }).from(Class).innerJoin(Namespace, eq(Namespace.oid, Class.relnamespace)).innerJoin(Attr, and(gt(Attr.num, 0), eq(Attr.relId, Class.oid))).innerJoin(Type, eq(Attr.typId, Type.oid)).leftJoin(
      AttrDef,
      and(eq(Attr.relId, AttrDef.relId), eq(Attr.num, AttrDef.num))
    ).where(eq(Class.relname, tableApi.name)).orderBy(asc(Attr.num)).as("def");
    const mapSequenceType = when(
      def.type,
      ["integer", "serial"],
      ["smallint", "smallserial"],
      ["bigint", "bigserial"],
      def.type
    );
    const isSequence = like(def.defaultValue, "nextval(%_seq%");
    const type = concat(
      when([isSequence, mapSequenceType], def.type),
      when(
        [isNotNull(def.identity), concat(" ", def.identity).inlineValues()],
        ""
      )
    ).inlineValues();
    const isNumeric = or(
      eq(def.type, "integer"),
      eq(def.type, "smallint"),
      eq(def.type, "bigint")
    );
    const isNumericSequence = and(isSequence, isNumeric);
    const defaultValue = when(
      [
        and(isNotNull(def.defaultValue), not(isNumericSequence)),
        def.defaultValue
      ],
      ""
    );
    const columnInfo = yield* tx.select({
      name: def.name,
      type,
      notNull: and(def.notNull, isNull(def.identity)),
      defaultValue
    }).from(def);
    const localColumns = new Map(
      columnInfo.map((column2) => {
        return [
          column2.name,
          {
            type: sql.unsafe(column2.type.toLowerCase()),
            notNull: column2.notNull,
            defaultValue: column2.defaultValue ? sql.unsafe(column2.defaultValue) : void 0
          }
        ];
      })
    );
    const schemaColumns = new Map(
      Object.entries(tableApi.columns).map(([name, column2]) => {
        const columnApi = getData(column2);
        return [columnApi.name ?? name, columnApi];
      })
    );
    const stmts = [];
    const columnNames = /* @__PURE__ */ new Set([
      ...localColumns.keys(),
      ...schemaColumns.keys()
    ]);
    for (const columnName of columnNames) {
      const alterTable = sql.identifier(tableApi.name);
      const column2 = sql.identifier(columnName);
      const localInstruction = localColumns.get(columnName);
      const schemaInstruction = schemaColumns.get(columnName);
      if (!schemaInstruction) {
        stmts.push(
          sql.query({
            alterTable,
            dropColumn: column2
          })
        );
      } else if (!localInstruction) {
        stmts.push(
          sql.query({
            alterTable,
            addColumn: [column2, sql.chunk("emitColumn", schemaInstruction)]
          })
        );
      } else {
        if (inline(localInstruction.type) !== inline(schemaInstruction.type)) {
          stmts.push(
            sql.query({
              alterTable,
              alterColumn: [column2, sql`type ${schemaInstruction.type}`]
            })
          );
        }
        if (Boolean(localInstruction.notNull) !== Boolean(schemaInstruction.notNull)) {
          stmts.push(
            sql.query({
              alterTable,
              alterColumn: [
                column2,
                schemaInstruction.notNull ? sql`set not null` : sql`drop not null`
              ]
            })
          );
        }
        const localDefault = localInstruction.defaultValue;
        const schemaDefault = schemaInstruction.defaultValue;
        const localDefaultStr = localDefault && inline(localDefault);
        const schemaDefaultStr = schemaDefault && inline(schemaDefault);
        if (localDefaultStr !== schemaDefaultStr) {
          stmts.push(
            sql.query({
              alterTable,
              alterColumn: [
                column2,
                schemaDefault ? sql`set default ${schemaDefault}` : sql`drop default`
              ]
            })
          );
        }
      }
    }
    return stmts.map(inline);
  });
};

// node_modules/rado/dist/postgres/transactions.js
function setTransaction2(options) {
  return [
    "set transaction",
    options.isolationLevel && `isolation level ${options.isolationLevel}`,
    options.accessMode,
    options.deferrable ? "deferrable" : "not deferrable"
  ].filter(Boolean).join(" ");
}

// node_modules/rado/dist/driver/pg.js
var PreparedStatement6 = class {
  constructor(client, sql2, name) {
    this.client = client;
    this.sql = sql2;
    this.name = name;
  }
  all(params) {
    return this.client.query({
      name: this.name,
      text: this.sql,
      values: params
    }).then((res) => res.rows);
  }
  async run(params) {
    await this.client.query(
      {
        name: this.name,
        text: this.sql,
        values: params
      },
      params
    );
  }
  get(params) {
    return this.all(params).then((rows) => rows[0] ?? null);
  }
  values(params) {
    return this.client.query({
      name: this.name,
      text: this.sql,
      values: params,
      rowMode: "array"
    }).then((res) => res.rows);
  }
  free() {
  }
};
var PgDriver = class _PgDriver {
  constructor(client, depth = 0) {
    this.client = client;
    this.depth = depth;
  }
  parsesJson = true;
  supportsTransactions = true;
  async exec(query) {
    await this.client.query(query);
  }
  prepare(sql2, options) {
    return new PreparedStatement6(this.client, sql2, options?.name);
  }
  async close() {
    if ("end" in this.client)
      return this.client.end();
    if ("release" in this.client)
      return this.client.release();
  }
  async batch(queries) {
    return this.transaction(async (tx) => {
      const results = [];
      for (const { sql: sql2, params } of queries)
        results.push(await tx.prepare(sql2).values(params));
      return results;
    }, {});
  }
  async transaction(run, options) {
    const client = "totalCount" in this.client ? await this.client.connect() : this.client;
    try {
      await client.query(this.depth > 0 ? `savepoint d${this.depth}` : "begin");
      if (this.depth === 0)
        await client.query(setTransaction2(options));
      const result = await run(new _PgDriver(client, this.depth + 1));
      await client.query(
        this.depth > 0 ? `release savepoint d${this.depth}` : "commit"
      );
      return result;
    } catch (error) {
      await client.query(
        this.depth > 0 ? `rollback to savepoint d${this.depth}` : "rollback"
      );
      throw error;
    } finally {
      if ("release" in client)
        client.release();
    }
  }
};
function connect7(client) {
  return new AsyncDatabase(new PgDriver(client), postgresDialect, postgresDiff);
}

// node_modules/rado/dist/driver/pglite.js
var PreparedStatement7 = class {
  constructor(client, sql2) {
    this.client = client;
    this.sql = sql2;
  }
  all(params) {
    return this.client.query(this.sql, params, {
      rowMode: "object"
    }).then((res) => res.rows);
  }
  async run(params) {
    await this.client.query(this.sql, params, {
      rowMode: "array"
    });
  }
  get(params) {
    return this.all(params).then((rows) => rows[0] ?? null);
  }
  values(params) {
    return this.client.query(this.sql, params, {
      rowMode: "array"
    }).then((res) => res.rows);
  }
  free() {
  }
};
var PGliteDriver = class _PGliteDriver {
  constructor(client, depth = 0) {
    this.client = client;
    this.depth = depth;
  }
  parsesJson = true;
  supportsTransactions = true;
  async exec(query) {
    await this.client.exec(query);
  }
  close() {
    if ("close" in this.client) {
      return Promise.resolve();
    }
    throw new Error("Cannot close a transaction");
  }
  prepare(sql2) {
    return new PreparedStatement7(this.client, sql2);
  }
  async batch(queries) {
    const transact = async (tx) => {
      const results = [];
      for (const { sql: sql2, params } of queries)
        results.push(await tx.prepare(sql2).values(params));
      return results;
    };
    if (this.depth > 0)
      return transact(this);
    return this.transaction(transact, {});
  }
  async transaction(run, options) {
    if (this.depth === 0 && "transaction" in this.client)
      return this.client.transaction(async (tx) => {
        await tx.query(setTransaction2(options));
        return run(new _PGliteDriver(tx, this.depth + 1));
      });
    await this.exec(`savepoint d${this.depth}`);
    try {
      const result = await run(new _PGliteDriver(this.client, this.depth + 1));
      await this.exec(`release d${this.depth}`);
      return result;
    } catch (error) {
      await this.exec(`rollback to d${this.depth}`);
      throw error;
    }
  }
};
function connect8(db) {
  return new AsyncDatabase(new PGliteDriver(db), postgresDialect, postgresDiff);
}

// src/backend/api/CreateBackend.ts
import { basicAuth } from "./BasicAuth.js";
import { databaseApi } from "./DatabaseApi.js";
import { githubApi } from "./GithubApi.js";
function createBackend(options) {
  const ghApi = githubApi(options.github);
  const db = driver_exports[options.database.driver](options.database.client);
  const dbApi = databaseApi({ ...options, db, target: ghApi.target });
  const auth = basicAuth(options.auth);
  return {
    ...ghApi,
    ...dbApi,
    auth
  };
}
export {
  createBackend
};
