import {
  blob,
  id,
  jsonb,
  text
} from "../../chunks/chunk-3RHKMXNG.js";
import {
  asc,
  eq,
  gt
} from "../../chunks/chunk-PE5G5FUO.js";
import "../../chunks/chunk-GR7UQZEH.js";
import {
  table
} from "../../chunks/chunk-AOFVF5VC.js";
import "../../chunks/chunk-NXCD6SNE.js";
import {
  PLazy
} from "../../chunks/chunk-IKINPSS5.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/backend/api/DatabaseApi.ts
import { createId } from "alinea/core/Id";
import { basename, extname } from "alinea/core/util/Paths";
import { slugify } from "alinea/core/util/Slugs";
import { HandleAction } from "../HandleAction.js";
var Draft = table("alinea_draft", {
  entryId: text().primaryKey(),
  fileHash: text().notNull(),
  draft: blob().notNull()
});
var Mutation = table("alinea_mutation", {
  id: id(),
  commitHash: text().unique().notNull(),
  mutations: jsonb().notNull()
});
var Upload = table("alinea_upload", {
  entryId: text().primaryKey(),
  content: blob().notNull()
});
function databaseApi(options) {
  const setup = PLazy.from(async () => {
    const { db } = options;
    await db.create(Draft, Mutation, Upload).catch(() => {
    });
    return options.db;
  });
  const drafts = {
    async get(ctx, entryId) {
      const db = await setup;
      const found = await db.select().from(Draft).where(eq(Draft.entryId, entryId)).get();
      return found ?? void 0;
    },
    async store(ctx, draft) {
      const db = await setup;
      const query = db.dialect.runtime === "mysql" ? db.insert(Draft).values(draft).onDuplicateKeyUpdate({
        set: draft
      }) : db.insert(Draft).values(draft).onConflictDoUpdate({
        target: Draft.entryId,
        set: draft
      });
      await query;
    }
  };
  const target = {
    async mutate(ctx, params) {
      const db = await setup;
      const { commitHash } = await options.target.mutate(ctx, params);
      const mutations = params.mutations.map((m) => m.meta);
      await db.insert(Mutation).values({ commitHash, mutations });
      return { commitHash };
    }
  };
  const pending = {
    async since(ctx, commitHash) {
      const db = await setup;
      const index = await db.select(Mutation.id).from(Mutation).where(eq(Mutation.commitHash, commitHash)).get();
      const rows = await db.select().from(Mutation).where(gt(Mutation.id, index ?? 0)).orderBy(asc(Mutation.id));
      if (rows.length === 0)
        return void 0;
      const mutations = rows.flatMap((row) => row.mutations);
      return { toCommitHash: rows[rows.length - 1].commitHash, mutations };
    }
  };
  const media = {
    async prepareUpload(ctx, file) {
      const entryId = createId();
      const extension = extname(file);
      const base = basename(file, extension);
      const filename = [slugify(base), entryId, slugify(extension)].join(".");
      const url = new URL(
        `?${new URLSearchParams({
          action: HandleAction.Upload,
          entryId
        })}`,
        ctx.handlerUrl
      );
      return {
        entryId,
        location: filename,
        previewUrl: url.href,
        url: url.href
      };
    },
    async handleUpload(ctx, entryId, file) {
      const db = await setup;
      const content = new Uint8Array(await file.arrayBuffer());
      await db.insert(Upload).values({
        entryId,
        content
      });
    },
    async previewUpload(ctx, entryId) {
      const db = await setup;
      const upload = await db.select().from(Upload).where(eq(Upload.entryId, entryId)).get();
      if (!upload)
        return new Response("Not found", { status: 404 });
      return new Response(upload.content, {
        headers: {
          "content-type": "application/octet-stream",
          "content-disposition": `inline; filename="${entryId}"`
        }
      });
    }
  };
  return { drafts, target, pending, media };
}
export {
  databaseApi
};
