import { EntryRow } from 'alinea/core/EntryRow';
import type * as pages from 'alinea/core/pages/index';
import { Realm } from 'alinea/core/pages/Realm';
import { ResolveRequest } from 'alinea/core/Resolver';
import { Schema } from 'alinea/core/Schema';
import { Query, Select, SelectionInput, Sql } from 'rado';
import { HasSql } from 'rado/core/Internal';
import { Either } from 'rado/core/MetaData';
import type { Database } from '../Database.js';
import { LinkResolver } from './LinkResolver.js';
import { ResolveContext } from './ResolveContext.js';
type Interim = any;
export interface PostContext {
    linkResolver: LinkResolver;
}
export declare class EntryResolver {
    db: Database;
    schema: Schema;
    targets: Schema.Targets;
    constructor(db: Database, schema: Schema);
    fieldOf(ctx: ResolveContext, target: pages.TargetData, field: string): HasSql;
    pageFields(ctx: ResolveContext): Array<[string, HasSql]>;
    selectFieldsOf(ctx: ResolveContext, target: pages.TargetData): Array<[string, HasSql]>;
    exprUnOp(ctx: ResolveContext, { op, expr }: pages.ExprData.UnOp): HasSql;
    exprBinOp(ctx: ResolveContext, { op, a, b }: pages.ExprData.BinOp): HasSql;
    exprField(ctx: ResolveContext, { target, field }: pages.ExprData.Field): HasSql;
    exprAccess(ctx: ResolveContext, { expr, field }: pages.ExprData.Access): HasSql;
    exprValue(ctx: ResolveContext, { value }: pages.ExprData.Value): HasSql;
    exprRecord(ctx: ResolveContext, { fields }: pages.ExprData.Record): Record<string, HasSql>;
    exprCall(ctx: ResolveContext, { method, args }: pages.ExprData.Call): HasSql;
    expr(ctx: ResolveContext, expr: pages.ExprData): HasSql<any>;
    selectRecord(ctx: ResolveContext, { fields }: pages.Selection.Record): {
        [k: string]: SelectionInput;
    };
    selectRow(ctx: ResolveContext, { target }: pages.Selection.Row): SelectionInput;
    selectCursor(ctx: ResolveContext, selection: pages.Selection.Cursor): SelectionInput;
    selectExpr(ctx: ResolveContext, { expr, fromParent }: pages.Selection.Expr): SelectionInput;
    selectCount(ctx: ResolveContext, hasSearch: boolean): SelectionInput;
    selectAll(ctx: ResolveContext, target: pages.TargetData): SelectionInput;
    select(ctx: ResolveContext, selection: pages.Selection, hasSearch?: boolean): SelectionInput;
    queryRecord(ctx: ResolveContext, selection: pages.Selection.Record): Query<any, Either>;
    querySource(ctx: ResolveContext, source: pages.CursorSource | undefined, hasSearch: boolean): Select<any>;
    orderBy(ctx: ResolveContext, orderBy: Array<pages.OrderBy>): Array<Sql>;
    conditionLocale(Table: typeof EntryRow, locale?: string): Sql<boolean>;
    conditionRealm(Table: typeof EntryRow, realm: Realm): import("rado/core/expr/Field.js").Field<boolean, "Entry"> | Sql<boolean>;
    conditionLocation(Table: typeof EntryRow, location: Array<string>): Sql<boolean>;
    conditionSearch(Table: typeof EntryRow, searchTerms?: Array<string>): Sql<boolean>;
    queryCursor(ctx: ResolveContext, { cursor }: pages.Selection.Cursor): Select<any>;
    query<Result>(ctx: ResolveContext, selection: pages.Selection): Query<Result, Either>;
    isSingleResult(ctx: ResolveContext, selection: pages.Selection): boolean;
    postRow(ctx: PostContext, interim: Interim, { target }: pages.Selection.Row): Promise<void>;
    postCursor(ctx: PostContext, interim: Interim, { cursor }: pages.Selection.Cursor): Promise<void>;
    postField(ctx: PostContext, interim: Interim, { target, field }: pages.ExprData.Field): Promise<void>;
    postExpr(ctx: PostContext, interim: Interim, { expr }: pages.Selection.Expr): Promise<void>;
    postFieldsOf(ctx: PostContext, interim: Interim, target: pages.TargetData): Promise<void>;
    postRecord(ctx: PostContext, interim: Interim, { fields }: pages.Selection.Record): Promise<void>;
    post(ctx: PostContext, interim: Interim, selection: pages.Selection): Promise<void>;
    resolve: <T>({ selection, location, locale, realm, preview }: ResolveRequest) => Promise<T>;
}
export {};
