import {
  count
} from "../../chunks/chunk-2G3ZDCXF.js";
import {
  Builder,
  Select,
  selection
} from "../../chunks/chunk-OMOA6SIP.js";
import {
  and,
  asc,
  desc,
  eq,
  gt,
  inArray,
  like,
  lt,
  ne
} from "../../chunks/chunk-PE5G5FUO.js";
import {
  Functions,
  alias,
  input
} from "../../chunks/chunk-AOFVF5VC.js";
import {
  getData,
  getSql,
  getTable,
  internalData,
  internalSql,
  sql
} from "../../chunks/chunk-NXCD6SNE.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/backend/resolver/EntryResolver.ts
import { EntryPhase, EntryRow } from "alinea/core/EntryRow";
import { EntrySearch } from "alinea/core/EntrySearch";
import { Field } from "alinea/core/Field";
import { Realm } from "alinea/core/pages/Realm";
import { BinaryOp, SourceType, UnaryOp } from "alinea/core/pages/ResolveData";
import { Schema } from "alinea/core/Schema";
import { Type } from "alinea/core/Type";
import { entries, fromEntries, keys } from "alinea/core/util/Objects";
import { unreachable } from "alinea/core/util/Types";

// node_modules/rado/dist/core/expr/Include.js
var Include = class {
  [internalData];
  constructor(data) {
    this[internalData] = data;
  }
  #mapFromDriverValue = (value, specs) => {
    const { select, first } = getData(this);
    const parsed = specs.parsesJson ? value : JSON.parse(value);
    if (first) {
      const result = parsed ? select.mapRow({ values: parsed, index: 0, specs }) : null;
      return result;
    }
    if (!parsed)
      return [];
    const rows = parsed;
    const ctx = {
      values: void 0,
      index: 0,
      specs
    };
    for (let i = 0; i < rows.length; i++) {
      ctx.values = rows[i];
      ctx.index = 0;
      rows[i] = select.mapRow(ctx);
    }
    return rows ?? [];
  };
  get [internalSql]() {
    return sql.chunk("emitInclude", getData(this)).mapWith({
      mapFromDriverValue: this.#mapFromDriverValue
    });
  }
};
function include(select) {
  return new Include({ ...getData(select), first: false });
}
((include2) => {
  function one(select) {
    return new Include({ ...getData(select), first: true });
  }
  include2.one = one;
})(include || (include = {}));

// node_modules/rado/dist/sqlite/functions.js
var count2 = Functions.count;
var iif = Functions.iif;
var bm25 = Functions.bm25;
var highlight = Functions.highlight;
var snippet = Functions.snippet;
var abs = Functions.abs;
var changes = Functions.changes;
var char = Functions.char;
var coalesce = Functions.coalesce;
var hex = Functions.hex;
var ifnull = Functions.ifnull;
var instr = Functions.instr;
var last_insert_rowid = Functions.last_insert_rowid;
var length = Functions.length;
var likelihood = Functions.likelihood;
var likely = Functions.likely;
var lower = Functions.lower;
var ltrim = Functions.ltrim;
var max = Functions.max;
var min = Functions.min;
var nullif = Functions.nullif;
var prnumberf = Functions.prnumberf;
var quote = Functions.quote;
var random = Functions.random;
var randomblob = Functions.randomblob;
var replace = Functions.replace;
var round = Functions.round;
var rtrim = Functions.rtrim;
var sign = Functions.sign;
var soundex = Functions.soundex;
var sqlite_version = Functions.sqlite_version;
var substr = Functions.substr;
var total_changes = Functions.total_changes;
var trim = Functions.trim;
var sqliteTypeof = Functions.typeof;
var unicode = Functions.unicode;
var unlikely = Functions.unlikely;
var upper = Functions.upper;
var avg = Functions.avg;
var group_concat = Functions.group_concat;
var sum = Functions.sum;
var acos = Functions.acos;
var acosh = Functions.acosh;
var asin = Functions.asin;
var asinh = Functions.asinh;
var atan = Functions.atan;
var atan2 = Functions.atan2;
var atanh = Functions.atanh;
var ceil = Functions.ceil;
var cos = Functions.cos;
var cosh = Functions.cosh;
var degrees = Functions.degrees;
var exp = Functions.exp;
var floor = Functions.floor;
var ln = Functions.ln;
var log = Functions.log;
var log2 = Functions.log2;
var mod = Functions.mod;
var pi = Functions.pi;
var pow = Functions.pow;
var radians = Functions.radians;
var sin = Functions.sin;
var sinh = Functions.sinh;
var sqrt = Functions.sqrt;
var tan = Functions.tan;
var tanh = Functions.tanh;
var trunc = Functions.trunc;
var date = Functions.date;
var time = Functions.time;
var datetime = Functions.datetime;
var julianday = Functions.julianday;
var strftime = Functions.strftime;

// src/backend/resolver/EntryResolver.ts
import { LinkResolver } from "./LinkResolver.js";
import { ResolveContext } from "./ResolveContext.js";
var builder = new Builder();
var unOps = {
  [UnaryOp.Not]: sql`not`,
  [UnaryOp.IsNull]: sql`is null`
};
var binOps = {
  [BinaryOp.Add]: sql`+`,
  [BinaryOp.Subt]: sql`-`,
  [BinaryOp.Mult]: sql`*`,
  [BinaryOp.Mod]: sql`%`,
  [BinaryOp.Div]: sql`/`,
  [BinaryOp.Greater]: sql`>`,
  [BinaryOp.GreaterOrEqual]: sql`>=`,
  [BinaryOp.Less]: sql`<`,
  [BinaryOp.LessOrEqual]: sql`<=`,
  [BinaryOp.Equals]: sql`=`,
  [BinaryOp.NotEquals]: sql`!=`,
  [BinaryOp.And]: sql`and`,
  [BinaryOp.Or]: sql`or`,
  [BinaryOp.Like]: sql`like`,
  [BinaryOp.In]: sql`in`,
  [BinaryOp.NotIn]: sql`not in`,
  [BinaryOp.Concat]: sql`||`
};
var MAX_DEPTH = 999;
var pageFields = keys(EntryRow);
var EntryResolver = class {
  constructor(db, schema) {
    this.db = db;
    this.schema = schema;
    this.targets = Schema.targets(schema);
  }
  targets;
  fieldOf(ctx, target, field) {
    const { name } = target;
    if (!name) {
      const fields = ctx.Table;
      if (field in fields)
        return fields[field];
      throw new Error(`Selecting unknown field: "${field}"`);
    }
    const type = this.schema[name];
    if (!type)
      throw new Error(`Selecting "${field}" from unknown type: "${name}"`);
    return ctx.Table.data[field];
  }
  pageFields(ctx) {
    return pageFields.map((key) => [key, this.fieldOf(ctx, {}, key)]);
  }
  selectFieldsOf(ctx, target) {
    const { name } = target;
    if (!name)
      return this.pageFields(ctx);
    const type = this.schema[name];
    if (!type)
      throw new Error(`Selecting from unknown type: "${name}"`);
    return keys(type).map((key) => {
      return [key, this.fieldOf(ctx, target, key)];
    });
  }
  exprUnOp(ctx, { op, expr }) {
    switch (op) {
      case UnaryOp.IsNull:
        return sql`${this.expr(ctx, expr)} is null`;
      default:
        return sql`${unOps[op]} ${this.expr(ctx, expr)}`;
    }
  }
  exprBinOp(ctx, { op, a, b }) {
    switch (op) {
      case BinaryOp.In:
        return inArray(this.expr(ctx, a), this.expr(ctx, b));
      case BinaryOp.NotIn:
        return inArray(this.expr(ctx, a), this.expr(ctx, b));
      default:
        return sql`(${this.expr(ctx, a)} ${binOps[op]} ${this.expr(ctx, b)})`;
    }
  }
  exprField(ctx, { target, field }) {
    return this.fieldOf(ctx, target, field);
  }
  exprAccess(ctx, { expr, field }) {
    return sql.jsonPath({
      target: getSql(this.expr(ctx.access, expr)),
      asSql: true,
      segments: [field]
    });
  }
  exprValue(ctx, { value }) {
    return sql.value(value ?? null);
  }
  exprRecord(ctx, { fields }) {
    return fromEntries(
      entries(fields).map(([key, expr]) => {
        return [key, this.expr(ctx, expr)];
      })
    );
  }
  exprCall(ctx, { method, args }) {
    switch (method) {
      case "snippet":
        return snippet(
          EntrySearch,
          1,
          this.expr(ctx, args[0]),
          this.expr(ctx, args[1]),
          this.expr(ctx, args[2]),
          this.expr(ctx, args[3])
        );
      default:
        throw new Error(`Unknown method: "${method}"`);
    }
  }
  expr(ctx, expr) {
    switch (expr.type) {
      case "unop":
        return this.exprUnOp(ctx, expr);
      case "binop":
        return this.exprBinOp(ctx, expr);
      case "field":
        return this.exprField(ctx, expr);
      case "access":
        return this.exprAccess(ctx, expr);
      case "value":
        return this.exprValue(ctx, expr);
      case "record":
        throw new Error("Record expressions are not supported");
      case "call":
        return this.exprCall(ctx, expr);
    }
  }
  selectRecord(ctx, { fields }) {
    return fromEntries(
      fields.flatMap((field) => {
        switch (field.length) {
          case 1:
            const [target] = field;
            return this.selectFieldsOf(ctx.select, target);
          case 2:
            const [key, selection2] = field;
            return [[key, this.select(ctx, selection2)]];
        }
      })
    );
  }
  selectRow(ctx, { target }) {
    return fromEntries(this.selectFieldsOf(ctx.select, target));
  }
  selectCursor(ctx, selection2) {
    const isSingle = selection2.cursor.first ?? false;
    const query = this.queryCursor(ctx, selection2);
    if (isSingle)
      return include.one(query);
    return include(query);
  }
  selectExpr(ctx, { expr, fromParent }) {
    ctx = fromParent ? ctx.decreaseDepth() : ctx;
    return this.expr(ctx.select, expr);
  }
  selectCount(ctx, hasSearch) {
    return count(hasSearch ? EntrySearch.rowid : ctx.Table.entryId).as("count");
  }
  selectAll(ctx, target) {
    const fields = this.selectFieldsOf(ctx.select, target);
    return fromEntries(fields);
  }
  select(ctx, selection2, hasSearch = false) {
    switch (selection2.type) {
      case "cursor":
        return this.selectCursor(ctx, selection2);
      case "row":
        return this.selectRow(ctx, selection2);
      case "record":
        return this.selectRecord(ctx, selection2);
      case "expr":
        return this.selectExpr(ctx, selection2);
      case "count":
        return this.selectCount(ctx, hasSearch);
    }
  }
  queryRecord(ctx, selection2) {
    const expr = this.selectRecord(ctx.select, selection2);
    return builder.select(expr);
  }
  querySource(ctx, source, hasSearch) {
    const { aliased } = getTable(ctx.Table);
    const cursor = hasSearch ? builder.select(ctx.Table).from(EntrySearch).innerJoin(
      ctx.Table,
      eq(sql`${sql.identifier(aliased)}.rowid`, EntrySearch.rowid)
    ) : builder.select().from(ctx.Table);
    if (!source)
      return cursor.orderBy(asc(ctx.Table.index));
    const from = alias(EntryRow, `E${ctx.depth - 1}`);
    switch (source.type) {
      case SourceType.Parent:
        return cursor.where(eq(ctx.Table.entryId, from.parent)).limit(1);
      case SourceType.Next:
        return cursor.where(
          eq(ctx.Table.parent, from.parent),
          gt(ctx.Table.index, from.index)
        ).limit(1);
      case SourceType.Previous:
        return cursor.where(
          eq(ctx.Table.parent, from.parent),
          lt(ctx.Table.index, from.index)
        ).limit(1);
      case SourceType.Siblings:
        return cursor.where(
          eq(ctx.Table.parent, from.parent),
          source.includeSelf ? void 0 : ne(ctx.Table.entryId, from.entryId)
        );
      case SourceType.Translations:
        return cursor.where(
          eq(ctx.Table.i18nId, from.i18nId),
          source.includeSelf ? void 0 : ne(ctx.Table.entryId, from.entryId)
        );
      case SourceType.Children:
        const Child = alias(EntryRow, "Child");
        const children = builder.$with("children").as(
          builder.select({
            entryId: Child.entryId,
            parent: Child.parent,
            level: sql`0`
          }).from(Child).where(
            eq(Child.entryId, from.entryId),
            this.conditionRealm(Child, ctx.realm),
            this.conditionLocale(Child, ctx.locale)
          ).unionAll(
            (self) => builder.select({
              entryId: Child.entryId,
              parent: Child.parent,
              level: sql`${self.level} + 1`
            }).from(Child).innerJoin(self, eq(self.entryId, Child.parent)).where(
              this.conditionRealm(Child, ctx.realm),
              this.conditionLocale(Child, ctx.locale),
              lt(self.level, Math.min(source.depth ?? MAX_DEPTH, MAX_DEPTH))
            )
          )
        );
        const childrenIds = builder.withRecursive(children).select(children.entryId).from(children).limit(-1).offset(1);
        return cursor.where(inArray(ctx.Table.entryId, childrenIds)).orderBy(asc(ctx.Table.index));
      case SourceType.Parents:
        const Parent = alias(EntryRow, "Parent");
        const parents = builder.$with("parents").as(
          builder.select({
            entryId: Parent.entryId,
            parent: Parent.parent,
            level: sql`0`
          }).from(Parent).where(
            eq(Parent.entryId, from.entryId),
            this.conditionRealm(Parent, ctx.realm),
            this.conditionLocale(Parent, ctx.locale)
          ).unionAll(
            (self) => builder.select({
              entryId: Parent.entryId,
              parent: Parent.parent,
              level: sql`${self.level} + 1`
            }).from(Parent).innerJoin(self, eq(self.parent, Parent.entryId)).where(
              this.conditionRealm(Parent, ctx.realm),
              this.conditionLocale(Parent, ctx.locale),
              lt(self.level, Math.min(source.depth ?? MAX_DEPTH, MAX_DEPTH))
            )
          )
        );
        const parentIds = builder.withRecursive(parents).select(parents.entryId).from(parents).limit(-1).offset(1);
        return cursor.where(inArray(ctx.Table.entryId, parentIds)).orderBy(asc(ctx.Table.level));
      default:
        throw unreachable(source.type);
    }
  }
  orderBy(ctx, orderBy) {
    return orderBy.map(({ expr, order }) => {
      const e = this.expr(ctx, expr);
      const direction = order === "Desc" ? desc : asc;
      return direction(e);
    });
  }
  conditionLocale(Table, locale) {
    if (!locale)
      return sql.value(true);
    return eq(Table.locale, locale);
  }
  conditionRealm(Table, realm) {
    switch (realm) {
      case Realm.Published:
        return eq(Table.phase, EntryPhase.Published);
      case Realm.Draft:
        return eq(Table.phase, EntryPhase.Draft);
      case Realm.Archived:
        return eq(Table.phase, EntryPhase.Archived);
      case Realm.PreferDraft:
        return Table.active;
      case Realm.PreferPublished:
        return Table.main;
      case Realm.All:
        return sql.value(true);
    }
  }
  conditionLocation(Table, location) {
    switch (location.length) {
      case 1:
        return eq(Table.workspace, location[0]);
      case 2:
        return and(
          eq(Table.workspace, location[0]),
          eq(Table.root, location[1])
        );
      case 3:
        return and(
          eq(Table.workspace, location[0]),
          eq(Table.root, location[1]),
          like(Table.parentDir, `/${location[2]}%`)
        );
      default:
        return sql.value(true);
    }
  }
  conditionSearch(Table, searchTerms) {
    if (!searchTerms?.length)
      return sql.value(true);
    const terms = searchTerms.map((term) => `"${term.replaceAll('"', "")}"*`).join(" AND ");
    return sql`${input(EntrySearch)} match ${input(terms)}`;
  }
  queryCursor(ctx, { cursor }) {
    const {
      target,
      where,
      skip,
      take,
      orderBy,
      groupBy,
      select,
      source,
      first,
      searchTerms
    } = cursor;
    ctx = ctx.increaseDepth().none;
    const { name } = target || {};
    const hasSearch = Boolean(searchTerms?.length);
    let query = this.querySource(ctx, source, hasSearch);
    const queryData = getData(query);
    let preCondition = queryData.where;
    let condition = and(
      preCondition,
      name ? eq(ctx.Table.type, name) : void 0,
      this.conditionLocation(ctx.Table, ctx.location),
      this.conditionRealm(ctx.Table, ctx.realm),
      source?.type === SourceType.Translations ? void 0 : this.conditionLocale(ctx.Table, ctx.locale),
      this.conditionSearch(ctx.Table, searchTerms)
    );
    if (skip)
      query = query.offset(skip);
    if (take)
      query = query.limit(take);
    const toSelect = select ? this.select(ctx.select, select, hasSearch) : this.selectAll(ctx, { name });
    let result = new Select({
      ...queryData,
      select: selection(toSelect),
      where: where ? and(condition, this.expr(ctx.condition, where)) : condition
    });
    if (groupBy)
      result = result.groupBy(...groupBy.map((expr) => this.expr(ctx, expr)));
    if (searchTerms)
      result = result.orderBy(asc(bm25(EntrySearch, 20, 1)));
    else if (orderBy)
      result = result.orderBy(...this.orderBy(ctx, orderBy));
    if (first)
      result = result.limit(1);
    return result;
  }
  query(ctx, selection2) {
    switch (selection2.type) {
      case "cursor":
        return this.queryCursor(ctx, selection2);
      case "record":
        return this.queryRecord(ctx, selection2);
      case "row":
      case "count":
      case "expr":
        throw new Error(`Cannot select ${selection2.type} at root level`);
    }
  }
  isSingleResult(ctx, selection2) {
    switch (selection2.type) {
      case "cursor":
        return selection2.cursor.first ?? false;
      default:
        return true;
    }
  }
  async postRow(ctx, interim, { target }) {
    await this.postFieldsOf(ctx, interim, target);
  }
  async postCursor(ctx, interim, { cursor }) {
    if (!interim)
      return;
    const { target = {}, select, first } = cursor;
    if (select) {
      if (first)
        await this.post(ctx, interim, select);
      else
        await Promise.all(
          interim.map((row) => this.post(ctx, row, select))
        );
    } else {
      if (first)
        await this.postFieldsOf(ctx, interim, target);
      else
        await Promise.all(
          interim.map((row) => this.postFieldsOf(ctx, row, target))
        );
    }
  }
  async postField(ctx, interim, { target, field }) {
    const { name } = target;
    if (!name)
      return;
    const type = this.schema[name];
    if (!type)
      return;
    const shape = Field.shape(Type.field(type, field));
    await shape.applyLinks(interim, ctx.linkResolver);
  }
  async postExpr(ctx, interim, { expr }) {
    if (expr.type === "field")
      await this.postField(ctx, interim, expr);
  }
  async postFieldsOf(ctx, interim, target) {
    if (!interim)
      return;
    const { name } = target;
    if (!name)
      return;
    const type = this.schema[name];
    if (!type)
      return;
    await Promise.all(
      keys(type).map((field) => {
        return this.postField(ctx, interim[field], {
          type: "field",
          target,
          field
        });
      })
    );
  }
  async postRecord(ctx, interim, { fields }) {
    if (!interim)
      return;
    const tasks = [];
    for (const field of fields) {
      switch (field.length) {
        case 1:
          const [target] = field;
          tasks.push(this.postFieldsOf(ctx, interim, target));
          continue;
        case 2:
          const [key, selection2] = field;
          tasks.push(this.post(ctx, interim[key], selection2));
          continue;
      }
    }
    await Promise.all(tasks);
  }
  post(ctx, interim, selection2) {
    switch (selection2.type) {
      case "row":
        return this.postRow(ctx, interim, selection2);
      case "cursor":
        return this.postCursor(ctx, interim, selection2);
      case "record":
        return this.postRecord(ctx, interim, selection2);
      case "expr":
        return this.postExpr(ctx, interim, selection2);
      case "count":
        return Promise.resolve();
    }
  }
  resolve = async ({
    selection: selection2,
    location,
    locale,
    realm = Realm.Published,
    preview
  }) => {
    const ctx = new ResolveContext({ realm, location, locale });
    const query = this.query(ctx, selection2);
    const singleResult = this.isSingleResult(ctx, selection2);
    const transact = async (tx) => {
      const rows = await query.all(tx);
      const linkResolver = new LinkResolver(this, tx, ctx.realm);
      const result = singleResult ? rows[0] : rows;
      if (result)
        await this.post({ linkResolver }, result, selection2);
      return result;
    };
    if (preview) {
      const updated = "entry" in preview ? preview.entry : void 0;
      if (updated) {
        const result = await this.db.preview(updated, transact);
        if (result)
          return result;
      }
    }
    return this.db.store.transaction(transact);
  };
};
export {
  EntryResolver
};
