import {
  Doc,
  applyUpdateV2,
  mergeUpdatesV2
} from "../../chunks/chunk-BWVXXM6J.js";
import "../../chunks/chunk-FFKU477P.js";
import "../../chunks/chunk-IOTY7UTU.js";
import "../../chunks/chunk-BRTZLYIM.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/backend/resolver/ParsePreview.ts
import { Entry } from "alinea/core";
import { parseYDoc } from "alinea/core/Doc";
import { createSelection } from "alinea/core/pages/CreateSelection";
import { Realm } from "alinea/core/pages/Realm";
import { decodePreviewPayload } from "alinea/preview/PreviewPayload";
function createPreviewParser(db) {
  const drafts = /* @__PURE__ */ new Map();
  return {
    async parse(preview, sync, getDraft) {
      if (!(preview && "payload" in preview))
        return preview;
      const update = await decodePreviewPayload(preview.payload);
      let meta = await db.meta();
      if (update.contentHash !== meta.contentHash) {
        await sync();
        meta = await db.meta();
      }
      const entry = await db.resolver.resolve({
        selection: createSelection(
          Entry({ entryId: update.entryId }).maybeFirst()
        ),
        realm: Realm.PreferDraft
      });
      if (!entry)
        return;
      const cachedDraft = await drafts.get(update.entryId);
      let currentDraft;
      if (cachedDraft?.contentHash === meta.contentHash) {
        currentDraft = cachedDraft.draft;
      } else {
        try {
          const pending = getDraft(update.entryId);
          drafts.set(
            update.entryId,
            pending.then((draft) => ({ contentHash: meta.contentHash, draft }))
          );
          currentDraft = await pending;
        } catch (error) {
          console.warn("> could not fetch draft", error);
        }
      }
      const apply = currentDraft ? mergeUpdatesV2([currentDraft.draft, update.update]) : update.update;
      const type = db.config.schema[entry.type];
      if (!type)
        return;
      const doc = new Doc();
      applyUpdateV2(doc, apply);
      const entryData = parseYDoc(type, doc);
      return { entry: { ...entry, ...entryData, path: entry.path } };
    },
    setDraft(entryId, input) {
      drafts.set(entryId, Promise.resolve(input));
    }
  };
}
export {
  createPreviewParser
};
