import {
  JsonColumn,
  column
} from "./chunk-GR7UQZEH.js";
import {
  sql
} from "./chunk-NXCD6SNE.js";

// node_modules/rado/dist/universal/columns.js
var idType = sql.universal({
  sqlite: sql`integer`,
  postgres: sql`integer generated always as identity`,
  mysql: sql`int not null auto_increment`
});
var blobType = sql.universal({
  postgres: sql`bytea`,
  default: sql`blob`
});
var numberType = sql.universal({
  mysql: sql`double`,
  default: sql`numeric`
});
var jsonbType = sql.universal({
  mysql: sql`json`,
  default: sql`jsonb`
});
function id(name) {
  return column({
    name,
    type: idType,
    primary: true
  });
}
function text(name) {
  return column({ name, type: column.text() });
}
function integer(name) {
  return column({ name, type: column.integer() });
}
function boolean(name) {
  return column({
    name,
    type: column.boolean(),
    mapFromDriverValue: Boolean
  });
}
function json(name) {
  return new JsonColumn({
    name,
    type: column.json(),
    mapToDriverValue(value) {
      return JSON.stringify(value);
    },
    mapFromDriverValue(value, { parsesJson }) {
      return parsesJson ? value : JSON.parse(value);
    }
  });
}
function jsonb(name) {
  return new JsonColumn({
    name,
    type: jsonbType,
    mapToDriverValue(value) {
      return JSON.stringify(value);
    },
    mapFromDriverValue(value, { parsesJson }) {
      return parsesJson ? value : JSON.parse(value);
    }
  });
}
function blob(name) {
  return column({
    name,
    type: blobType,
    mapFromDriverValue(value) {
      return new Uint8Array(value);
    }
  });
}

export {
  id,
  text,
  integer,
  boolean,
  json,
  jsonb,
  blob
};
