// package.json
var package_default = {
  bin: "./dist/cli.js",
  name: "alinea",
  version: "0.11.2",
  license: "MIT",
  type: "module",
  scripts: {
    dev: "run-p dev:check dev:run",
    web: "run-p dev:check web:run",
    stories: "ladle serve",
    demo: "node build.js --watch -- node dev.js --dir apps/demo --config src/cms -- yarn workspace @alinea/demo dev",
    build: "tsc && node build.js",
    "cycles:server": "CHECK_CYCLES=server node build.js && madge --warning --circular dist/index.js",
    "cycles:browser": "CHECK_CYCLES=browser node build.js && madge --warning --circular dist/index.js dist/ui.js dist/dashboard/App.js",
    alinea: "node build.js && node dist/cli.js",
    "build:alinea": "node build.js",
    "release:types": "tsc",
    "release:build": "node build.js",
    type: "tsc --diagnostics",
    test: "NODE_OPTIONS=--enable-source-maps node build.js --test",
    tag: "esbx tag",
    "dev:check": "tsc -w",
    "dev:run": "node build.js --watch -- node dev.js --dir apps/dev",
    "web:run": "node build.js --watch -- node dev.js -- yarn workspace @alinea/web dev",
    "dev:stories": 'ladle serve --stories "src/**/*.stories.tsx"',
    "gen:bundleTs": "esbx bundleTs",
    "build:stories": 'ladle build --base /stories --stories  "packages/**/*.stories.tsx" --outDir apps/web/public/stories',
    trace: "tsc --noEmit --incremental false --emitDeclarationOnly false --diagnostics --generateTrace private/trace && analyze-trace private/trace & speedscope private/trace/trace.json",
    bump: "esbx version",
    prepublishOnly: "yarn build"
  },
  sideEffects: false,
  typesVersions: {
    "*": {
      "*": [
        "./dist/*"
      ]
    }
  },
  files: [
    "dist"
  ],
  dependencies: {
    "@alinea/iso": "^0.3.2",
    "@alinea/sqlite-wasm": "^0.1.18",
    esbuild: "^0.23.1"
  },
  peerDependencies: {
    react: "*",
    "react-dom": "*"
  },
  devDependencies: {
    "@alinea/styler": "^1.0.3",
    "@esbx/reporter": "^0.0.20",
    "@esbx/target": "^0.0.20",
    "@esbx/util": "^0.0.20",
    "@esbx/workspaces": "^0.0.20",
    "@ladle/react": "^4.0.2",
    "@types/fs-extra": "^11.0.1",
    "@types/glob": "^7.1.4",
    "@types/node": "^22.5.2",
    "@types/react": "^18.2.13",
    "@types/react-dom": "^18.2.6",
    "@types/sql.js": "^1.4.4",
    "@typescript/analyze-trace": "^0.9.1",
    "@ungap/with-resolvers": "^0.1.0",
    alinea: "link:.",
    esbx: "^0.0.18",
    glob: "^7.2.0",
    madge: "^6.1.0",
    "npm-run-all": "^4.1.5",
    "postcss-modules": "^6.0.0",
    "postcss-pxtorem": "^6.0.0",
    react: "^18.3.1",
    "react-dom": "^18.3.1",
    sass: "^1.63.4",
    sharp: "0.32.6",
    speedscope: "^1.14.0",
    typescript: "^5.6.2",
    "typescript-plugin-css-modules": "^5.0.2",
    uvu: "^0.5.1",
    "yjs-src": "npm:yjs@13.6.11"
  },
  packageManager: "yarn@4.5.0",
  resolutions: {
    esbuild: "0.20.1",
    yjs: "13.6.11",
    lib0: "0.2.88"
  },
  exports: {
    "./package.json": "./package.json",
    ".": "./dist/index.js",
    "./css": "./dist/index.css",
    "./*.cjs": "./dist/*.cjs",
    "./*": "./dist/*.js",
    "./cloud/view/CloudAuth": {
      worker: "./dist/cloud/view/CloudAuth.js",
      browser: "./dist/cloud/view/CloudAuth.browser.js",
      default: "./dist/cloud/view/CloudAuth.js"
    },
    "./core/media/CreatePreview": {
      worker: "./dist/core/media/CreatePreview.js",
      browser: "./dist/core/media/CreatePreview.browser.js",
      default: "./dist/core/media/CreatePreview.js"
    },
    "./dashboard/view/Preview": {
      worker: "./dist/dashboard/view/Preview.js",
      browser: "./dist/dashboard/view/Preview.browser.js",
      default: "./dist/dashboard/view/Preview.js"
    },
    "./picker/entry/EntryPicker": {
      worker: "./dist/picker/entry/EntryPicker.js",
      browser: "./dist/picker/entry/EntryPicker.browser.js",
      default: "./dist/picker/entry/EntryPicker.js"
    },
    "./picker/url/UrlPicker": {
      worker: "./dist/picker/url/UrlPicker.js",
      browser: "./dist/picker/url/UrlPicker.browser.js",
      default: "./dist/picker/url/UrlPicker.js"
    },
    "./cli/util/ExportStore": {
      worker: "./dist/cli/util/ExportStore.server.js",
      browser: "./dist/cli/util/ExportStore.js",
      default: "./dist/cli/util/ExportStore.server.js"
    }
  }
};

export {
  package_default
};
