import {
  and
} from "./chunk-PE5G5FUO.js";
import {
  Field,
  input
} from "./chunk-AOFVF5VC.js";
import {
  Sql,
  getData,
  getField,
  getQuery,
  getResolver,
  getSelection,
  getSql,
  getTable,
  getTarget,
  hasField,
  hasSelection,
  hasSql,
  hasTable,
  hasTarget,
  internalData,
  internalQuery,
  internalSelection,
  internalSql,
  internalTarget,
  sql
} from "./chunk-NXCD6SNE.js";

// node_modules/rado/dist/core/Virtual.js
function virtual(alias, source) {
  const target = { [internalTarget]: sql.identifier(alias) };
  if (source && hasSql(source)) {
    const expr = getSql(source);
    const name = expr.alias;
    if (!name)
      throw new Error("Cannot alias a virtual field without a name");
    return Object.assign(new Field(alias, name, expr), target);
  }
  return new Proxy(target, {
    get(target2, field) {
      if (field in target2)
        return target2[field];
      const from = source?.[field];
      if (typeof field !== "string")
        return from;
      return target2[field] = new Field(
        alias,
        field,
        from ? getSql(from) : void 0
      );
    }
  });
}

// node_modules/rado/dist/core/Selection.js
var SqlColumn = class {
  constructor(sql2, targetName) {
    this.sql = sql2;
    this.targetName = targetName;
  }
  result(ctx) {
    const value = ctx.values[ctx.index++];
    if (!this.sql.mapFromDriverValue)
      return value;
    return this.sql.mapFromDriverValue(value, ctx.specs);
  }
};
var ObjectColumn = class {
  constructor(nullable, entries) {
    this.nullable = nullable;
    this.entries = entries;
  }
  result(ctx) {
    const result = {};
    let isNullable = this.nullable.size > 0;
    for (const entry of this.entries) {
      const name = entry[0];
      const col = entry[1];
      const value = col.result(ctx);
      result[name] = value;
      if (isNullable) {
        if (value === null) {
          if (col.targetName && !this.nullable.has(col.targetName))
            isNullable = false;
        } else {
          isNullable = false;
        }
      }
    }
    if (isNullable)
      return null;
    return result;
  }
};
var Selection = class {
  constructor(input2, nullable) {
    this.input = input2;
    this.nullable = nullable;
    const root = this.#defineColumn(nullable, input2);
    this.mapRow = root.result.bind(root);
  }
  mapRow;
  makeVirtual(name) {
    return virtual(name, this.input);
  }
  #defineColumn(nullable, input2) {
    const expr = getSql(input2);
    if (expr)
      return new SqlColumn(expr, getField(input2)?.targetName);
    return new ObjectColumn(
      nullable,
      Object.entries(input2).map(([name, value]) => [
        name,
        this.#defineColumn(nullable, value)
      ])
    );
  }
  fieldNames() {
    return this.#fieldNames(this.input, /* @__PURE__ */ new Set());
  }
  #fieldNames(input2, names, name) {
    const expr = getSql(input2);
    if (expr) {
      let exprName = name ?? expr.alias;
      if (!exprName)
        throw new Error("Missing field name");
      while (names.has(exprName))
        exprName = `${exprName}_`;
      return [exprName];
    }
    return Object.entries(input2).flatMap(
      ([name2, value]) => this.#fieldNames(value, names, name2)
    );
  }
  #selectionToSql(input2, names, name) {
    const expr = getSql(input2);
    if (expr) {
      let exprName = name ?? expr.alias;
      if (exprName) {
        while (names.has(exprName))
          exprName = `${exprName}_`;
        names.add(exprName);
        if (hasField(input2)) {
          const field = getField(input2);
          if (field.fieldName === exprName)
            return expr;
        }
        return sql`${expr.forSelection()} as ${sql.identifier(exprName)}`;
      }
      return expr;
    }
    return sql.join(
      Object.entries(input2).map(
        ([name2, value]) => this.#selectionToSql(value, names, name2)
      ),
      sql`, `
    );
  }
  get [internalSql]() {
    return this.#selectionToSql(this.input, /* @__PURE__ */ new Set());
  }
  join(right, operator) {
    return this;
  }
};
var TableSelection = class extends Selection {
  constructor(table) {
    super(table, /* @__PURE__ */ new Set());
    this.table = table;
  }
  join(right, operator) {
    const leftTable = getTable(this.table);
    const rightTable = getTable(right);
    const nullable = new Set(this.nullable);
    if (operator === "right" || operator === "full")
      nullable.add(leftTable.aliased);
    if (operator === "left" || operator === "full")
      nullable.add(rightTable.aliased);
    return new JoinSelection([this.table, right], nullable);
  }
};
var JoinSelection = class _JoinSelection extends Selection {
  constructor(tables, nullable) {
    super(
      Object.fromEntries(tables.map((table) => [getTable(table).aliased, table])),
      nullable
    );
    this.tables = tables;
  }
  join(right, operator) {
    const rightTable = getTable(right);
    const nullable = new Set(this.nullable);
    if (operator === "right" || operator === "full")
      this.tables.map((table) => getTable(table).aliased).forEach(nullable.add, nullable);
    if (operator === "left" || operator === "full")
      nullable.add(rightTable.aliased);
    return new _JoinSelection([...this.tables, right], nullable);
  }
};
function selection(input2, nullable = []) {
  return new Selection(input2, new Set(nullable));
}
((selection2) => {
  function table(table2) {
    return new TableSelection(table2);
  }
  selection2.table = table;
})(selection || (selection = {}));

// node_modules/rado/dist/core/Query.js
var Executable = class {
  #execute;
  constructor(exec) {
    this.#execute = exec;
  }
  *[Symbol.iterator]() {
    const interim = this.#execute();
    const isAsync = interim instanceof Promise;
    if (!isAsync)
      return interim;
    let result;
    yield interim.then((v) => result = v);
    return result;
  }
  run() {
    return this.#execute();
  }
  // biome-ignore lint/suspicious/noThenProperty:
  async then(onfulfilled, onrejected) {
    try {
      const result = await this.#execute();
      return onfulfilled ? onfulfilled(result) : result;
    } catch (error) {
      return onrejected ? onrejected(error) : Promise.reject(error);
    }
  }
  catch(onrejected) {
    return this.then().catch(onrejected);
  }
  finally(onfinally) {
    return this.then().finally(onfinally);
  }
};
var QueryBatch = class extends Executable {
  constructor(queryResolver, queries) {
    super(() => {
      return queryResolver.batch(queries).execute();
    });
  }
};
var Query = class extends Executable {
  [internalData];
  constructor(data) {
    super(() => this.#exec(void 0));
    this[internalData] = data;
  }
  #exec(method, db) {
    const data = getData(this);
    const resolver = db ? getResolver(db) : data.resolver;
    const isSelection = hasSelection(this);
    const prepared = resolver.prepare(this, "");
    const resultType = method ?? (isSelection ? "all" : "run");
    try {
      const result = prepared[resultType]();
      if (result instanceof Promise)
        return result.then((res) => res ?? null).finally(prepared.free.bind(prepared));
      prepared.free();
      return result ?? null;
    } catch (error) {
      prepared.free();
      throw error;
    }
  }
  all(db) {
    return this.#exec("all", db);
  }
  get(db) {
    return this.#exec("get", db);
  }
  run(db) {
    return this.#exec("run", db);
  }
  prepare(name) {
    return getData(this).resolver.prepare(this, name);
  }
  toSQL(db) {
    const resolver = db ? getResolver(db) : getData(this).resolver;
    if (!resolver)
      throw new Error("Query has no resolver");
    return resolver.toSQL(this);
  }
};

// node_modules/rado/dist/core/query/Union.js
var UnionBase = class extends Query {
  [internalData];
  #makeSelf() {
    const { select } = getData(this);
    return select.makeVirtual(Sql.SELF_TARGET);
  }
  union(right) {
    return new Union({
      ...getData(this),
      left: this,
      operator: sql`union`,
      right: typeof right === "function" ? right(this.#makeSelf()) : right
    });
  }
  unionAll(right) {
    return new Union({
      ...getData(this),
      left: this,
      operator: sql`union all`,
      right: typeof right === "function" ? right(this.#makeSelf()) : right
    });
  }
  intersect(right) {
    return new Union({
      ...getData(this),
      left: this,
      operator: sql`intersect`,
      right: typeof right === "function" ? right(this.#makeSelf()) : right
    });
  }
  intersectAll(right) {
    return new Union({
      ...getData(this),
      left: this,
      operator: sql`intersect all`,
      right: typeof right === "function" ? right(this.#makeSelf()) : right
    });
  }
  except(right) {
    return new Union({
      ...getData(this),
      left: this,
      operator: sql`except`,
      right: typeof right === "function" ? right(this.#makeSelf()) : right
    });
  }
  exceptAll(right) {
    return new Union({
      ...getData(this),
      left: this,
      operator: sql`except all`,
      right: typeof right === "function" ? right(this.#makeSelf()) : right
    });
  }
};
var Union = class extends UnionBase {
  [internalData];
  [internalSelection];
  constructor(data) {
    super(data);
    this[internalData] = data;
    this[internalSelection] = data.select;
  }
  get [internalQuery]() {
    return sql.chunk("emitUnion", getData(this));
  }
};

// node_modules/rado/dist/core/query/Select.js
var Select = class _Select extends UnionBase {
  [internalData];
  constructor(data) {
    super(data);
    this[internalData] = data;
  }
  as(alias) {
    const fields = getSelection(this).makeVirtual(alias);
    return Object.assign(fields, {
      [internalTarget]: sql`(${getQuery(this)}) as ${sql.identifier(
        alias
      )}`.inlineFields(true)
    });
  }
  from(target) {
    const { select: current } = getData(this);
    const from = hasTarget(target) ? getTarget(target) : getSql(target);
    const isTable = hasTable(target);
    const selected = current ?? (isTable ? selection.table(target) : selection(sql`*`));
    return new _Select({
      ...getData(this),
      select: selected,
      from
    });
  }
  #join(operator, right, on) {
    const { from, select: current } = getData(this);
    return new _Select({
      ...getData(this),
      select: hasTable(right) ? current?.join(right, operator) : current,
      from: sql.join([
        from,
        sql.unsafe(`${operator} join`),
        getTarget(right),
        sql`on ${on}`
      ])
    });
  }
  leftJoin(right, on) {
    return this.#join("left", right, on);
  }
  rightJoin(right, on) {
    return this.#join("right", right, on);
  }
  innerJoin(right, on) {
    return this.#join("inner", right, on);
  }
  fullJoin(right, on) {
    return this.#join("full", right, on);
  }
  where(...where) {
    return new _Select({ ...getData(this), where: and(...where) });
  }
  groupBy(...exprs) {
    return new _Select({
      ...getData(this),
      groupBy: sql.join(exprs, sql.unsafe(", "))
    });
  }
  having(having) {
    return new _Select({
      ...getData(this),
      having: typeof having === "function" ? having(getSelection(this).input) : having
    });
  }
  orderBy(...exprs) {
    return new _Select({
      ...getData(this),
      orderBy: sql.join(exprs, sql.unsafe(", "))
    });
  }
  limit(limit) {
    return new _Select({ ...getData(this), limit: input(limit) });
  }
  offset(offset) {
    return new _Select({
      ...getData(this),
      offset: input(offset)
    });
  }
  get [internalSelection]() {
    const { select } = getData(this);
    if (!select)
      throw new Error("No selection defined");
    return select;
  }
  get [internalQuery]() {
    return sql.chunk("emitSelect", getData(this));
  }
  get [internalSql]() {
    return sql`(${getQuery(this)})`;
  }
};

// node_modules/rado/dist/core/query/Delete.js
var Delete = class extends Query {
  [internalData];
  constructor(data) {
    super(data);
    this[internalData] = data;
    if (data.returning)
      this[internalSelection] = data.returning;
  }
  get [internalQuery]() {
    return sql.chunk("emitDelete", this);
  }
};
var DeleteFrom = class _DeleteFrom extends Delete {
  where(...where) {
    return new _DeleteFrom({ ...getData(this), where: and(...where) });
  }
  returning(returning) {
    const data = getData(this);
    return new Delete({
      ...data,
      returning: returning ? selection(returning) : selection.table(data.from)
    });
  }
};

// node_modules/rado/dist/core/query/Insert.js
var Insert = class extends Query {
  [internalData];
  constructor(data) {
    super(data);
    this[internalData] = data;
    if (data.returning)
      this[internalSelection] = data.returning;
  }
  get [internalQuery]() {
    return sql.chunk("emitInsert", this);
  }
};
var InsertCanReturn = class extends Insert {
  returning(returning) {
    const data = getData(this);
    return new Insert({
      ...data,
      returning: returning ? selection(returning) : selection.table(data.into)
    });
  }
};
var InsertCanConflict = class extends InsertCanReturn {
  onConflictDoNothing(onConflict) {
    return this.#onConflict(onConflict ?? {});
  }
  onConflictDoUpdate(onConflict) {
    return this.#onConflict(onConflict);
  }
  onDuplicateKeyUpdate(update) {
    return new InsertCanReturn({
      ...getData(this),
      onDuplicateKeyUpdate: this.#updateFields(update.set)
    });
  }
  #updateFields(update) {
    return sql.join(
      Object.entries(update).map(
        ([key, value]) => sql`${sql.identifier(key)} = ${input(value)}`
      ),
      sql`, `
    );
  }
  #onConflict({
    target,
    targetWhere,
    set,
    setWhere
  }) {
    const update = set && this.#updateFields(set);
    return new InsertCanReturn({
      ...getData(this),
      onConflict: sql.join([
        target && sql`(${Array.isArray(target) ? sql.join(target, sql`, `) : target})`,
        targetWhere && sql`where ${targetWhere}`,
        update ? sql.join([
          sql`do update set ${update}`,
          setWhere && sql`where ${setWhere}`
        ]) : sql`do nothing`
      ])
    });
  }
};
var defaultKeyword = sql.universal({
  sqlite: sql`null`,
  default: sql`default`
});
var InsertInto = class {
  [internalData];
  constructor(data) {
    this[internalData] = data;
  }
  values(insert) {
    const { into } = getData(this);
    const rows = Array.isArray(insert) ? insert : [insert];
    const table = getTable(into);
    const values = sql.join(
      rows.map((row) => {
        return sql`(${sql.join(
          Object.entries(table.columns).map(([key, column]) => {
            const value = row[key];
            const { $default, mapToDriverValue } = getData(column);
            if (value !== void 0)
              return input(mapToDriverValue?.(value) ?? value);
            if ($default)
              return $default();
            return defaultKeyword;
          }),
          sql`, `
        )})`;
      }),
      sql`, `
    );
    return new InsertCanConflict({ ...getData(this), values });
  }
  select(query) {
    return new InsertCanConflict({
      ...getData(this),
      select: getQuery(query)
    });
  }
};

// node_modules/rado/dist/core/query/Update.js
var Update = class extends Query {
  [internalData];
  constructor(data) {
    super(data);
    this[internalData] = data;
    if (data.returning)
      this[internalSelection] = data.returning;
  }
  get [internalQuery]() {
    return sql.chunk("emitUpdate", this);
  }
};
var UpdateTable = class _UpdateTable extends Update {
  set(values) {
    const { table } = getData(this);
    const set = sql.join(
      Object.entries(values).map(([key, value]) => {
        const column = getTable(table).columns[key];
        const { mapToDriverValue } = getData(column);
        return sql`${sql.identifier(key)} = ${input(
          mapToDriverValue?.(value) ?? value
        )}`;
      }),
      sql`, `
    );
    return new _UpdateTable({ ...getData(this), set });
  }
  where(...where) {
    return new _UpdateTable({
      ...getData(this),
      where: and(...where)
    });
  }
  returning(returning) {
    const data = getData(this);
    return new Update({
      ...data,
      returning: returning ? selection(returning) : selection.table(data.table)
    });
  }
};

// node_modules/rado/dist/core/Builder.js
var BuilderBase = class {
  [internalData];
  constructor(data = {}) {
    this[internalData] = data;
  }
  select(input2) {
    return new Select({
      ...getData(this),
      select: input2 && selection(input2)
    });
  }
  selectDistinct(input2) {
    return new Select({
      ...getData(this),
      select: input2 && selection(input2),
      distinct: true
    });
  }
  selectDistinctOn(columns, input2) {
    return new Select({
      ...getData(this),
      select: input2 && selection(input2),
      distinctOn: columns
    });
  }
  update(table) {
    return new UpdateTable({ ...getData(this), table });
  }
  insert(into) {
    return new InsertInto({ ...getData(this), into });
  }
  delete(from) {
    return new DeleteFrom({ ...getData(this), from });
  }
};
var Builder = class extends BuilderBase {
  $with(cteName) {
    return {
      as(query) {
        const fields = getSelection(query).makeVirtual(cteName);
        return Object.assign(fields, {
          [internalQuery]: getQuery(query).nameSelf(cteName)
        });
      }
    };
  }
  with(...definitions) {
    return new BuilderBase({
      ...getData(this),
      cte: { recursive: false, definitions }
    });
  }
  withRecursive(...definitions) {
    return new BuilderBase({
      ...getData(this),
      cte: { recursive: true, definitions }
    });
  }
};

export {
  selection,
  QueryBatch,
  Select,
  Builder
};
