import {
  prettyBytes
} from "../chunks/chunk-VRNNS7VT.js";
import "../chunks/chunk-U5RRZUYZ.js";

// src/cli/Generate.ts
import { exportStore } from "alinea/cli/util/ExportStore.server";
import { Config } from "alinea/core/Config";
import { genEffect } from "alinea/core/util/Async";
import { basename, join } from "alinea/core/util/Paths";
import fs from "node:fs";
import { createRequire } from "node:module";
import path from "node:path";
import { compileConfig } from "./generate/CompileConfig.js";
import { copyStaticFiles } from "./generate/CopyStaticFiles.js";
import { fillCache } from "./generate/FillCache.js";
import { generateDashboard } from "./generate/GenerateDashboard.js";
import { LocalData } from "./generate/LocalData.js";
import { dirname } from "./util/Dirname.js";
import { findConfigFile } from "./util/FindConfigFile.js";
import { reportHalt } from "./util/Report.js";
var __dirname = dirname(import.meta.url);
var require2 = createRequire(import.meta.url);
var alineaPackageDir = path.dirname(require2.resolve("alinea/package.json"));
async function generatePackage(context, cms) {
  const { config } = cms;
  if (!config.dashboardFile)
    return;
  const staticFile = config.dashboardFile ? join(config.publicDir, config.dashboardFile) : void 0;
  if (!staticFile)
    return;
  await generateDashboard(
    context,
    cms,
    config.handlerUrl ?? "/api/cms",
    staticFile
  );
  return basename(staticFile);
}
async function createDb() {
  const { default: sqlite } = await import("@alinea/sqlite-wasm");
  const { Database } = await sqlite();
  const { connect } = await import("../chunks/sql.js-J6L75I65.js");
  const db = new Database();
  const store = connect(db);
  return [store, () => db.export()];
}
async function* generate(options) {
  const {
    cmd,
    wasmCache = false,
    cwd = process.cwd(),
    configFile,
    staticDir = path.join(__dirname, "static"),
    quiet = false,
    onAfterGenerate
  } = options;
  const now = performance.now();
  const configLocation = configFile ? path.join(path.resolve(cwd), configFile) : findConfigFile(cwd);
  if (!configLocation)
    throw new Error(`No config file specified`);
  const location = path.relative(process.cwd(), configLocation).replace(/\\/g, "/");
  const rootDir = path.resolve(cwd);
  const configDir = path.dirname(configLocation);
  const nodeModules = alineaPackageDir.includes("node_modules") ? path.join(alineaPackageDir, "..") : path.join(alineaPackageDir, "node_modules");
  const context = {
    cmd,
    wasmCache,
    rootDir,
    staticDir,
    quiet,
    configDir,
    configLocation,
    fix: options.fix || false,
    outDir: path.join(nodeModules, "@alinea/generated")
  };
  await copyStaticFiles(context);
  let indexing;
  const builder = compileConfig(context);
  const builds = genEffect(builder, () => indexing?.return());
  let afterGenerateCalled = false;
  function writeStore(data) {
    return exportStore(data, join(context.outDir, "store.js"));
  }
  const [store, storeData] = await createDb();
  for await (const cms of builds) {
    if (cmd === "build") {
      const handlerUrl = cms.config.handlerUrl;
      const baseUrl = Config.baseUrl(cms.config, "production");
      if (handlerUrl && !baseUrl) {
        reportHalt(
          "No baseUrl was set for the production build in Alinea config"
        );
        process.exit(1);
      }
    }
    const write = async () => {
      let dbSize = 0;
      if (cmd === "build") {
        ;
        [, dbSize] = await Promise.all([
          generatePackage(context, cms),
          writeStore(storeData())
        ]);
      } else {
        await writeStore(new Uint8Array());
      }
      let message = `${cmd} ${location} in `;
      const duration = performance.now() - now;
      if (duration > 1e3)
        message += `${(duration / 1e3).toFixed(2)}s`;
      else
        message += `${duration.toFixed(0)}ms`;
      if (dbSize > 0)
        message += ` (db ${prettyBytes(dbSize)})`;
      return message;
    };
    const fileData = new LocalData({
      config: cms.config,
      fs: fs.promises,
      rootDir,
      dashboardUrl: await options.dashboardUrl
    });
    indexing = fillCache(context, fileData, store, cms.config);
    for await (const db of indexing) {
      yield { cms, db, localData: fileData };
      if (onAfterGenerate && !afterGenerateCalled)
        await write().then(
          (message) => {
            afterGenerateCalled = true;
            onAfterGenerate(message);
          },
          () => {
            reportHalt("Alinea failed to write dashboard files");
            if (cmd === "build")
              process.exit(1);
          }
        );
    }
  }
}
export {
  generate
};
