import {
  package_default
} from "../chunks/chunk-7EETFPPX.js";
import "../chunks/chunk-U5RRZUYZ.js";

// src/cli/Serve.ts
import { createHandler } from "alinea/backend/Handler";
import { gitUser } from "alinea/backend/util/ExecGit";
import { cloudBackend } from "alinea/cloud/CloudBackend";
import { cloudDebug } from "alinea/cloud/CloudDebug";
import { genEffect } from "alinea/core/util/Async";
import path from "node:path";
import { generate } from "./Generate.js";
import { buildOptions } from "./build/BuildOptions.js";
import { createLocalServer } from "./serve/CreateLocalServer.js";
import { GitHistory } from "./serve/GitHistory.js";
import { LiveReload } from "./serve/LiveReload.js";
import { localAuth } from "./serve/LocalAuth.js";
import { MemoryDrafts } from "./serve/MemoryDrafts.js";
import { startNodeServer } from "./serve/StartNodeServer.js";
import { dirname } from "./util/Dirname.js";
import { findConfigFile } from "./util/FindConfigFile.js";
import { bold, cyan, gray, reportHalt } from "./util/Report.js";
var __dirname = dirname(import.meta.url);
async function serve(options) {
  const {
    cwd = process.cwd(),
    base,
    configFile,
    staticDir = path.join(__dirname, "static"),
    alineaDev = false,
    production = false,
    cmd
  } = options;
  const configLocation = configFile ? path.join(path.resolve(cwd), configFile) : findConfigFile(cwd);
  if (!configLocation) {
    reportHalt(`No Alinea config file found @ ${cwd}`);
    process.exit(1);
  }
  const preferredPort = options.port ? Number(options.port) : 4500;
  const server = startNodeServer(preferredPort, 0, cmd === "build");
  const dashboardUrl = server.then((server2) => `http://localhost:${server2.port}`);
  const rootDir = path.resolve(cwd);
  const context = {
    cmd,
    configLocation,
    rootDir,
    base,
    staticDir,
    alineaDev,
    buildOptions: {
      ...buildOptions,
      ...options.buildOptions,
      plugins: (buildOptions.plugins || []).concat(
        options.buildOptions?.plugins || []
      )
    },
    production,
    liveReload: new LiveReload()
  };
  const drafts = new MemoryDrafts();
  let currentCMS;
  let serveController = new AbortController();
  let localServer;
  const fileEmitter = generate({
    ...options,
    dashboardUrl,
    watch: cmd === "dev",
    onAfterGenerate(msg) {
      dashboardUrl.then((url) => {
        const version = gray(package_default.version);
        const header = `  ${cyan(bold("\u0251"))} Alinea ${version}
`;
        const connector = gray(cmd === "dev" ? "\u251C" : "\u2570");
        const details = `  ${connector} ${gray(msg)}
`;
        const footer = cmd === "dev" ? `  ${gray("\u2570")} Local CMS:    ${url}

` : "\n";
        process.stdout.write(header + details + footer);
        options.onAfterGenerate?.({
          ALINEA_DEV_SERVER: url
        });
      });
    }
  });
  const generateFiles = genEffect(fileEmitter, () => {
    serveController.abort();
    serveController = new AbortController();
  });
  const user = await gitUser(rootDir);
  for await (const { cms, localData: fileData, db } of generateFiles) {
    if (currentCMS === cms) {
      context.liveReload.reload("refetch");
    } else {
      let createBackend2 = function() {
        if (process.env.ALINEA_CLOUD_DEBUG)
          return cloudDebug(cms.config, rootDir);
        if (process.env.ALINEA_CLOUD_URL)
          return cloudBackend(cms.config);
        return {
          auth: localAuth(rootDir),
          target: fileData,
          media: fileData,
          drafts,
          history
        };
      };
      var createBackend = createBackend2;
      const history = new GitHistory(cms.config, rootDir);
      const backend = createBackend2();
      const handleApi = createHandler(cms, backend, Promise.resolve(db));
      if (localServer)
        localServer.close();
      localServer = createLocalServer(context, cms, handleApi, user);
      currentCMS = cms;
    }
    const { serve: serve2 } = await server;
    for await (const { request, respondWith } of serve2(serveController)) {
      localServer.handle(request).then(respondWith);
    }
  }
  const { close } = await server;
  close();
}
export {
  serve
};
