import "../../chunks/chunk-U5RRZUYZ.js";

// src/cli/serve/GitHistory.ts
import { JsonLoader } from "alinea/backend/loader/JsonLoader";
import { execGit } from "alinea/backend/util/ExecGit";
var encoder = new TextEncoder();
var GitHistory = class {
  constructor(config, rootDir) {
    this.config = config;
    this.rootDir = rootDir;
  }
  async list(ctx, file) {
    const output = await execGit(this.rootDir, [
      "log",
      "--follow",
      "--name-status",
      "--pretty=format:%H%n%at%n%s%n%ae%n%an%n%f",
      "--",
      file
    ]);
    const revisions = output.split("\n\n").map((entry) => {
      const [hash, timestamp, message, email, name, changedFile, ...rest] = entry.split("\n");
      const fileLocation = rest.length ? rest[rest.length - 1].split("	").pop().trim() : file;
      return {
        ref: hash,
        createdAt: parseInt(timestamp) * 1e3,
        description: message,
        file: fileLocation,
        user: {
          sub: email,
          name
        }
      };
    });
    return revisions;
  }
  async revision(ctx, file, ref) {
    const { config } = this;
    const data = await execGit(this.rootDir, [
      "show",
      `${ref}:${file}`,
      "--format=%B"
    ]);
    try {
      return JsonLoader.parse(config.schema, encoder.encode(data));
    } catch (cause) {
      throw new Error(`Failed to parse revision ${ref} of ${file}`, {
        cause
      });
    }
  }
};
export {
  GitHistory
};
