import "../../chunks/chunk-U5RRZUYZ.js";

// src/cli/serve/StartNodeServer.ts
import { fromNodeRequest, respondTo } from "alinea/backend/router/NodeHandler";
import http from "node:http";
import { createEmitter } from "../util/Emitter.js";
async function startNodeServer(port = 4500, attempt = 0, silent = false) {
  const messages = createEmitter();
  function serve(incoming, outgoing) {
    messages.emit({
      request: fromNodeRequest(incoming),
      respondWith(response) {
        return respondTo(outgoing, response);
      }
    });
  }
  return new Promise((resolve, reject) => {
    const server = http.createServer(serve);
    server.on("error", reject);
    server.on("listening", () => resolve(server));
    server.on("close", () => messages.return());
    server.listen(port);
  }).then((server) => {
    return {
      port,
      close() {
        server.close();
      },
      async *serve(abortController) {
        if (abortController)
          abortController.signal.addEventListener(
            "abort",
            () => messages.return(),
            true
          );
        try {
          yield* messages;
        } catch (e) {
          throw e;
        }
      }
    };
  }).catch((err) => {
    if (attempt > 10)
      throw err;
    const incrementedPort = port + 1;
    if (err.code === "EADDRINUSE" && incrementedPort < 65535) {
      if (!silent)
        console.info(
          `> Port ${port} is in use, attempting ${incrementedPort} instead`
        );
      return startNodeServer(incrementedPort, attempt++, silent);
    }
    throw err;
  });
}
export {
  startNodeServer
};
