import "../chunks/chunk-U5RRZUYZ.js";

// src/cloud/CloudDebug.ts
import { gitUser } from "alinea/backend/util/ExecGit";
import { GitHistory } from "alinea/cli/serve/GitHistory";
import { createId } from "alinea/core/Id";
var latency = 0;
var lag = (ms) => new Promise((resolve) => setTimeout(resolve, ms));
function cloudDebug(config, rootDir) {
  const draftCache = /* @__PURE__ */ new Map();
  const mutations = [];
  const user = gitUser(rootDir);
  const gitHistory = new GitHistory(config, rootDir);
  const auth = {
    async authenticate(ctx, request) {
      return new Response("ok");
    },
    async verify(ctx, request) {
      return { ...ctx, user: await user, token: "dev" };
    }
  };
  const target = {
    async mutate(ctx, params) {
      await lag(latency);
      for (const mutation of params.mutations) {
        console.info(
          `> cloud: mutate ${mutation.meta.type} - ${mutation.meta.entryId}`
        );
      }
      const toCommitHash = createId();
      mutations.push({ ...params, toCommitHash });
      console.info(`> cloud: current ${toCommitHash}`);
      return { commitHash: toCommitHash };
    }
  };
  const media = {
    prepareUpload(ctx, file) {
      throw new Error(`Not implemented`);
    }
  };
  const drafts = {
    async get(ctx, entryId) {
      await lag(latency);
      return draftCache.get(entryId);
    },
    async store(ctx, draft) {
      await lag(latency);
      console.info(`> cloud: store draft ${draft.entryId}`);
      draftCache.set(draft.entryId, draft);
    }
  };
  const history = {
    async list(ctx, file) {
      await lag(latency);
      return gitHistory.list(ctx, file);
    },
    async revision(ctx, file, ref) {
      await lag(latency);
      return gitHistory.revision(ctx, file, ref);
    }
  };
  const pending = {
    async since(ctx, commitHash) {
      await lag(latency);
      console.info(`> cloud: pending since ${commitHash}`);
      let i = mutations.length;
      for (; i >= 0; i--)
        if (i > 0 && mutations[i - 1].toCommitHash === commitHash)
          break;
      const pending2 = mutations.slice(i);
      if (pending2.length === 0)
        return void 0;
      return {
        toCommitHash: pending2[pending2.length - 1].toCommitHash,
        mutations: pending2.flatMap(
          (params) => params.mutations.flatMap((mutate) => mutate.meta)
        )
      };
    }
  };
  return { auth, target, media, drafts, history, pending };
}
export {
  cloudDebug
};
