import { Revision } from 'alinea/backend/Backend';
import { PreviewInfo } from 'alinea/backend/Previews';
import { Connection, SyncResponse } from './Connection.js';
import { Draft } from './Draft.js';
import { EntryRecord } from './EntryRecord.js';
import { Mutation } from './Mutation.js';
import { ResolveParams } from './Resolver.js';
import { User } from './User.js';
export type AuthenticateRequest = (request?: RequestInit) => RequestInit | undefined;
export interface ClientOptions {
    url: string;
    applyAuth?: AuthenticateRequest;
    unauthorized?: () => void;
}
export declare class Client implements Connection {
    #private;
    constructor(options: ClientOptions);
    get url(): string;
    previewToken(request: PreviewInfo): Promise<string>;
    prepareUpload(file: string): Promise<Connection.UploadResponse>;
    user(): Promise<User | undefined>;
    resolve(params: ResolveParams): Promise<unknown>;
    mutate(mutations: Array<Mutation>): Promise<{
        commitHash: string;
    }>;
    authenticate(applyAuth: AuthenticateRequest, unauthorized: () => void): Client;
    revisions(file: string): Promise<Array<Revision>>;
    revisionData(file: string, revisionId: string): Promise<EntryRecord | undefined>;
    syncRequired(contentHash: string): Promise<boolean>;
    sync(contentHashes: Array<string>): Promise<SyncResponse>;
    getDraft(entryId: string): Promise<Draft | undefined>;
    storeDraft(draft: Draft): Promise<void>;
}
