import { CloudAuthView } from 'alinea/cloud/view/CloudAuth';
import { Preview } from 'alinea/core/Preview';
import { Auth } from './Auth.js';
import { Schema } from './Schema.js';
import { Type } from './Type.js';
import { Workspace, WorkspaceData } from './Workspace.js';
/** Configuration options */
export interface Config {
    /** A schema describing the types of entries */
    schema: Schema;
    /** A record containing workspace configurations */
    workspaces: Record<string, Workspace>;
    /** A url which will be embedded in the dashboard for live previews */
    preview?: Preview;
    /** Every edit will pass through a draft phase before being published */
    enableDrafts?: boolean;
    /** The interval in seconds at which the frontend will poll for updates */
    syncInterval?: number;
    /** The base url of the application */
    baseUrl?: string | {
        development?: string;
        production?: string;
    };
    /** The url of the handler endpoint */
    handlerUrl?: string;
    /** The folder where public assets are stored, defaults to /public */
    publicDir?: string;
    /** Filename of the generated dashboard */
    dashboardFile?: string;
    /** @deprecated Use the publicDir and dashboardFile settings */
    dashboard?: never;
    auth?: Auth.View;
}
export declare namespace Config {
    function baseUrl(config: Config, env?: "development" | "production"): string | undefined;
    function mainWorkspace(config: Config): WorkspaceData;
    function type(config: Config, name: string): Type | undefined;
    function hasAuth(config: Config): boolean;
    function validate(config: Config): void;
    function referencedViews(config: Config): Array<string>;
}
/** Create a new config instance */
export declare function createConfig<Definition extends Config>(definition: Definition): Definition & {
    publicDir: string;
    schema: {
        MediaLibrary: Type<{
            title: import("alinea/field/hidden").HiddenField<string>;
            path: import("alinea/field/hidden").HiddenField<string>;
        }>;
        MediaFile: Type<{
            title: import("alinea/field/text").TextField;
            path: import("alinea/field/hidden").HiddenField<string>;
            location: import("alinea/field/hidden").HiddenField<string>;
            extension: import("alinea/field/hidden").HiddenField<string>;
            size: import("alinea/field/hidden").HiddenField<number>;
            hash: import("alinea/field/hidden").HiddenField<string>;
            width: import("alinea/field/hidden").HiddenField<number>;
            height: import("alinea/field/hidden").HiddenField<number>;
            preview: import("alinea/field/hidden").HiddenField<string>;
            averageColor: import("alinea/field/hidden").HiddenField<string>;
            focus: import("alinea/field/hidden").HiddenField<{
                x: number;
                y: number;
            }>;
            thumbHash: import("alinea/field/hidden").HiddenField<string>;
        }>;
    };
    auth: typeof CloudAuthView;
};
