import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Config.ts
import { CloudAuthView } from "alinea/cloud/view/CloudAuth";
import { MediaFile, MediaLibrary } from "alinea/core/media/MediaTypes";
import { Schema } from "./Schema.js";
import { Workspace } from "./Workspace.js";
import { isValidIdentifier } from "./util/Identifiers.js";
import { entries, values } from "./util/Objects.js";
var Config;
((Config2) => {
  function baseUrl(config, env = process.env.NODE_ENV ?? "production") {
    const result = typeof config.baseUrl === "object" ? config.baseUrl[env] : config.baseUrl;
    if (!result)
      return result;
    if (result.includes("://"))
      return result;
    return `https://${result}`;
  }
  Config2.baseUrl = baseUrl;
  function mainWorkspace(config) {
    const key = Object.keys(config.workspaces)[0];
    return Workspace.data(config.workspaces[key]);
  }
  Config2.mainWorkspace = mainWorkspace;
  function type(config, name) {
    return config.schema[name];
  }
  Config2.type = type;
  function hasAuth(config) {
    return Boolean(config.auth);
  }
  Config2.hasAuth = hasAuth;
  function validate(config) {
    Schema.validate(config.schema);
    for (const [key, workspace] of entries(config.workspaces)) {
      if (!isValidIdentifier(key))
        throw new Error(
          `Invalid Workspace name "${key}", use only a-z, A-Z, 0-9, and _`
        );
      Workspace.validate(workspace, config.schema);
    }
  }
  Config2.validate = validate;
  function referencedViews(config) {
    return Schema.referencedViews(config.schema).concat(
      values(config.workspaces).flatMap(Workspace.referencedViews)
    );
  }
  Config2.referencedViews = referencedViews;
})(Config || (Config = {}));
function createConfig(definition) {
  if (definition.schema.MediaFile && definition.schema.MediaFile !== MediaFile)
    throw new Error(`"MediaFile" is a reserved Type name`);
  if (definition.schema.MediaLibrary && definition.schema.MediaLibrary !== MediaLibrary)
    throw new Error(`"MediaLibrary" is a reserved Type name`);
  const res = {
    ...definition,
    publicDir: definition.publicDir ?? "/public",
    schema: { ...definition.schema, MediaLibrary, MediaFile },
    auth: CloudAuthView
  };
  Config.validate(res);
  return res;
}
export {
  Config,
  createConfig
};
