import { LinkResolver } from 'alinea/backend/resolver/LinkResolver';
import { Expr } from 'alinea/core/pages/Expr';
import { Shape } from './Shape.js';
import { View } from './View.js';
export interface FieldOptions<StoredValue> {
    /** A description of the field */
    label: string;
    /** Hide this field in the dashboard */
    hidden?: boolean;
    /** Mark this field as read-only */
    readOnly?: boolean;
    /** The initial value of the field */
    initialValue?: StoredValue;
    /** The value of this field is shared across all languages */
    shared?: boolean;
    /** Providing a value for this field is required */
    required?: boolean;
    /** Validate the given value */
    validate?(value: StoredValue): boolean | string | undefined;
}
export type WithoutLabel<Options extends FieldOptions<any>> = Omit<Options, 'label'>;
export interface FieldMeta<StoredValue, QueryValue, Mutator, Options extends FieldOptions<StoredValue>> {
    options: Options;
    view: View<{
        field: Field<StoredValue, QueryValue, Mutator, Options>;
    }>;
    postProcess?: (value: StoredValue, loader: LinkResolver) => Promise<void>;
}
export interface FieldData<StoredValue, QueryValue, Mutator, Options extends FieldOptions<StoredValue>> extends FieldMeta<StoredValue, QueryValue, Mutator, Options> {
    shape: Shape<StoredValue, Mutator>;
    referencedViews: Array<string>;
}
export interface Field<StoredValue, QueryValue, Mutator, Options extends FieldOptions<StoredValue>> extends Expr<QueryValue> {
    [Field.Data]: FieldData<StoredValue, QueryValue, Mutator, Options>;
    [Field.Ref]: symbol;
}
export declare class Field<StoredValue = any, QueryValue = any, Mutator = any, Options extends FieldOptions<StoredValue> = FieldOptions<StoredValue>> {
    static index: number;
    constructor(data: FieldData<StoredValue, QueryValue, Mutator, Options>);
}
export declare namespace Field {
    const Data: unique symbol;
    const Value: unique symbol;
    const Ref: unique symbol;
    function ref(field: Field): symbol;
    function shape(field: Field<any, any>): Shape;
    function label(field: Field): string;
    function view<StoredValue, QueryValue, Mutator, Options extends FieldOptions<StoredValue>>(field: Field<StoredValue, QueryValue, Mutator, Options>): View<{
        field: Field<StoredValue, QueryValue, Mutator, Options>;
    }>;
    function referencedViews(field: Field): Array<string>;
    function options<StoredValue, QueryValue, Options extends FieldOptions<StoredValue>>(field: Field<StoredValue, QueryValue, any, Options>): Options;
    function isField(value: any): value is Field;
}
