import { Config } from './Config.js';
import { Entry } from './Entry.js';
import { PageSeed } from './Page.js';
import { ResolveRequest, Resolver } from './Resolver.js';
import { Root } from './Root.js';
import { Type } from './Type.js';
import { Workspace } from './Workspace.js';
import { Cursor } from './pages/Cursor.js';
import type { Projection } from './pages/Projection.js';
import { Selection } from './pages/ResolveData.js';
export type Location = Root | Workspace | PageSeed;
export interface GraphRealmApi {
    /** Filter results by location */
    in(location: Location): GraphRealmApi;
    /** Filter results by locale */
    locale(locale: string): GraphRealmApi;
    /** Find a single entry or null */
    maybeGet<S extends Projection | Type>(select: S): Promise<Projection.InferOne<S> | null>;
    /** Preview an entry */
    previewEntry(entry: Entry): GraphRealmApi;
    /** Find a single entry */
    get<S extends Projection | Type>(select: S): Promise<Projection.InferOne<S>>;
    /** Find a set of entries */
    find<S extends Projection | Type>(select: S): Promise<Selection.Infer<S>>;
    /** The time in seconds to poll for updates to content */
    syncInterval(interval: number): GraphRealmApi;
    /** Disable polling for updates to content */
    disableSync(): GraphRealmApi;
    /** The amount of results found */
    count(cursor: Cursor.Find<any>): Promise<number>;
}
export interface GraphOrigin {
    location?: Location;
    locale?: string;
}
export declare class GraphRealm implements GraphRealmApi {
    #private;
    constructor(config: Config, resolver: Resolver, params?: Partial<ResolveRequest>);
    disableSync(): GraphRealm;
    syncInterval(interval: number): GraphRealm;
    in(location: Location): GraphRealmApi;
    locale(locale: string): GraphRealm;
    previewEntry(entry: Entry): GraphRealmApi;
    maybeGet<S extends Projection | Type>(select: S): Promise<Projection.InferOne<S> | null>;
    get<S extends Projection | Type>(select: S): Promise<Projection.InferOne<S>>;
    find<S extends Projection | Type>(select: S): Promise<Selection.Infer<S>>;
    count(cursor: Cursor.Find<any>): Promise<number>;
}
export declare class Graph {
    config: Config;
    resolver: Resolver;
    drafts: GraphRealm;
    archived: GraphRealm;
    published: GraphRealm;
    preferPublished: GraphRealm;
    preferDraft: GraphRealm;
    all: GraphRealm;
    constructor(config: Config, resolver: Resolver);
}
