import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Graph.ts
import { Schema } from "./Schema.js";
import { Type } from "./Type.js";
import { createSelection } from "./pages/CreateSelection.js";
import { Cursor } from "./pages/Cursor.js";
import { Realm } from "./pages/Realm.js";
import { seralizeLocation, serializeSelection } from "./pages/Serialize.js";
var GraphRealm = class _GraphRealm {
  #resolver;
  #config;
  #targets;
  #params;
  constructor(config, resolver, params) {
    this.#config = config;
    this.#resolver = resolver;
    this.#targets = Schema.targets(config.schema);
    this.#params = { ...params };
  }
  disableSync() {
    return new _GraphRealm(this.#config, this.#resolver, {
      ...this.#params,
      syncInterval: Infinity
    });
  }
  syncInterval(interval) {
    return new _GraphRealm(this.#config, this.#resolver, {
      ...this.#params,
      syncInterval: interval
    });
  }
  in(location) {
    return new _GraphRealm(this.#config, this.#resolver, {
      ...this.#params,
      location: seralizeLocation(this.#config, location)
    });
  }
  locale(locale) {
    return new _GraphRealm(this.#config, this.#resolver, {
      ...this.#params,
      locale
    });
  }
  previewEntry(entry) {
    return new _GraphRealm(this.#config, this.#resolver, {
      ...this.#params,
      preview: { entry }
    });
  }
  async maybeGet(select) {
    if (select instanceof Cursor.Find)
      select = select.first();
    if (Type.isType(select))
      select = select().first();
    const selection = createSelection(select);
    serializeSelection(this.#targets, selection);
    return await this.#resolver.resolve({
      ...this.#params,
      selection
    }) ?? null;
  }
  async get(select) {
    const result = await this.maybeGet(select);
    if (result === null)
      throw new Error("Entry not found");
    return result;
  }
  async find(select) {
    if (Type.isType(select))
      select = select();
    const selection = createSelection(select);
    serializeSelection(this.#targets, selection);
    return this.#resolver.resolve({
      ...this.#params,
      selection
    });
  }
  async count(cursor) {
    const selection = createSelection(cursor.count());
    serializeSelection(this.#targets, selection);
    return this.#resolver.resolve({
      ...this.#params,
      selection
    });
  }
};
var Graph = class {
  constructor(config, resolver) {
    this.config = config;
    this.resolver = resolver;
    this.drafts = new GraphRealm(this.config, resolver, { realm: Realm.Draft });
    this.archived = new GraphRealm(this.config, resolver, {
      realm: Realm.Archived
    });
    this.published = new GraphRealm(this.config, resolver, {
      realm: Realm.Published
    });
    this.preferDraft = new GraphRealm(this.config, resolver, {
      realm: Realm.PreferDraft
    });
    this.preferPublished = new GraphRealm(this.config, resolver, {
      realm: Realm.PreferPublished
    });
    this.all = new GraphRealm(this.config, resolver, { realm: Realm.All });
  }
  drafts;
  archived;
  published;
  preferPublished;
  preferDraft;
  all;
};
export {
  Graph,
  GraphRealm
};
