import { Entry } from './Entry.js';
import { Type } from './Type.js';
import { Cursor } from './pages/Cursor.js';
import { EV, Expr } from './pages/Expr.js';
import type { Projection } from './pages/Projection.js';
import { snippet as snippetHelper } from './pages/Snippet.js';
export declare function Query<Definition>(type: Type<Definition>): Cursor.Typed<Definition>;
export declare namespace Query {
    const and: typeof Expr.and;
    const or: typeof Expr.or;
    const not: <T>(a: EV<T>) => Expr<boolean>;
    const is: <T>(a: EV<T>, b: EV<T>) => Expr<boolean>;
    const isNot: <T>(a: EV<T>, b: EV<T>) => Expr<boolean>;
    const isNull: <T>(a: EV<T>) => Expr<boolean>;
    const isNotNull: <T>(a: EV<T>) => Expr<boolean>;
    const isIn: <T>(a: EV<T>, b: EV<ReadonlyArray<T>> | Cursor.Find<T>) => Expr<boolean>;
    const isNotIn: <T>(a: EV<T>, b: EV<ReadonlyArray<T>> | Cursor.Find<T>) => Expr<boolean>;
    const isGreater: <T>(a: EV<T>, b: EV<T>) => Expr<boolean>;
    const isGreaterOrEqual: <T>(a: EV<T>, b: EV<T>) => Expr<boolean>;
    const isLess: <T>(a: EV<T>, b: EV<T>) => Expr<boolean>;
    const isLessOrEqual: <T>(a: EV<T>, b: EV<T>) => Expr<boolean>;
    const add: (a: EV<number>, b: EV<number>, ...rest: Array<EV<number>>) => Expr<number>;
    const subtract: (a: EV<number>, b: EV<number>, ...rest: Array<EV<number>>) => Expr<number>;
    const multiply: (a: EV<number>, b: EV<number>, ...rest: Array<EV<number>>) => Expr<number>;
    const divide: (a: EV<number>, b: EV<number>, ...rest: Array<EV<number>>) => Expr<number>;
    const remainder: (a: EV<number>, b: EV<number>) => Expr<number>;
    const concat: (a: EV<string>, b: EV<string>) => Expr<string>;
    const like: (a: EV<string>, b: EV<string>) => Expr<boolean>;
    const at: <T>(a: EV<Array<T>>, index: number) => Expr<T | null>;
    const includes: <T>(a: EV<Array<T>>, value: EV<T>) => Expr<boolean>;
    const url: Expr<string>;
    const path: Expr<string>;
    const title: Expr<string>;
    const id: Expr<string>;
    const parent: Expr<string | null>;
    const type: Expr<string>;
    const workspace: Expr<string>;
    const root: Expr<string>;
    const locale: Expr<string | null>;
    const level: Expr<number>;
    const entry: {
        id: Expr<string>;
        type: Expr<string>;
        workspace: Expr<string>;
        root: Expr<string>;
        locale: Expr<string | null>;
        path: Expr<string>;
        title: Expr<string>;
        url: Expr<string>;
        parent: Expr<string | null>;
        level: Expr<number>;
    };
    const where: (...where: Array<EV<boolean>>) => Cursor.Find<Entry>;
    const whereId: (id: string) => Cursor.Find<Entry>;
    const whereUrl: (url: string) => Cursor.Find<Entry>;
    const wherePath: (path: string) => Cursor.Find<Entry>;
    const whereParent: (parentId: string) => Cursor.Find<Entry>;
    const whereLocale: (locale: string) => Cursor.Find<Entry>;
    const whereWorkspace: (workspace: string) => Cursor.Find<Entry>;
    const whereRoot: (root: string) => Cursor.Find<Entry>;
    const whereType: (type: string) => Cursor.Find<Entry>;
    const select: <S extends Projection>(select: S) => Cursor.Find<Projection.Infer<S>>;
    const search: (...searchTerms: Array<string>) => Cursor.Find<Entry>;
    const snippet: typeof snippetHelper;
    function children<Definition>(type?: Type<Definition>, depth?: number): Cursor.Find<Type.Infer<Definition>>;
    function parents<Definition>(): Cursor.Find<Type.Infer<Definition>>;
    function next<Definition>(type?: Type<Definition>): Cursor.Get<Type.Infer<Definition>>;
    function previous<Definition>(type?: Type<Definition>): Cursor.Get<Type.Infer<Definition>>;
    function siblings<Definition>(type?: Type<Definition>): Cursor.Find<Type.Infer<Definition>>;
    function translations<Definition>(type?: Type<Definition>, includeSelf?: boolean): Cursor.Find<Type.Infer<Definition>>;
}
