import { Preview } from 'alinea/core/Preview';
import type { ComponentType } from 'react';
import { Label } from './Label.js';
import { Meta, StripMeta } from './Meta.js';
import { PageSeed } from './Page.js';
import { Schema } from './Schema.js';
import { Type } from './Type.js';
import { View } from './View.js';
export interface RootI18n {
    locales: Array<string>;
}
export interface RootMeta {
    contains?: Array<string | Type>;
    icon?: ComponentType;
    i18n?: RootI18n;
    /** Point to a React component used to view this root in the dashboard */
    view?: View<{
        root: RootData;
    }>;
    isMediaRoot?: boolean;
    preview?: Preview;
}
export interface RootDefinition {
    [key: string]: PageSeed;
    [Meta]?: RootMeta;
}
export interface RootData extends RootMeta {
    label: string;
}
type Seed = Record<string, PageSeed>;
export type Root<Definition extends Seed = Seed> = Definition & {
    [Root.Data]: RootData;
};
export declare namespace Root {
    const Data: unique symbol;
    function label(root: Root): Label;
    function data(root: Root): RootData;
    function preview(root: Root): Preview | undefined;
    function defaultLocale(root: Root): string | undefined;
    function isRoot(value: any): value is Root;
    function isMediaRoot(root: Root): boolean;
    function validate(root: Root, workspaceLabel: string, schema: Schema): void;
    function referencedViews(root: Root): Array<string>;
}
export interface RootOptions<Definition> extends RootMeta {
    entries?: Definition;
}
export declare function root<Definition extends RootDefinition>(label: Label): Root<{}>;
export declare function root<Definition extends RootDefinition>(label: Label, definition: RootOptions<Definition>): Root<Definition>;
/** @deprecated See https://github.com/alineacms/alinea/issues/373 */
export declare function root<Definition extends RootDefinition>(label: Label, definition: Definition): Root<StripMeta<Definition>>;
export declare namespace root {
    const meta: typeof Meta;
}
export {};
