import { File } from '@alinea/iso';
import { StoredRow } from 'alinea/core/Infer';
import { ImagePreviewDetails } from 'alinea/core/media/CreatePreview';
import type { CMS } from './CMS.js';
import { Config } from './Config.js';
import { Entry } from './Entry.js';
import { EntryRow } from './EntryRow.js';
import { Mutation } from './Mutation.js';
import { Type, TypeI } from './Type.js';
export interface Transaction {
    (cms: CMS): Promise<Array<Mutation>>;
}
export interface Operation {
    [Operation.Data]: Transaction;
}
export declare class Operation {
    static readonly Data: unique symbol;
    constructor(tx: Transaction);
    protected typeName(config: Config, type: TypeI): string;
}
export interface UploadOptions {
    createPreview?(blob: Blob): Promise<ImagePreviewDetails>;
}
export declare class UploadOperation extends Operation {
    entryId: string;
    private parentId?;
    private workspace?;
    private root?;
    constructor(file: File | [string, Uint8Array], options?: UploadOptions);
    setParent(parentId: string): this;
    setWorkspace(workspace: string): this;
    setRoot(root: string): this;
}
export declare class DeleteOp extends Operation {
    protected entryId: string;
    constructor(entryId: string);
}
export declare class EditOperation<Definition> extends Operation {
    protected entryId: string;
    private entryData?;
    private changePhase?;
    constructor(entryId: string);
    set(entryData: Partial<StoredRow<Definition>>): this;
    draft(): this;
    archive(): this;
    publish(): this;
}
export declare class CreateOperation<Definition> extends Operation {
    private type?;
    private parentRow?;
    /** @internal */
    entry: Partial<Entry>;
    private entryRow;
    constructor(entry: Partial<Entry>, type?: Type<Definition> | undefined, parentRow?: ((cms: CMS) => Promise<EntryRow>) | undefined);
    setParent(parentId: string): this;
    setWorkspace(workspace: string): this;
    setRoot(root: string): this;
    setLocale(locale: string | null): this;
    set(entryData: Partial<StoredRow<Definition>>): this;
    createChild<Definition>(type: Type<Definition>): CreateOperation<Definition>;
    get entryId(): string;
    static fromType<Definition>(type: Type<Definition>): CreateOperation<Definition>;
}
