import {
  object,
  string
} from "../chunks/chunk-AOATU4QL.js";
import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Type.ts
import { Field } from "./Field.js";
import { Meta } from "./Meta.js";
import { Cursor } from "./pages/Cursor.js";
import { Expr, createExprData } from "./pages/Expr.js";
import {
  BinaryOp,
  ExprData,
  Selection,
  toSelection
} from "./pages/ResolveData.js";
import { Section, section } from "./Section.js";
import { RecordShape } from "./shape/RecordShape.js";
import { isValidIdentifier } from "./util/Identifiers.js";
import {
  assign,
  defineProperty,
  entries,
  fromEntries,
  keys,
  values
} from "./util/Objects.js";
var TypeTarget = class {
};
var Type;
((Type2) => {
  Type2.Data = Symbol.for("@alinea/Type.Data");
  function label(type2) {
    return type2[Type2.Data].label;
  }
  Type2.label = label;
  function meta(type2) {
    return type2[Type2.Data].meta;
  }
  Type2.meta = meta;
  function contains(type2) {
    return meta(type2).contains ?? [];
  }
  Type2.contains = contains;
  function isHidden(type2) {
    return Boolean(meta(type2).hidden || meta(type2).isHidden);
  }
  Type2.isHidden = isHidden;
  function shape(type2) {
    return type2[Type2.Data].shape;
  }
  Type2.shape = shape;
  function toV1(type2, value) {
    return shape(type2).toV1(value);
  }
  Type2.toV1 = toV1;
  function searchableText(type2, value) {
    return shape(type2).searchableText(value).trim();
  }
  Type2.searchableText = searchableText;
  function fields(type2) {
    return type2;
  }
  Type2.fields = fields;
  function sections(type2) {
    return type2[Type2.Data].sections;
  }
  Type2.sections = sections;
  function isContainer(type2) {
    const { meta: meta2 } = type2[Type2.Data];
    return Boolean(meta2.isContainer || meta2.contains);
  }
  Type2.isContainer = isContainer;
  function target(type2) {
    return type2[Type2.Data].target;
  }
  Type2.target = target;
  function field(type2, name) {
    return type2[name];
  }
  Type2.field = field;
  function isType(type2) {
    return Boolean(type2 && type2[Type2.Data]);
  }
  Type2.isType = isType;
  function sharedData(type2, entryData) {
    const res = {};
    for (const [key, field2] of entries(fields(type2))) {
      if (Field.options(field2).shared)
        res[key] = entryData[key];
    }
    if (keys(res).length === 0)
      return void 0;
    return res;
  }
  Type2.sharedData = sharedData;
  const TypeOptions = object({
    view: string.optional,
    summaryRow: string.optional,
    summaryThumb: string.optional
  });
  function validate(type2) {
    TypeOptions(meta(type2));
    for (const [key, field2] of entries(fields(type2))) {
      if (!isValidIdentifier(key))
        throw new Error(
          `Invalid field name "${key}" in Type "${label(
            type2
          )}", must match [A-Za-z][A-Za-z0-9_]*`
        );
    }
  }
  Type2.validate = validate;
  function referencedViews(type2) {
    const { view, summaryRow, summaryThumb } = meta(type2);
    return [
      view,
      summaryRow,
      summaryThumb,
      ...viewsOfDefinition(type2[Type2.Data].definition)
    ].filter((v) => typeof v === "string");
  }
  Type2.referencedViews = referencedViews;
})(Type || (Type = {}));
function viewsOfDefinition(definition) {
  return values(definition).flatMap((value) => {
    if (Field.isField(value))
      return Field.referencedViews(value);
    if (Section.isSection(value))
      return Section.referencedViews(value).concat(
        viewsOfDefinition(Section.fields(value))
      );
    return [];
  });
}
function fieldsOfDefinition(definition) {
  return entries(definition).flatMap(([key, value]) => {
    if (Field.isField(value))
      return [[key, value]];
    if (Section.isSection(value))
      return entries(Section.fields(value));
    return [];
  });
}
var TypeInstance = class {
  constructor(label, definition, meta) {
    this.label = label;
    this.definition = definition;
    this.meta = meta;
    this.shape = new RecordShape(
      label,
      fromEntries(
        fieldsOfDefinition(definition).map(([key, field]) => {
          return [key, Field.shape(field)];
        })
      )
    );
    let current = {};
    const addCurrent = () => {
      if (keys(current).length > 0)
        this.sections.push(section({ definition: current }));
      current = {};
    };
    const seen = /* @__PURE__ */ new Map();
    function validateField(key, field) {
      const ref = Field.ref(field);
      if (!seen.has(ref))
        return seen.set(ref, key);
      const fieldLabel = Field.label(field);
      throw new Error(
        `Duplicate field "${fieldLabel}" in type "${label}", found under key "${key}" and "${seen.get(
          ref
        )}"
See: https://alinea.sh/docs/configuration/schema/type#fields-must-be-unique`
      );
    }
    for (const [key, value] of entries(definition)) {
      if (Field.isField(value)) {
        current[key] = value;
        validateField(key, value);
      } else if (Section.isSection(value)) {
        addCurrent();
        this.sections.push(value);
        for (const [key2, field] of entries(Section.fields(value))) {
          validateField(key2, field);
        }
      }
    }
    addCurrent();
    const name = label;
    const callable = {
      [name]: (...condition) => this.call(...condition)
    }[name];
    delete callable.length;
    this.target = callable;
    this.defineProperties(callable);
  }
  shape;
  sections = [];
  target;
  condition(input) {
    if (input.length === 0)
      return void 0;
    const isConditionalRecord = input.length === 1 && !Expr.isExpr(input[0]);
    const conditions = isConditionalRecord ? entries(input[0]).map(([key, value]) => {
      const field = Expr(ExprData.Field({ type: this.target }, key));
      return Expr(
        ExprData.BinOp(
          field[Expr.Data],
          BinaryOp.Equals,
          createExprData(value)
        )
      );
    }) : input.map((ev) => Expr(createExprData(ev)));
    return Expr.and(...conditions)[Expr.Data];
  }
  call(...input) {
    return new Cursor.Typed(this.target, input[0]);
  }
  field(def, name) {
    return assign(Expr(ExprData.Field({ type: this.target }, name)), {
      [Field.Data]: def[Field.Data],
      [Field.Ref]: def[Field.Ref]
    });
  }
  defineProperties(instance) {
    for (const [key, value] of fieldsOfDefinition(this.definition)) {
      defineProperty(instance, key, {
        value: this.field(value, key),
        enumerable: true,
        configurable: true
      });
    }
    defineProperty(instance, Type.Data, {
      value: this,
      enumerable: false
    });
    defineProperty(instance, toSelection, {
      value: () => {
        return Selection.Row({ type: Type.target(this.target) });
      },
      enumerable: false
    });
  }
};
function parseTypeParams(definition) {
  const def = definition;
  const isOptions = "fields" in def && !Field.isField(def.fields);
  const options = (isOptions ? def : def[Meta]) ?? {};
  const d = isOptions ? def.fields : def;
  return { definition: d, options };
}
function type(label, definition) {
  const title = typeof label === "string" ? label : "Anonymous";
  const def = typeof label === "string" ? definition : label;
  const { definition: d, options } = parseTypeParams(def);
  const instance = new TypeInstance(title, d, options);
  Type.validate(instance.target);
  return instance.target;
}
((type2) => {
  type2.meta = Meta;
})(type || (type = {}));
export {
  Type,
  TypeTarget,
  parseTypeParams,
  type
};
