import { Preview } from 'alinea/core/Preview';
import type { ComponentType } from 'react';
import { Meta, StripMeta } from './Meta.js';
import { Root } from './Root.js';
import { Schema } from './Schema.js';
export interface WorkspaceMeta {
    /** A directory which contains the json entry files */
    source: string;
    /** The directory where media files are placed in case a file backend is used */
    mediaDir?: string;
    /** The main theme color used in the dashboard */
    color?: string;
    icon?: ComponentType;
    preview?: Preview;
}
export interface WorkspaceData extends WorkspaceMeta {
    label: string;
    roots: Roots;
    color: string;
}
type Roots = Record<string, Root>;
export interface WorkspaceDefinition {
    [key: string]: Root;
    [Meta]?: WorkspaceMeta;
}
export type Workspace<Definition extends Roots = Roots> = Definition & {
    [Workspace.Data]: WorkspaceData;
};
export declare namespace Workspace {
    const Data: unique symbol;
    const Meta: unique symbol;
    function data(workspace: Workspace): WorkspaceData;
    function roots(workspace: Workspace): Roots;
    function label(workspace: Workspace): string;
    function preview(workspace: Workspace): Preview | undefined;
    function isWorkspace(value: any): value is Workspace;
    function defaultMediaRoot(workspace: Workspace): string;
    function defaultRoot(workspace: Workspace): string;
    function validate(workspace: Workspace, schema: Schema): void;
    function referencedViews(workspace: Workspace): string[];
}
export interface WorkspaceOptions<Definition> extends WorkspaceMeta {
    roots: Definition;
}
/** Create a workspace */
export declare function workspace<Definition extends WorkspaceDefinition>(
/** The name of the workspace */
label: string, definition: WorkspaceOptions<Definition>): Workspace<Definition>;
/** @deprecated See https://github.com/alineacms/alinea/issues/373 */
export declare function workspace<Definition extends WorkspaceDefinition>(
/** The name of the workspace */
label: string, definition: Definition): Workspace<StripMeta<Definition>>;
export declare namespace workspace {
    const meta: typeof Meta;
}
export {};
