import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Workspace.ts
import { Meta } from "./Meta.js";
import { Root } from "./Root.js";
import { getRandomColor } from "./util/GetRandomColor.js";
import { isValidIdentifier } from "./util/Identifiers.js";
import { entries, values } from "./util/Objects.js";
var Workspace;
((Workspace2) => {
  Workspace2.Data = Symbol.for("@alinea/Workspace.Data");
  Workspace2.Meta = Symbol.for("@alinea/Workspace.Meta");
  function data(workspace2) {
    return workspace2[Workspace2.Data];
  }
  Workspace2.data = data;
  function roots(workspace2) {
    return workspace2[Workspace2.Data].roots;
  }
  Workspace2.roots = roots;
  function label(workspace2) {
    return workspace2[Workspace2.Data].label;
  }
  Workspace2.label = label;
  function preview(workspace2) {
    return workspace2[Workspace2.Data].preview;
  }
  Workspace2.preview = preview;
  function isWorkspace(value) {
    return Boolean(value && value[Workspace2.Data]);
  }
  Workspace2.isWorkspace = isWorkspace;
  function defaultMediaRoot(workspace2) {
    const { roots: roots2 } = workspace2[Workspace2.Data];
    for (const [name, root] of entries(roots2))
      if (Root.isMediaRoot(root))
        return name;
    throw new Error(`Workspace has no media root`);
  }
  Workspace2.defaultMediaRoot = defaultMediaRoot;
  function defaultRoot(workspace2) {
    return Object.keys(workspace2[Workspace2.Data].roots)[0];
  }
  Workspace2.defaultRoot = defaultRoot;
  function validate(workspace2, schema) {
    for (const [key, root] of entries(workspace2)) {
      if (!isValidIdentifier(key))
        throw new Error(
          `Invalid Root name "${key}" in workspace "${label(
            workspace2
          )}", use only a-z, A-Z, 0-9, and _`
        );
      Root.validate(root, label(workspace2), schema);
    }
  }
  Workspace2.validate = validate;
  function referencedViews(workspace2) {
    return values(roots(workspace2)).flatMap(Root.referencedViews);
  }
  Workspace2.referencedViews = referencedViews;
})(Workspace || (Workspace = {}));
function workspace(label, definition) {
  const isOptions = "roots" in definition && !Root.isRoot(definition.roots);
  const def = definition;
  const roots = isOptions ? def.roots : def;
  const options = (isOptions ? def : def[Meta]) ?? {};
  return {
    ...roots,
    [Workspace.Data]: {
      label,
      roots,
      ...options,
      color: options.color ?? getRandomColor(JSON.stringify(label))
    }
  };
}
((workspace2) => {
  workspace2.meta = Meta;
})(workspace || (workspace = {}));
export {
  Workspace,
  workspace
};
