import { Entry } from '../Entry.js';
import type { Type } from '../Type.js';
import type { Condition } from './Expr.js';
import { EV, Expr, HasExpr } from './Expr.js';
import type { Projection } from './Projection.js';
import { CursorData, ExprData, OrderBy, Selection, SourceType, toExpr, toSelection } from './ResolveData.js';
import type { TargetI } from './Target.js';
export interface Cursor<T> {
    [Cursor.Data]: CursorData;
}
declare const brand: unique symbol;
export declare class Cursor<T> implements HasExpr<boolean> {
    [brand]: T;
    constructor(data: CursorData);
    protected with(data: Partial<CursorData>): CursorData;
    static isCursor<T>(input: any): input is Cursor<T>;
    [toExpr](): Expr<unknown>;
    [toSelection](): Selection.Cursor;
    toJSON(): {
        target?: {
            name?: string | undefined;
            type?: import("../Type.js").TypeTarget | undefined;
        } | undefined;
        where?: ExprData | undefined;
        searchTerms?: string[] | undefined;
        skip?: number | undefined;
        take?: number | undefined;
        orderBy?: {
            expr: ExprData;
            order: import("./ResolveData.js").OrderDirection;
        }[] | undefined;
        groupBy?: ExprData[] | undefined;
        select?: Selection<T_1> | undefined;
        first?: boolean | undefined;
        source?: {
            type: SourceType;
            depth?: number | undefined;
            includeSelf?: boolean | undefined;
        } | undefined;
    };
}
export declare namespace Cursor {
    const Data: unique symbol;
    class Find<Row> extends Cursor<Array<Row>> {
        where(...where: Array<Condition | boolean>): Find<Row>;
        whereId(entryId: string): Get<Row>;
        whereUrl(url: string): Find<Row>;
        wherePath(path: string): Find<Row>;
        whereParent(parentId: string): Find<Row>;
        whereLocale(locale: string): Find<Row>;
        whereRoot(root: string): Find<Row>;
        whereWorkspace(workspace: string): Find<Row>;
        search(...searchTerms: Array<string>): Find<Row>;
        get<S extends Projection>(select: S): Get<Selection.Infer<S>>;
        count(): Cursor<number>;
        first(): Get<Row>;
        maybeFirst(): Get<Row | undefined>;
        select<S extends Projection>(select: S): Find<Selection.Infer<S>>;
        groupBy(...groupBy: Array<Expr<any>>): Find<Row>;
        orderBy(...orderBy: Array<OrderBy>): Find<Row>;
        skip(skip: number): Find<Row>;
        take(take: number): Find<Row>;
    }
    class Typed<Definition> extends Find<Type.Infer<Definition>> {
        type: Type<Definition>;
        partial: Partial<Type.Infer<Definition>>;
        constructor(type: Type<Definition>, partial?: Partial<Type.Infer<Definition>>);
        where(partial: Partial<Type.Infer<Definition>>): Typed<Definition>;
        where(...where: Array<EV<boolean>>): Find<Type.Infer<Definition>>;
        static condition(type: Type, input: Record<string, any>): ExprData | undefined;
    }
    class Get<Row> extends Cursor<Row> {
        where(...where: Array<EV<boolean>>): Get<Row>;
        whereId(entryId: string): Get<Row>;
        whereUrl(url: string): Get<Row>;
        wherePath(path: string): Get<Row>;
        whereParent(parentId: string): Get<Row>;
        whereLocale(locale: string): Get<Row>;
        whereRoot(root: string): Get<Row>;
        whereWorkspace(workspace: string): Get<Row>;
        search(...searchTerms: Array<string>): Get<Row>;
        select<S extends Projection>(select: S): Get<Selection.Infer<S>>;
    }
}
type Narrow = Cursor.Find<any> | TargetI<any>;
type Output<T> = [Narrow] extends [T] ? Entry : Selection.Infer<T>;
export declare class Tree {
    constructor();
    protected narrowData(narrow?: any): Partial<CursorData>;
    protected find<T>(sourceType: SourceType, narrow?: any, extraOptions?: {
        depth?: number;
        includeSelf?: boolean;
    }): Cursor.Find<T>;
    protected get<T>(sourceType: SourceType, narrow?: any): Cursor.Get<T>;
    children<N extends Narrow>(depth?: number): Cursor.Find<Output<N>>;
    children<N extends Narrow>(narrow?: N, depth?: number): Cursor.Find<Output<N>>;
    parents<N extends Narrow>(depth?: number): Cursor.Find<Output<N>>;
    parents<N extends Narrow>(narrow?: N, depth?: number): Cursor.Find<Output<N>>;
    previous: <N extends Narrow>(narrow?: N) => Cursor.Get<Output<N>>;
    next: <N extends Narrow>(narrow?: N) => Cursor.Get<Output<N>>;
    parent: <N extends Narrow>(narrow?: N) => Cursor.Get<Output<N>>;
    siblings: <N extends Narrow>(narrow?: N) => Cursor.Find<Output<N>>;
    translations: (includeSelf?: boolean) => Cursor.Find<Entry>;
}
export {};
