import "../../chunks/chunk-U5RRZUYZ.js";

// src/core/pages/Cursor.ts
import { entries } from "../util/Objects.js";
import { createSelection } from "./CreateSelection.js";
import { Expr, createExprData } from "./Expr.js";
import {
  BinaryOp,
  ExprData,
  Selection,
  SourceType,
  targetData,
  toExpr,
  toSelection
} from "./ResolveData.js";
var Cursor = class _Cursor {
  constructor(data) {
    this[_Cursor.Data] = data;
  }
  with(data) {
    return { ...this[_Cursor.Data], ...data };
  }
  static isCursor(input) {
    return Boolean(input && input[_Cursor.Data]);
  }
  [toExpr]() {
    const { where } = this[_Cursor.Data];
    return Expr(where ?? ExprData.Value(true));
  }
  [toSelection]() {
    return Selection.Cursor(this[_Cursor.Data]);
  }
  toJSON() {
    return this[_Cursor.Data];
  }
};
((Cursor2) => {
  Cursor2.Data = Symbol.for("@alinea/Cursor.Data");
  class Find extends Cursor2 {
    where(...where) {
      const current = this[Cursor2.Data].where;
      return new Find(
        this.with({
          where: Expr.and(current ? Expr(current) : true, ...where)[Expr.Data]
        })
      );
    }
    whereId(entryId) {
      return new Get(
        this.with({
          first: true,
          where: Expr(ExprData.Field({}, "entryId")).is(entryId)[Expr.Data]
        })
      );
    }
    whereUrl(url) {
      return new Find(
        this.with({
          where: Expr(ExprData.Field({}, "url")).is(url)[Expr.Data]
        })
      );
    }
    wherePath(path) {
      return new Find(
        this.with({
          where: Expr(ExprData.Field({}, "path")).is(path)[Expr.Data]
        })
      );
    }
    whereParent(parentId) {
      return new Find(
        this.with({
          where: Expr(ExprData.Field({}, "parent")).is(parentId)[Expr.Data]
        })
      );
    }
    whereLocale(locale) {
      return new Find(
        this.with({
          where: Expr(ExprData.Field({}, "locale")).is(locale)[Expr.Data]
        })
      );
    }
    whereRoot(root) {
      return new Find(
        this.with({
          where: Expr(ExprData.Field({}, "root")).is(root)[Expr.Data]
        })
      );
    }
    whereWorkspace(workspace) {
      return new Find(
        this.with({
          where: Expr(ExprData.Field({}, "workspace")).is(workspace)[Expr.Data]
        })
      );
    }
    search(...searchTerms) {
      if (searchTerms.length === 0)
        return this;
      return new Find(this.with({ searchTerms }));
    }
    get(select) {
      const query = this.with({ first: true });
      if (select)
        query.select = createSelection(select);
      return new Get(query);
    }
    count() {
      return new Cursor2(
        this.with({ first: true, select: Selection.Count() })
      );
    }
    first() {
      return new Get(this.with({ first: true }));
    }
    maybeFirst() {
      return new Get(this.with({ first: true }));
    }
    select(select) {
      return new Find(
        this.with({ select: createSelection(select) })
      );
    }
    groupBy(...groupBy) {
      return new Find(this.with({ groupBy: groupBy.map(createExprData) }));
    }
    orderBy(...orderBy) {
      return new Find(this.with({ orderBy }));
    }
    skip(skip) {
      return new Find(this.with({ skip }));
    }
    take(take) {
      return new Find(this.with({ take }));
    }
  }
  Cursor2.Find = Find;
  class Typed extends Find {
    constructor(type, partial = {}) {
      super({
        target: { type },
        where: Typed.condition(type, partial)
      });
      this.type = type;
      this.partial = partial;
    }
    where(...input) {
      const isConditionalRecord = input.length === 1 && !Expr.isExpr(input[0]);
      const current = this[Cursor2.Data].where;
      if (isConditionalRecord) {
        return new Typed(this.type, {
          ...this.partial,
          ...input[0]
        });
      }
      return new Find(
        this.with({
          where: Expr.and(current ? Expr(current) : true, ...input)[Expr.Data]
        })
      );
    }
    static condition(type, input) {
      const conditions = entries(input || {}).map(([key, value]) => {
        const field = Expr(ExprData.Field({ type }, key));
        return Expr(
          ExprData.BinOp(
            field[Expr.Data],
            BinaryOp.Equals,
            createExprData(value)
          )
        );
      });
      return Expr.and(...conditions)[Expr.Data];
    }
  }
  Cursor2.Typed = Typed;
  class Get extends Cursor2 {
    where(...where) {
      const current = this[Cursor2.Data].where;
      return new Get(
        this.with({
          where: Expr.and(current ? Expr(current) : true, ...where)[Expr.Data]
        })
      );
    }
    whereId(entryId) {
      return new Get(
        this.with({
          where: Expr(ExprData.Field({}, "entryId")).is(entryId)[Expr.Data]
        })
      );
    }
    whereUrl(url) {
      return new Get(
        this.with({
          where: Expr(ExprData.Field({}, "url")).is(url)[Expr.Data]
        })
      );
    }
    wherePath(path) {
      return new Get(
        this.with({
          where: Expr(ExprData.Field({}, "path")).is(path)[Expr.Data]
        })
      );
    }
    whereParent(parentId) {
      return new Get(
        this.with({
          where: Expr(ExprData.Field({}, "parent")).is(parentId)[Expr.Data]
        })
      );
    }
    whereLocale(locale) {
      return new Get(
        this.with({
          where: Expr(ExprData.Field({}, "locale")).is(locale)[Expr.Data]
        })
      );
    }
    whereRoot(root) {
      return new Get(
        this.with({
          where: Expr(ExprData.Field({}, "root")).is(root)[Expr.Data]
        })
      );
    }
    whereWorkspace(workspace) {
      return new Get(
        this.with({
          where: Expr(ExprData.Field({}, "workspace")).is(workspace)[Expr.Data]
        })
      );
    }
    search(...searchTerms) {
      return new Get(this.with({ searchTerms }));
    }
    select(select) {
      return new Get(
        this.with({ select: createSelection(select) })
      );
    }
  }
  Cursor2.Get = Get;
})(Cursor || (Cursor = {}));
var Tree = class {
  constructor() {
    this.children = this.children.bind(this);
    this.parents = this.parents.bind(this);
  }
  narrowData(narrow) {
    return narrow && (Cursor.isCursor(narrow) ? narrow[Cursor.Data] : { target: narrow[targetData] });
  }
  find(sourceType, narrow, extraOptions) {
    return new Cursor.Find({
      ...this.narrowData(narrow),
      source: { type: sourceType, ...extraOptions }
    });
  }
  get(sourceType, narrow) {
    return new Cursor.Get({
      ...this.narrowData(narrow),
      first: true,
      source: { type: sourceType }
    });
  }
  children(narrow, depth) {
    ;
    [narrow, depth] = typeof narrow === "number" ? [void 0, narrow] : [narrow, depth || 1];
    return this.find(SourceType.Children, narrow, { depth });
  }
  parents(narrow, depth) {
    ;
    [narrow, depth] = typeof narrow === "number" ? [void 0, narrow] : [narrow, depth];
    return this.find(SourceType.Parents, narrow, { depth });
  }
  previous = (narrow) => {
    return this.get(SourceType.Previous, narrow);
  };
  next = (narrow) => {
    return this.get(SourceType.Next, narrow);
  };
  parent = (narrow) => {
    return this.get(SourceType.Parent, narrow);
  };
  siblings = (narrow) => {
    return this.find(SourceType.Siblings, narrow);
  };
  translations = (includeSelf = false) => {
    return this.find(SourceType.Translations, void 0, { includeSelf });
  };
};
export {
  Cursor,
  Tree
};
