import type { Cursor } from './Cursor.js';
import { ExprData, OrderBy, Selection, toExpr, toSelection } from './ResolveData.js';
/** Expression or value of type T */
export type EV<T> = Expr<T> | T;
export type Condition = Expr<boolean> | HasExpr<boolean>;
export interface Expr<T> extends ExprI<T> {
}
export declare function Expr<T>(expr: ExprData): Expr<T>;
export interface HasExpr<T> {
    [toExpr](): Expr<T>;
}
export interface ExprI<T> {
    [Expr.Data]: ExprData;
    [Expr.ExprRef]: boolean;
}
export declare class ExprI<T> {
    constructor(expr: ExprData);
    asc(): OrderBy;
    desc(): OrderBy;
    not(): Expr<boolean>;
    or(this: Expr<boolean>, ...that: Array<EV<boolean>>): Expr<boolean>;
    and(this: Expr<boolean>, ...that: Array<EV<boolean>>): Expr<boolean>;
    is(that: EV<T>): Expr<boolean>;
    isConstant(value: T): boolean;
    isNot(that: EV<T>): Expr<boolean>;
    isNull(): Expr<boolean>;
    isNotNull(): Expr<boolean>;
    isIn(that: EV<ReadonlyArray<T>> | Cursor.Find<T>): Expr<boolean>;
    isNotIn(that: EV<ReadonlyArray<T>> | Cursor.Find<T>): Expr<boolean>;
    isBetween(min: EV<T>, max: EV<T>): Expr<boolean>;
    isGreater(that: EV<any>): Expr<boolean>;
    isGreaterOrEqual(that: EV<any>): Expr<boolean>;
    isLess(that: EV<any>): Expr<boolean>;
    isLessOrEqual(that: EV<any>): Expr<boolean>;
    add(this: Expr<number>, that: EV<number>, ...rest: Array<EV<number>>): Expr<number>;
    subtract(this: Expr<number>, that: EV<number>, ...rest: Array<EV<number>>): Expr<number>;
    multiply(this: Expr<number>, that: EV<number>, ...rest: Array<EV<number>>): Expr<number>;
    remainder(this: Expr<number>, that: EV<number>): Expr<number>;
    divide(this: Expr<number>, that: EV<number>, ...rest: Array<EV<number>>): Expr<number>;
    concat(this: Expr<string>, that: EV<string>): Expr<string>;
    endsWith(this: Expr<string>, that: string): Expr<boolean>;
    startsWith(this: Expr<string>, that: string): Expr<boolean>;
    like(this: Expr<string>, that: EV<string>): Expr<boolean>;
    at<T>(this: Expr<Array<T>>, index: number): Expr<T | null>;
    includes<T>(this: Expr<Array<T>>, value: EV<T>): Expr<boolean>;
    sure(): Expr<NonNullable<T>>;
    get<T>(name: string): Expr<T>;
    toJSON(): ExprData;
    [toSelection](): Selection.Expr;
}
export declare namespace Expr {
    const Data: unique symbol;
    const ExprRef: unique symbol;
    const NULL: Expr<null>;
    function create<T>(input: EV<T>): Expr<T>;
    function hasExpr<T>(input: any): input is HasExpr<T>;
    function isExpr<T>(input: any): input is Expr<T>;
    function and(...conditions: Array<Condition | boolean>): Expr<boolean>;
    function or(...conditions: Array<Condition | boolean>): Expr<boolean>;
}
export declare function createExprData(input: any): ExprData;
