import * as cito from 'cito';
import { TypeTarget } from '../Type.js';
import { Expand } from '../util/Types.js';
import type { Projection } from './Projection.js';
export type Selection<T = any> = Selection.Row | Selection.Record | Selection.Cursor | Selection.Expr | Selection.Count;
export declare namespace Selection {
    namespace types {
        class Row {
            type: cito.Type<"row">;
            target: cito.Type<{
                name?: string | undefined;
                type?: TypeTarget | undefined;
            }>;
        }
        class Record {
            type: cito.Type<"record">;
            fields: cito.Type<([string, Selection<T>] | [{
                name?: string | undefined;
                type?: TypeTarget | undefined;
            }])[]>;
        }
        class Cursor {
            type: cito.Type<"cursor">;
            cursor: cito.Type<{
                target?: {
                    name?: string | undefined;
                    type?: TypeTarget | undefined;
                } | undefined;
                where?: ExprData | undefined;
                searchTerms?: string[] | undefined;
                skip?: number | undefined;
                take?: number | undefined;
                orderBy?: {
                    expr: ExprData;
                    order: OrderDirection;
                }[] | undefined;
                groupBy?: ExprData[] | undefined;
                select?: Selection<T> | undefined;
                first?: boolean | undefined;
                source?: {
                    type: SourceType;
                    depth?: number | undefined;
                    includeSelf?: boolean | undefined;
                } | undefined;
            }>;
        }
        class Expr {
            type: cito.Type<"expr">;
            expr: cito.Type<ExprData>;
            fromParent: cito.Type<boolean>;
        }
        class Count {
            type: cito.Type<"count">;
        }
    }
    export interface Row extends cito.Infer<types.Row> {
    }
    export function Row(target: TargetData): Selection.Row;
    export interface Record extends cito.Infer<types.Record> {
    }
    export function Record(fields: Array<[string, Selection] | [TargetData]>): Selection.Record;
    export interface Cursor extends cito.Infer<types.Cursor> {
    }
    export function Cursor(cursor: CursorData): Selection.Cursor;
    export interface Expr extends cito.Infer<types.Expr> {
    }
    export function Expr(expr: ExprData, fromParent?: boolean): Selection.Expr;
    export interface Count extends cito.Infer<types.Count> {
    }
    export function Count(): Selection.Count;
    export const adt: cito.Type<Selection>;
    export type Infer<T> = Projection.Infer<T>;
    export type Combine<A, B> = Expand<Omit<A, keyof Infer<B>> & Infer<B>>;
    export {};
}
export declare enum OrderDirection {
    Asc = "Asc",
    Desc = "Desc"
}
export type OrderBy = typeof OrderBy.infer;
export declare const OrderBy: cito.Type<{
    expr: ExprData;
    order: OrderDirection;
}>;
export type CursorData = typeof CursorData.infer;
export declare const CursorData: cito.Type<{
    target?: {
        name?: string | undefined;
        type?: TypeTarget | undefined;
    } | undefined;
    where?: ExprData | undefined;
    searchTerms?: string[] | undefined;
    skip?: number | undefined;
    take?: number | undefined;
    orderBy?: {
        expr: ExprData;
        order: OrderDirection;
    }[] | undefined;
    groupBy?: ExprData[] | undefined;
    select?: Selection<T> | undefined;
    first?: boolean | undefined;
    source?: {
        type: SourceType;
        depth?: number | undefined;
        includeSelf?: boolean | undefined;
    } | undefined;
}>;
export declare enum SourceType {
    Children = "Children",
    Siblings = "Siblings",
    Translations = "Translations",
    Parents = "Parents",
    Parent = "Parent",
    Next = "Next",
    Previous = "Previous"
}
export type CursorSource = typeof CursorSource.infer;
export declare const CursorSource: cito.Type<{
    type: SourceType;
    depth?: number | undefined;
    includeSelf?: boolean | undefined;
}>;
export declare enum UnaryOp {
    Not = "Not",
    IsNull = "IsNull"
}
export declare enum BinaryOp {
    Add = "Add",
    Subt = "Subt",
    Mult = "Mult",
    Mod = "Mod",
    Div = "Div",
    Greater = "Greater",
    GreaterOrEqual = "GreaterOrEqual",
    Less = "Less",
    LessOrEqual = "LessOrEqual",
    Equals = "Equals",
    NotEquals = "NotEquals",
    And = "And",
    Or = "Or",
    Like = "Like",
    In = "In",
    NotIn = "NotIn",
    Concat = "Concat"
}
export type ExprData = ExprData.UnOp | ExprData.BinOp | ExprData.Field | ExprData.Access | ExprData.Value | ExprData.Record | ExprData.Call;
export declare namespace ExprData {
    namespace types {
        class UnOp {
            type: cito.Type<"unop">;
            op: cito.Type<UnaryOp>;
            expr: cito.Type<ExprData>;
        }
        class BinOp {
            type: cito.Type<"binop">;
            a: cito.Type<ExprData>;
            op: cito.Type<BinaryOp>;
            b: cito.Type<ExprData>;
        }
        class Field {
            type: cito.Type<"field">;
            target: cito.Type<{
                name?: string | undefined;
                type?: TypeTarget | undefined;
            }>;
            field: cito.Type<string>;
        }
        class Access {
            type: cito.Type<"access">;
            expr: cito.Type<ExprData>;
            field: cito.Type<string>;
        }
        class Value {
            type: cito.Type<"value">;
            value: cito.Type<any>;
        }
        class Record {
            type: cito.Type<"record">;
            fields: cito.Type<globalThis.Record<string, ExprData>>;
        }
        class Call {
            type: cito.Type<"call">;
            method: cito.Type<string>;
            args: cito.Type<ExprData[]>;
        }
    }
    export interface UnOp extends cito.Infer<types.UnOp> {
    }
    export function UnOp(op: UnaryOp, expr: ExprData): ExprData;
    export interface BinOp extends cito.Infer<types.BinOp> {
    }
    export function BinOp(a: ExprData, op: BinaryOp, b: ExprData): ExprData;
    export interface Field extends cito.Infer<types.Field> {
    }
    export function Field(target: TargetData, field: string): ExprData;
    export interface Access extends cito.Infer<types.Access> {
    }
    export function Access(expr: ExprData, field: string): ExprData;
    export interface Value extends cito.Infer<types.Value> {
    }
    export function Value(value: any): ExprData;
    export interface Record extends cito.Infer<types.Record> {
    }
    export function Record(fields: {
        [k: string]: ExprData;
    }): ExprData;
    export interface Call extends cito.Infer<types.Call> {
    }
    export function Call(method: string, args: Array<ExprData>): ExprData;
    export const adt: cito.Type<ExprData>;
    export {};
}
export type TargetData = typeof TargetData.infer;
export declare const TargetData: cito.Type<{
    name?: string | undefined;
    type?: TypeTarget | undefined;
}>;
export declare const toSelection: unique symbol;
export declare const toExpr: unique symbol;
export declare const targetData: unique symbol;
