import "../../chunks/chunk-U5RRZUYZ.js";

// src/core/pages/Target.ts
import { rowId } from "../util/RowId.js";
import { Cursor } from "./Cursor.js";
import { Expr, createExprData } from "./Expr.js";
import {
  BinaryOp,
  ExprData,
  Selection,
  targetData,
  toSelection
} from "./ResolveData.js";
var { create, entries } = Object;
var Target = class {
  constructor(data) {
    this.data = data;
  }
  static IsTarget = Symbol.for("@alinea/Target.IsTarget");
  cache = create(null);
  static isTarget(target) {
    return target && target[Target.IsTarget];
  }
  call(...input) {
    return new Cursor.Find({
      target: this.data,
      where: this.condition(input)
    });
  }
  condition(input) {
    if (input.length === 0)
      return void 0;
    const isConditionalRecord = input.length === 1 && !Expr.isExpr(input[0]);
    const conditions = isConditionalRecord ? entries(input[0]).map(([key, value]) => {
      const field = Expr(ExprData.Field(this.data, key));
      return Expr(
        ExprData.BinOp(
          field[Expr.Data],
          BinaryOp.Equals,
          createExprData(value)
        )
      );
    }) : input.map((ev) => Expr(createExprData(ev)));
    return Expr.and(...conditions)[Expr.Data];
  }
  get(field) {
    if (field in this.cache)
      return this.cache[field];
    return this.cache[field] = Expr(ExprData.Field(this.data, field));
  }
  static create(data) {
    const impl = new this(data);
    const name = data.name || "target";
    const call = { [name]: (...args) => impl.call(...args) }[name];
    const id = rowId();
    return new Proxy(call, {
      ownKeys() {
        return ["prototype", id];
      },
      getOwnPropertyDescriptor(target, prop) {
        if (prop === id)
          return { enumerable: true, configurable: true };
        return Reflect.getOwnPropertyDescriptor(target, prop);
      },
      get: (_, prop) => {
        if (typeof prop === "string") {
          if (prop === id)
            return data;
          return impl.get(prop);
        }
        switch (prop) {
          case Target.IsTarget:
            return true;
          case targetData:
            return data;
          case toSelection:
            return Selection.Row(data);
        }
      }
    });
  }
};
export {
  Target
};
