/**
 * Given a base64 string, optionally decompress it and returns its `ArrayBuffer` representation.
 * @param {string} btoa a previously encoded base64 representation of a buffer.
 * @param {CompressionFormat | ''} format an optional compression format with `deflate` as default.
 * @returns {Promise<ArrayBuffer>} the buffer representing the optionally decompressed base64.
 */
export declare const decode: (btoa: string, format?: CompressionFormat | "") => Promise<ArrayBuffer>;
/**
 * Given a generic buffer, optionally compress it and returns its `base64` representation.
 * @param {ArrayBuffer | Uint8Array} buffer a generic buffer to optionally compress and return as base64.
 * @param {CompressionFormat | ''} format an optional compression format with `deflate` as default.
 * @returns {Promise<string>} the base64 representation of the optionally compressed buffer.
 */
export declare const encode: (buffer: ArrayBuffer | Uint8Array, format?: CompressionFormat | "") => Promise<string>;
