import {
  atomFamily
} from "../chunks/chunk-ZHH24SIG.js";
import {
  atom
} from "../chunks/chunk-OBOPLPUQ.js";
import "../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/Routes.tsx
import { Entry } from "alinea/core/Entry";
import { graphAtom } from "alinea/dashboard/atoms/DbAtoms";
import { locationAtom, useNavigate } from "alinea/dashboard/atoms/LocationAtoms";
import { useNav } from "alinea/dashboard/hook/UseNav";
import { Loader } from "alinea/ui";
import { useLayoutEffect } from "react";
import { entryEditorAtoms } from "./atoms/EntryEditorAtoms.js";
import { entryLocationAtom, localeAtom } from "./atoms/NavigationAtoms.js";
import { Route, Router } from "./atoms/RouterAtoms.js";
import { ContentView } from "./pages/ContentView.js";
import { DraftsOverview } from "./pages/DraftsOverview.js";
import { jsx } from "react/jsx-runtime";
var editorLoader = atomFamily(() => {
  return atom(async (get) => {
    const entryLocation = get(entryLocationAtom);
    const locale = get(localeAtom) ?? null;
    return {
      editor: await get(
        entryEditorAtoms({ locale, i18nId: entryLocation?.entryId })
      )
    };
  });
});
var entryRoute = new Route({
  path: "*",
  loader: editorLoader,
  component: ContentView
});
var draftRoute = new Route({
  path: "/draft/:workspace?/:root?/:id?",
  loader: editorLoader,
  component: DraftsOverview
});
var editLoader = atomFamily(() => {
  return atom(async (get) => {
    const location = get(locationAtom);
    const searchParams = new URLSearchParams(location.search);
    const url = searchParams.get("url");
    const workspace = searchParams.get("workspace") ?? void 0;
    const root = searchParams.get("root") ?? void 0;
    const where = { url };
    if (workspace)
      where.workspace = workspace;
    if (root)
      where.root = root;
    const graph = await get(graphAtom);
    const entry = await graph.preferDraft.maybeGet(
      Entry(where).select({
        entryId: Entry.entryId,
        locale: Entry.locale,
        i18nId: Entry.i18nId,
        root: Entry.root,
        workspace: Entry.workspace
      })
    );
    if (!entry)
      return null;
    return { ...entry, locale: entry.locale || void 0 };
  });
});
var editRoute = new Route({
  path: "/edit",
  loader: editLoader,
  component: EditRoute
});
function EditRoute(location) {
  const nav = useNav();
  const navigate = useNavigate();
  useLayoutEffect(() => {
    if (location)
      navigate(nav.entry(location));
  }, [location]);
  return /* @__PURE__ */ jsx(Loader, { absolute: true });
}
var routes = [draftRoute, editRoute, entryRoute];
var router = new Router({ routes });
export {
  draftRoute,
  editRoute,
  entryRoute,
  router
};
