import {
  useAtomValue
} from "../../chunks/chunk-WF77DMLN.js";
import {
  atom
} from "../../chunks/chunk-OBOPLPUQ.js";
import {
  require_dataloader
} from "../../chunks/chunk-ICLGQ53P.js";
import {
  __toESM
} from "../../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/atoms/EntryAtoms.ts
var import_dataloader = __toESM(require_dataloader(), 1);
import { Entry } from "alinea/core/Entry";
import { MutationType } from "alinea/core/Mutation";
import { Type } from "alinea/core/Type";
import { entryFileName } from "alinea/core/util/EntryFilenames";
import {
  generateKeyBetween,
  generateNKeysBetween
} from "alinea/core/util/FractionalIndexing";
import { entries } from "alinea/core/util/Objects";
import { useMemo } from "react";
import { useDashboard } from "../hook/UseDashboard.js";
import { configAtom } from "./DashboardAtoms.js";
import { graphAtom, useMutate } from "./DbAtoms.js";
import { localeAtom, rootAtom, workspaceAtom } from "./NavigationAtoms.js";
function rootId(rootName) {
  return `@alinea/root-${rootName}`;
}
var visibleTypesAtom = atom((get) => {
  const { schema } = get(configAtom);
  return entries(schema).filter(([_, type]) => !Type.isHidden(type)).map(([name]) => name);
});
async function entryTreeRoot(active, workspace, root, visibleTypes) {
  const rootEntries = Entry().where(
    Entry.workspace.is(workspace),
    Entry.root.is(root),
    Entry.parent.isNull(),
    Entry.active,
    Entry.type.isIn(visibleTypes)
  ).select(Entry.i18nId).groupBy(Entry.i18nId).orderBy(Entry.index.asc());
  const children = await active.find(rootEntries);
  return {
    id: rootId(root),
    index: "",
    type: "",
    isFolder: true,
    entries: [],
    children
  };
}
var entryTreeItemLoaderAtom = atom(async (get) => {
  const graph = await get(graphAtom);
  const locale = get(localeAtom);
  const visibleTypes = get(visibleTypesAtom);
  const { schema } = get(configAtom);
  const root = get(rootAtom);
  const workspace = get(workspaceAtom);
  return new import_dataloader.default(async (ids) => {
    const indexed = /* @__PURE__ */ new Map();
    const search = ids.filter((id) => id !== rootId(root.name));
    const data = {
      id: Entry.i18nId,
      entryId: Entry.entryId,
      type: Entry.type,
      title: Entry.title,
      phase: Entry.phase,
      locale: Entry.locale,
      workspace: Entry.workspace,
      root: Entry.root,
      path: Entry.path,
      parentPaths({ parents }) {
        return parents(Entry).select(Entry.path);
      }
      /*children({children}) {
        return children(Entry)
          .where(Entry.type.isIn(visibleTypes))
          .select(Entry.i18nId)
          .groupBy(Entry.i18nId)
          .orderBy(Entry.index.asc())
      }*/
    };
    const entries2 = Entry().select({
      id: Entry.i18nId,
      entryId: Entry.entryId,
      index: Entry.index,
      type: Entry.type,
      data,
      translations({ translations }) {
        return translations().select(data);
      }
    }).groupBy(Entry.i18nId).where(Entry.i18nId.isIn(search));
    const rows = await graph.preferDraft.find(entries2);
    for (const row of rows) {
      const type = schema[row.type];
      const orderBy = Type.meta(type).orderChildrenBy ?? Entry.index.asc();
      const ids2 = row.translations.map((row2) => row2.entryId).concat(row.entryId);
      const children = await graph.preferDraft.find(
        Entry().where(Entry.parent.isIn(ids2), Entry.type.isIn(visibleTypes)).select({ locale: Entry.locale, i18nId: Entry.i18nId }).orderBy(orderBy)
      );
      const entries3 = [row.data].concat(row.translations);
      const translatedChildren = new Set(
        children.filter((child) => child.locale === locale).map((child) => child.i18nId)
      );
      const untranslated = /* @__PURE__ */ new Set();
      const orderedChildren = children.filter((child) => {
        if (translatedChildren.has(child.i18nId))
          return child.locale === locale;
        if (untranslated.has(child.i18nId))
          return false;
        untranslated.add(child.i18nId);
        return true;
      });
      indexed.set(row.id, {
        id: row.id,
        type: row.type,
        index: row.index,
        entries: entries3,
        children: [...new Set(orderedChildren.map((child) => child.i18nId))]
      });
    }
    const res = [];
    for (const id of ids) {
      if (id === rootId(root.name)) {
        res.push(
          await entryTreeRoot(
            graph.preferDraft,
            workspace.name,
            root.name,
            visibleTypes
          )
        );
        continue;
      }
      const entry = indexed.get(id);
      if (!entry) {
        res.push(void 0);
        continue;
      }
      const typeName = entry.entries[0].type;
      const type = schema[typeName];
      const isFolder = Type.isContainer(type);
      res.push({ ...entry, isFolder });
    }
    return res;
  });
});
var loaderAtom = atom((get) => {
  return { loader: get(entryTreeItemLoaderAtom) };
});
function useEntryTreeProvider() {
  const { loader } = useAtomValue(loaderAtom);
  const mutate = useMutate();
  const { config } = useDashboard();
  return useMemo(() => {
    return {
      onDrop(items, { item: parent, childIndex, insertionIndex }) {
        if (items.length !== 1)
          return;
        const [dropping] = items;
        if (childIndex === null)
          return;
        if (dropping.getParent() !== parent) {
          console.warn("Todo: move entries");
          return;
        }
        const children = parent.getChildren();
        const previous = children[childIndex - 1];
        const previousIndexKey = previous?.getItemData()?.index ?? null;
        const next = children.slice(childIndex).find(
          (entry) => !entry || entry?.getItemData().index !== previousIndexKey
        );
        const nextChildIndex = next ? children.indexOf(next) : void 0;
        const nextIndexKey = next?.getItemData()?.index ?? null;
        try {
          const brokenChildren = children.slice(childIndex, nextChildIndex);
          const newIndexKey = generateKeyBetween(previousIndexKey, nextIndexKey);
          const mutations = [];
          if (brokenChildren.length > 0) {
            const newKeys = generateNKeysBetween(
              newIndexKey,
              nextIndexKey,
              brokenChildren.length
            );
            for (let i = 0; i < brokenChildren.length; i++) {
              const child = brokenChildren[i];
              const correctedIndexKey = newKeys[i];
              for (const entry of child.getItemData().entries) {
                mutations.push({
                  type: MutationType.Order,
                  entryId: entry.entryId,
                  file: entryFileName(config, entry, entry.parentPaths),
                  index: correctedIndexKey
                });
              }
            }
          }
          for (const entry of dropping.getItemData().entries) {
            mutations.push({
              type: MutationType.Order,
              entryId: entry.entryId,
              file: entryFileName(config, entry, entry.parentPaths),
              index: newIndexKey
            });
          }
          mutate(mutations, true);
        } catch (err) {
          console.error(err);
        }
      },
      async getItem(id) {
        return await (await loader).clear(id).load(id);
      },
      async getChildren(id) {
        return this.getItem(id).then((item) => item?.children ?? []);
      }
    };
  }, [loader]);
}
export {
  rootId,
  useEntryTreeProvider
};
