import {
  useSetAtom
} from "../../chunks/chunk-WF77DMLN.js";
import "../../chunks/chunk-OBOPLPUQ.js";
import {
  QueryClient
} from "../../chunks/chunk-DJKGEOOC.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/dev/DevDashboard.tsx
import { Client } from "alinea/core/Client";
import { useEffect, useState } from "react";
import { App } from "../App.js";
import { dbUpdateAtom } from "../atoms/DbAtoms.js";
import { jsx } from "react/jsx-runtime";
function setupDevReload({ refresh, refetch, open, close }) {
  const source = new EventSource("/~dev");
  source.onmessage = (e) => {
    console.info(`[reload] received ${e.data}`);
    switch (e.data) {
      case "refetch":
        return refetch();
      case "reload":
        return window.location.reload();
      default:
        return refresh();
    }
  };
  source.onopen = open;
  source.onerror = close;
  return () => {
    source.close();
  };
}
var queryClient = new QueryClient({ defaultOptions: { queries: { retry: false } } });
function DevDashboard({ loadConfig, loadViews }) {
  const [app, setApp] = useState();
  const [connected, setConnected] = useState(true);
  const forceDbUpdate = useSetAtom(dbUpdateAtom);
  async function getConfig() {
    const link = document.querySelector(
      'link[href^="/views.css"]'
    );
    const copy = link.cloneNode();
    copy.href = "/views.css?" + Math.random();
    copy.onload = () => link.remove();
    link.after(copy);
    const [config, views] = await Promise.all([loadConfig(), loadViews()]);
    const client = new Client({
      url: new URL("/api", location.href).href
    });
    return setApp({ config, views, client });
  }
  useEffect(() => {
    getConfig();
    return setupDevReload({
      refresh: () => getConfig().then(() => forceDbUpdate(true)),
      refetch: () => forceDbUpdate(true),
      open: () => setConnected(true),
      close: () => setConnected(false)
    });
  }, []);
  if (!app)
    return null;
  return /* @__PURE__ */ jsx(
    App,
    {
      queryClient,
      dev: process.env.NODE_ENV === "development",
      ...app
    }
  );
}
export {
  DevDashboard
};
