import {
  useAtom,
  useSetAtom
} from "../../chunks/chunk-WF77DMLN.js";
import {
  atom
} from "../../chunks/chunk-OBOPLPUQ.js";
import {
  pLimit
} from "../../chunks/chunk-QQTYTFWR.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/hook/UseUploads.ts
import { Entry } from "alinea/core/Entry";
import { EntryPhase } from "alinea/core/EntryRow";
import { HttpError } from "alinea/core/HttpError";
import { createId } from "alinea/core/Id";
import { MutationType } from "alinea/core/Mutation";
import { Workspace } from "alinea/core/Workspace";
import { createPreview } from "alinea/core/media/CreatePreview.browser";
import { isImage } from "alinea/core/media/IsImage";
import { MEDIA_LOCATION } from "alinea/core/media/MediaLocation";
import { createFileHash } from "alinea/core/util/ContentHash";
import { entryFileName, entryFilepath } from "alinea/core/util/EntryFilenames";
import { createEntryRow } from "alinea/core/util/EntryRows";
import { generateKeyBetween } from "alinea/core/util/FractionalIndexing";
import {
  basename,
  dirname,
  extname,
  join,
  normalize
} from "alinea/core/util/Paths";
import { slugify } from "alinea/core/util/Slugs";
import { useEffect } from "react";
import { useMutate } from "../atoms/DbAtoms.js";
import { errorAtom } from "../atoms/ErrorAtoms.js";
import { withResolvers } from "../util/WithResolvers.js";
import { useConfig } from "./UseConfig.js";
import { useGraph } from "./UseGraph.js";
import { useSession } from "./UseSession.js";
var UploadStatus = /* @__PURE__ */ ((UploadStatus2) => {
  UploadStatus2[UploadStatus2["Queued"] = 0] = "Queued";
  UploadStatus2[UploadStatus2["CreatingPreview"] = 1] = "CreatingPreview";
  UploadStatus2[UploadStatus2["Uploading"] = 2] = "Uploading";
  UploadStatus2[UploadStatus2["Uploaded"] = 3] = "Uploaded";
  UploadStatus2[UploadStatus2["Done"] = 4] = "Done";
  return UploadStatus2;
})(UploadStatus || {});
var defaultTasker = pLimit(Infinity);
var cpuTasker = pLimit(1);
var networkTasker = pLimit(8);
var batchTasker = pLimit(1);
var tasker = {
  [0 /* Queued */]: defaultTasker,
  [1 /* CreatingPreview */]: cpuTasker,
  [2 /* Uploading */]: networkTasker,
  [3 /* Uploaded */]: defaultTasker,
  [4 /* Done */]: defaultTasker
};
async function process(upload, publishUpload, client) {
  switch (upload.status) {
    case 0 /* Queued */:
      const next = isImage(upload.file.name) ? 1 /* CreatingPreview */ : 2 /* Uploading */;
      return { ...upload, status: next };
    case 1 /* CreatingPreview */: {
      const previewData = await createPreview(upload.file);
      return {
        ...upload,
        ...previewData,
        status: 2 /* Uploading */
      };
    }
    case 2 /* Uploading */: {
      const fileName = upload.file.name;
      const extension = extname(fileName);
      const path = slugify(basename(fileName, extension));
      const file = join(upload.to.directory, path + extension);
      const info = await client.prepareUpload(file);
      await fetch(info.url, {
        method: info.method ?? "POST",
        body: upload.file
      }).then(async (result) => {
        if (!result.ok)
          throw new HttpError(
            result.status,
            `Could not reach server for upload`
          );
      });
      return { ...upload, info, status: 3 /* Uploaded */ };
    }
    case 3 /* Uploaded */: {
      const entry = await publishUpload(upload);
      return { ...upload, result: entry, status: 4 /* Done */ };
    }
    case 4 /* Done */:
      throw new Error("Should not end up here");
  }
}
function createBatch(mutate) {
  let trigger = withResolvers();
  let nextRun = void 0;
  const batch = [];
  async function run() {
    const todo = batch.splice(0, batch.length);
    try {
      await batchTasker(() => mutate(todo));
      trigger.resolve(void 0);
    } catch (error) {
      trigger.reject(error);
    } finally {
      trigger = withResolvers();
    }
  }
  return (...mutations) => {
    batch.push(...mutations);
    clearTimeout(nextRun);
    nextRun = setTimeout(run, 200);
    return trigger.promise;
  };
}
var uploadsAtom = atom([]);
function useUploads(onSelect) {
  const config = useConfig();
  const graph = useGraph();
  const { cnx: client } = useSession();
  const mutate = useMutate();
  const setErrorAtom = useSetAtom(errorAtom);
  const [uploads, setUploads] = useAtom(uploadsAtom);
  const batch = createBatch(mutate);
  useEffect(() => {
    return () => setUploads([]);
  }, []);
  async function batchMutations(...mutations) {
    await batch(...mutations);
  }
  async function createEntry(upload2) {
    const entryId = upload2.info?.entryId ?? createId();
    const { parentId } = upload2.to;
    const buffer = await upload2.file.arrayBuffer();
    const parent = await graph.preferPublished.maybeGet(
      Entry({ entryId: parentId }).select({
        level: Entry.level,
        entryId: Entry.entryId,
        url: Entry.url,
        path: Entry.path,
        parentPaths({ parents }) {
          return parents().select(Entry.path);
        }
      })
    );
    const extensionOriginal = extname(upload2.file.name);
    const extension = extensionOriginal.toLowerCase();
    const path = slugify(basename(upload2.file.name, extensionOriginal));
    const prev = await graph.preferPublished.maybeGet(Entry({ parent: parentId }));
    const entryLocation = {
      workspace: upload2.to.workspace,
      root: upload2.to.root,
      locale: null,
      path,
      phase: EntryPhase.Published
    };
    const filePath = entryFilepath(
      config,
      entryLocation,
      parent ? parent.parentPaths.concat(parent.path) : []
    );
    const parentDir = dirname(filePath);
    const { location } = upload2.info;
    const workspace = Workspace.data(config.workspaces[upload2.to.workspace]);
    const prefix = workspace.mediaDir && normalize(workspace.mediaDir);
    const fileLocation = prefix && location.startsWith(prefix) ? location.slice(prefix.length) : location;
    const hash = await createFileHash(new Uint8Array(buffer));
    const title = basename(upload2.file.name, extensionOriginal);
    const entry = await createEntryRow(config, {
      ...entryLocation,
      parent: parent?.entryId ?? null,
      entryId,
      type: "MediaFile",
      url: (parent ? parent.url : "") + "/" + path,
      title,
      seeded: null,
      modifiedAt: Date.now(),
      searchableText: "",
      index: generateKeyBetween(null, prev?.index ?? null),
      i18nId: entryId,
      level: parent ? parent.level + 1 : 0,
      parentDir,
      filePath,
      childrenDir: filePath.slice(0, -".json".length),
      active: true,
      main: true,
      data: {
        title,
        location: fileLocation,
        extension,
        size: buffer.byteLength,
        hash,
        width: upload2.width,
        height: upload2.height,
        averageColor: upload2.averageColor,
        focus: upload2.focus,
        thumbHash: upload2.thumbHash,
        preview: upload2.preview
      }
    });
    const file = entryFileName(
      config,
      entry,
      parent ? parent.parentPaths.concat(parent.path) : []
    );
    return { file, entry };
  }
  async function uploadFile(upload2) {
    function update(upload3) {
      setUploads((current) => {
        const result = current.slice();
        const index = current.findIndex((u) => u.id === upload3.id);
        if (index === -1)
          return result;
        result[index] = upload3;
        return result;
      });
    }
    while (true) {
      const next = await tasker[upload2.status](
        () => process(upload2, publishUpload, client)
      ).catch((error) => {
        return { ...upload2, error, status: 4 /* Done */ };
      });
      update(next);
      if (next.status === 4 /* Done */) {
        if (next.error) {
          setErrorAtom(next.error.message, next.error);
        }
        const result = next.result;
        if (!result)
          break;
        onSelect?.(result);
        break;
      } else {
        upload2 = next;
      }
    }
  }
  async function publishUpload(upload2) {
    const { replace } = upload2;
    const info = upload2.info;
    const { file, entry } = await createEntry(upload2);
    if (!replace) {
      await batchMutations(
        {
          type: MutationType.Create,
          entryId: entry.entryId,
          file,
          entry
        },
        {
          type: MutationType.Upload,
          entryId: entry.entryId,
          url: info.previewUrl,
          file: info.location
        }
      );
      return entry;
    }
    const newEntry = await createEntryRow(config, {
      ...replace.entry,
      data: { ...entry.data, title: replace.entry.title }
    });
    const mediaFile = replace.entry.data;
    await batchMutations(
      {
        type: MutationType.Edit,
        entryId: replace.entry.entryId,
        file: replace.entryFile,
        entry: newEntry
      },
      {
        type: MutationType.Upload,
        entryId: replace.entry.entryId,
        url: info.previewUrl,
        file: info.location
      },
      {
        type: MutationType.FileRemove,
        entryId: replace.entry.entryId,
        file: replace.entryFile,
        workspace: replace.entry.workspace,
        location: MEDIA_LOCATION in mediaFile ? mediaFile[MEDIA_LOCATION] : mediaFile.location,
        replace: true
      }
    );
    return newEntry;
  }
  async function upload(files, to, replace) {
    const uploads2 = Array.from(files).map((file) => {
      return { id: createId(), file, to, replace, status: 0 /* Queued */ };
    });
    setUploads((current) => [...uploads2, ...current]);
    return Promise.all(uploads2.map(uploadFile));
  }
  return { upload, uploads };
}
export {
  UploadStatus,
  useUploads
};
