import {
  package_default
} from "../../chunks/chunk-7EETFPPX.js";
import {
  create as create2
} from "../../chunks/chunk-IOTY7UTU.js";
import {
  create
} from "../../chunks/chunk-BRTZLYIM.js";
import {
  connect
} from "../../chunks/chunk-4FCVG65Y.js";
import "../../chunks/chunk-OMOA6SIP.js";
import "../../chunks/chunk-PE5G5FUO.js";
import "../../chunks/chunk-GR7UQZEH.js";
import "../../chunks/chunk-AOFVF5VC.js";
import "../../chunks/chunk-NXCD6SNE.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/util/PersistentStore.ts
import sqlInit from "@alinea/sqlite-wasm";

// node_modules/lib0/indexeddb.js
var rtop = (request) => create2((resolve, reject) => {
  request.onerror = (event) => reject(new Error(event.target.error));
  request.onsuccess = (event) => resolve(event.target.result);
});
var openDB = (name, initDB) => create2((resolve, reject) => {
  const request = indexedDB.open(name);
  request.onupgradeneeded = (event) => initDB(event.target.result);
  request.onerror = (event) => reject(create(event.target.error));
  request.onsuccess = (event) => {
    const db = event.target.result;
    db.onversionchange = () => {
      db.close();
    };
    resolve(db);
  };
});
var createStores = (db, definitions) => definitions.forEach(
  (d) => (
    // @ts-ignore
    db.createObjectStore.apply(db, d)
  )
);
var transact = (db, stores, access = "readwrite") => {
  const transaction = db.transaction(stores, access);
  return stores.map((store) => getStore(transaction, store));
};
var get = (store, key) => rtop(store.get(key));
var del = (store, key) => rtop(store.delete(key));
var put = (store, item, key) => rtop(store.put(item, key));
var getAllKeys = (store, range, limit) => rtop(store.getAllKeys(range, limit));
var getStore = (t, store) => t.objectStore(store);

// src/dashboard/util/PersistentStore.ts
var STORAGE_NAME = `@alinea/peristent.store`;
var dbName = `db-${package_default.version}`;
async function createPersistentStore() {
  const storagePromise = openDB(
    STORAGE_NAME,
    (db2) => createStores(db2, [[STORAGE_NAME, { autoIncrement: true }]])
  );
  const sqlitePromise = sqlInit();
  const [storage, { Database }] = await Promise.all([
    storagePromise,
    sqlitePromise
  ]);
  let store = transact(storage, [STORAGE_NAME], "readwrite")[0];
  const databases = await getAllKeys(store);
  for (const name of databases) {
    if (name !== dbName) {
      await del(store, name);
    }
  }
  store = transact(storage, [STORAGE_NAME], "readonly")[0];
  const buffer = await get(store, dbName);
  const init = buffer instanceof ArrayBuffer ? new Uint8Array(buffer) : void 0;
  let db = new Database(init);
  const persistent = {
    store: connect(db),
    async flush() {
      store = transact(storage, [STORAGE_NAME], "readwrite")[0];
      await put(store, db.export().buffer, dbName);
    },
    clone() {
      const clone = new Database(db.export());
      return connect(clone);
    },
    async clear() {
      store = transact(storage, [STORAGE_NAME], "readwrite")[0];
      await del(store, dbName);
      db = new Database();
      persistent.store = connect(db);
    }
  };
  return persistent;
}
export {
  createPersistentStore
};
