import {
  dist_default
} from "../../../chunks/chunk-ANJZSESR.js";
import {
  useAtomValue,
  useSetAtom
} from "../../../chunks/chunk-WF77DMLN.js";
import "../../../chunks/chunk-OBOPLPUQ.js";
import {
  useQuery
} from "../../../chunks/chunk-DJKGEOOC.js";
import "../../../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/view/entry/NewEntry.tsx
import { Entry } from "alinea/core/Entry";
import { EntryPhase } from "alinea/core/EntryRow";
import { createId } from "alinea/core/Id";
import { MutationType } from "alinea/core/Mutation";
import { Reference } from "alinea/core/Reference";
import { Schema } from "alinea/core/Schema";
import { track } from "alinea/core/Tracker";
import { Type, type } from "alinea/core/Type";
import {
  entryChildrenDir,
  entryFileName,
  entryFilepath,
  entryUrl
} from "alinea/core/util/EntryFilenames";
import { createEntryRow } from "alinea/core/util/EntryRows";
import { generateKeyBetween } from "alinea/core/util/FractionalIndexing";
import { entries, fromEntries, keys } from "alinea/core/util/Objects";
import { dirname } from "alinea/core/util/Paths";
import { slugify } from "alinea/core/util/Slugs";
import { useForm } from "alinea/dashboard/atoms/FormAtoms";
import { InputForm } from "alinea/dashboard/editor/InputForm";
import { useLocation, useNavigate } from "alinea/dashboard/util/HashRouter";
import { Modal } from "alinea/dashboard/view/Modal";
import { EntryLink, entry } from "alinea/field/link";
import { select } from "alinea/field/select";
import { text } from "alinea/field/text";
import { entryPicker } from "alinea/picker/entry/EntryPicker";
import { EntryReference } from "alinea/picker/entry/EntryReference";
import { Button, Loader } from "alinea/ui";
import { Link } from "alinea/ui/Link";
import { Suspense, useEffect, useMemo, useState } from "react";
import { changedEntriesAtom, graphAtom, useMutate } from "../../atoms/DbAtoms.js";
import { useConfig } from "../../hook/UseConfig.js";
import { useLocale } from "../../hook/UseLocale.js";
import { useNav } from "../../hook/UseNav.js";
import { useRoot } from "../../hook/UseRoot.js";
import { useWorkspace } from "../../hook/UseWorkspace.js";

// src/dashboard/view/entry/NewEntry.module.scss
var NewEntry_module_default = {
  "root": "alinea-NewEntry",
  "root-header": "alinea-NewEntry-header",
  "rootHeader": "alinea-NewEntry-header",
  "root-footer": "alinea-NewEntry-footer",
  "rootFooter": "alinea-NewEntry-footer",
  "root-footer-link": "alinea-NewEntry-footer-link",
  "rootFooterLink": "alinea-NewEntry-footer-link",
  "form": "alinea-NewEntry-form",
  "is-loading": "alinea-NewEntry-is-loading",
  "isLoading": "alinea-NewEntry-is-loading"
};

// src/dashboard/view/entry/NewEntry.tsx
import { jsx, jsxs } from "react/jsx-runtime";
var styles = dist_default(NewEntry_module_default);
var parentData = {
  id: Entry.entryId,
  i18nId: Entry.i18nId,
  type: Entry.type,
  path: Entry.path,
  url: Entry.url,
  level: Entry.level,
  parent: Entry.parent,
  parentPaths({ parents }) {
    return parents().select(Entry.path);
  },
  childrenIndex({ children }) {
    return children().select(Entry.index).orderBy(Entry.index.asc()).first();
  }
};
var titleField = text("Title", { autoFocus: true });
function NewEntryForm({ parentId }) {
  const config = useConfig();
  const graph = useAtomValue(graphAtom);
  const { data: requestedParent } = useQuery(
    ["parent-req", parentId],
    async () => {
      return graph.preferDraft.maybeGet(
        Entry({ entryId: parentId }).select(parentData)
      );
    },
    { suspense: true, keepPreviousData: true, staleTime: 0 }
  );
  const preselectedId = requestedParent && (Type.isContainer(config.schema[requestedParent.type]) ? requestedParent.id : requestedParent.parent);
  const { pathname } = useLocation();
  const nav = useNav();
  const navigate = useNavigate();
  const locale = useLocale();
  const mutate = useMutate();
  const { name: workspace } = useWorkspace();
  const containerTypes = entries(config.schema).filter(([, type2]) => {
    return Type.isContainer(type2);
  }).map((pair) => pair[0]);
  const root = useRoot();
  const parentField = useMemo(() => {
    return entry("Parent", {
      location: {
        workspace,
        root: root.name
      },
      condition: Entry.type.isIn(containerTypes),
      initialValue: preselectedId ? {
        [Reference.id]: "parent",
        [Reference.type]: "entry",
        [EntryReference.entry]: preselectedId
      } : void 0
    });
  }, []);
  async function allowedTypes(parentId2) {
    if (!parentId2) {
      return root.contains ? Schema.contained(config.schema, root.contains) : keys(config.schema);
    } else {
      const parent = await graph.preferDraft.get(
        Entry({ entryId: parentId2 }).select(parentData)
      );
      const parentType = parent && config.schema[parent.type];
      if (parentType)
        return Schema.contained(config.schema, Type.contains(parentType));
      return keys(config.schema);
    }
  }
  const typeField = useMemo(() => {
    const typeField2 = select("Select type", {
      options: {}
    });
    return track.options(typeField2, async (get) => {
      const selectedParent2 = get(parentField);
      const parentId2 = selectedParent2?.[EntryReference.entry];
      const types = await allowedTypes(parentId2);
      return {
        options: fromEntries(
          types.map((key) => {
            return [key, config.schema[key]];
          }).filter((row) => row[1]).map(([key, type2]) => {
            return [key, Type.label(type2) || key];
          })
        )
      };
    });
  }, []);
  const copyFromField = useMemo(() => {
    const copyFromField2 = entry("Copy content from");
    return track.options(copyFromField2, (get) => {
      const type2 = get(typeField);
      return {
        readOnly: !type2,
        pickers: {
          entry: entryPicker({
            condition: Entry.type.is(type2),
            withNavigation: false,
            title: "Copy content from",
            max: 1,
            selection: EntryLink
          })
        }
      };
    });
  }, []);
  const [isCreating, setIsCreating] = useState(false);
  const updateEntries = useSetAtom(changedEntriesAtom);
  const formType = useMemo(
    () => type({
      fields: {
        parent: parentField,
        title: titleField,
        type: typeField,
        copyFrom: copyFromField
      }
    }),
    []
  );
  const form = useForm(formType);
  const parentAtoms = form.fieldInfo(parentField);
  const typeAtoms = form.fieldInfo(typeField);
  const copyFromAtoms = form.fieldInfo(copyFromField);
  const selectedType = useAtomValue(typeAtoms.value);
  const selectedParent = useAtomValue(parentAtoms.value);
  useEffect(() => {
    allowedTypes(selectedParent?.[EntryReference.entry]).then((types) => {
      if (types.length > 0)
        typeAtoms.mutator(types[0]);
    });
  }, [selectedParent]);
  useEffect(() => {
    copyFromAtoms.mutator.replace(void 0);
  }, [selectedType]);
  async function handleCreate(e) {
    e.preventDefault();
    const { title, type: selected } = form.data();
    if (!selected || !title)
      return;
    setIsCreating(true);
    const path = slugify(title);
    const entryId = createId();
    const data = {
      workspace,
      root: root.name,
      locale: locale ?? null,
      path,
      phase: config.enableDrafts ? EntryPhase.Draft : EntryPhase.Published
    };
    const parentId2 = form.data().parent?.[EntryReference.entry];
    const parent = await graph.preferPublished.maybeGet(
      Entry({ entryId: parentId2 }).select(parentData)
    );
    const parentPaths = parent ? parent.parentPaths.concat(parent.path) : [];
    const filePath = entryFilepath(config, data, parentPaths);
    const childrenDir = entryChildrenDir(config, data, parentPaths);
    const parentDir = dirname(filePath);
    const entryType = config.schema[selected];
    const url = entryUrl(entryType, { ...data, parentPaths });
    const copyFrom = form.data().copyFrom?.[EntryReference.entry];
    const entryData = copyFrom ? await graph.preferPublished.maybeGet(
      Entry({ entryId: copyFrom }).select(Entry.data)
    ) : {};
    const entry2 = await createEntryRow(config, {
      entryId,
      ...data,
      filePath,
      type: selected,
      path,
      title,
      url,
      index: generateKeyBetween(null, parent?.childrenIndex || null),
      parent: parent?.id ?? null,
      seeded: null,
      level: parent ? parent.level + 1 : 0,
      parentDir,
      childrenDir,
      i18nId: root.i18n ? createId() : entryId,
      modifiedAt: Date.now(),
      active: true,
      main: false,
      data: { ...entryData, title, path },
      searchableText: ""
    });
    return mutate([
      {
        type: MutationType.Create,
        entryId: entry2.entryId,
        entry: entry2,
        file: entryFileName(config, data, parentPaths)
      }
    ]).then(() => {
      setIsCreating(false);
      navigate(nav.entry({ entryId: entry2.i18nId }));
      if (parent)
        updateEntries([parent.i18nId]);
    });
  }
  return /* @__PURE__ */ jsxs(
    "form",
    {
      onSubmit: handleCreate,
      className: styles.form({ loading: isCreating }),
      children: [
        isCreating && /* @__PURE__ */ jsx(Loader, { absolute: true }),
        /* @__PURE__ */ jsx(InputForm, { border: false, form }),
        /* @__PURE__ */ jsxs("div", { className: styles.root.footer(), children: [
          /* @__PURE__ */ jsx(Link, { href: pathname, className: styles.root.footer.link(), children: "Cancel" }),
          /* @__PURE__ */ jsx(Button, { children: "Create" })
        ] })
      ]
    }
  );
}
function NewEntry({ parentId }) {
  const navigate = useNavigate();
  const { pathname } = useLocation();
  function handleClose() {
    navigate(pathname);
  }
  return /* @__PURE__ */ jsx(Modal, { open: true, onClose: handleClose, className: styles.root(), children: /* @__PURE__ */ jsx(Suspense, { fallback: /* @__PURE__ */ jsx(Loader, {}), children: /* @__PURE__ */ jsx(NewEntryForm, { parentId }) }) });
}
export {
  NewEntry
};
