import "../../chunks/chunk-U5RRZUYZ.js";

// src/field/link/LinkField.ts
import { Reference } from "alinea/core/Reference";
import { ListField } from "alinea/core/field/ListField";
import { UnionField } from "alinea/core/field/UnionField";
import { ListRow } from "alinea/core/shape/ListShape";
import { entries, fromEntries } from "alinea/core/util/Objects";
var LinkField = class extends UnionField {
};
function createLink(label, options) {
  const pickers = entries(options.pickers);
  const schema = fromEntries(
    pickers.filter(([type, picker]) => picker.fields).map(([type, picker]) => [type, picker.fields])
  );
  const shapes = fromEntries(
    pickers.map(([type, picker]) => [type, picker.shape])
  );
  return new LinkField(schema, shapes, {
    options: { label, ...options },
    async postProcess(value, loader) {
      const type = value[Reference.type];
      const picker = options.pickers[type];
      if (!picker)
        return;
      if (picker.postProcess)
        await picker.postProcess(value, loader);
    },
    view: "alinea/field/link/LinkField.view#SingleLinkInput"
  });
}
var LinksField = class extends ListField {
};
function createLinks(label, options) {
  const pickers = entries(options.pickers);
  const schema = fromEntries(
    pickers.filter(([type, picker]) => picker.fields).map(([type, picker]) => [type, picker.fields])
  );
  const shapes = fromEntries(
    pickers.map(([type, picker]) => [type, picker.shape])
  );
  return new LinksField(schema, shapes, {
    options: { label, ...options },
    async postProcess(rows, loader) {
      const tasks = [];
      for (const row of rows) {
        const type = row[ListRow.type];
        const picker = options.pickers[type];
        if (!picker)
          return;
        if (picker.postProcess)
          tasks.push(picker.postProcess(row, loader));
      }
      await Promise.all(tasks);
    },
    view: "alinea/field/link/LinkField.view#MultipleLinksInput"
  });
}
export {
  LinkField,
  LinksField,
  createLink,
  createLinks
};
